package com.smppw.analysis.application.dto.info;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.application.dto.BaseReq;
import com.smppw.analysis.domain.dto.performance.IndicatorParams;
import com.smppw.common.pojo.enums.*;
import com.smppw.common.pojo.enums.strategy.Strategy;
import com.smppw.constants.Consts;
import lombok.Setter;

import java.util.Date;

/**
 * @author wangzaijun
 * @date 2023/8/9 11:47
 * @description 头部成立以来指标实时计算接口请求参数
 */
@Setter
public class HeadIndicatorReq extends BaseReq<IndicatorParams> {
    /**
     * 标的id
     */
    private String secId;
    /**
     * 净值类型
     */
    private NavType navType;
    /**
     * 净值频率
     */
    private Frequency frequency;
    /**
     * 募集类型
     */
    private RaiseType raiseType;
    /**
     * 核心策略
     */
    private String strategy;

    public String getSecId() {
        return secId;
    }

    public NavType getNavType() {
        return StrUtil.isBlankIfStr(this.navType) ? NavType.CumulativeNav : this.navType;
    }

    public Frequency getFrequency() {
        return StrUtil.isBlankIfStr(this.frequency) ? Frequency.Default : this.frequency;
    }

    public RaiseType getRaiseType() {
        return StrUtil.isBlankIfStr(this.raiseType) ? RaiseType.Private : this.raiseType;
    }

    public String getStrategy() {
        return StrUtil.isBlank(this.strategy) ? Strategy.All.getStrategyOriginName() : this.strategy;
    }

    public IndicatorParams convert() {
        IndicatorParams params = new IndicatorParams();
        params.setSecIds(CollectionUtil.newArrayList());
        params.setRefIds(CollectionUtil.newArrayList(this.getSecId()));
        params.setBenchmarkId(Consts.BENCHMARK);
        params.setNavType(this.getNavType());
        params.setRaiseType(this.getRaiseType());
        params.setStrategy(this.getStrategy());
        params.setCalcExtraRet(false);
        params.setRiskOfFreeId(Consts.RISK_OF_FREE);
        params.setTimeRange(TimeRange.FromSetup);
        params.setFrequency(this.getFrequency());
        params.setStartDate(Consts.START_DATE);
        params.setEndDate(DateUtil.formatDate(new Date()));
        params.setIndicators(ListUtil.of(Indicator.AnnualReturn, Indicator.AnnualStdDev, Indicator.MaxDrawdown, Indicator.SharpeRatio));
        params.setExtraIndicators(ListUtil.of(Indicator.IntervalReturn));
        return params;
    }
}
