package com.smppw.analysis.application.service.info;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import com.smppw.analysis.application.dto.info.*;
import com.smppw.analysis.domain.dataobject.FundSimilarDo;
import com.smppw.analysis.domain.dto.info.FundSimilarParams;
import com.smppw.analysis.domain.dto.info.FundSimilarVO;
import com.smppw.analysis.domain.dto.performance.IndicatorParams;
import com.smppw.analysis.domain.manager.performance.Performance;
import com.smppw.analysis.domain.manager.performance.PerformanceConstants;
import com.smppw.analysis.domain.manager.performance.PerformanceFactory;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.common.pojo.enums.Indicator;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class FundInfoService {
    private final PerformanceFactory factory;
    private final BaseInfoService baseInfoService;

    public FundInfoService(PerformanceFactory factory, BaseInfoService baseInfoService) {
        this.factory = factory;
        this.baseInfoService = baseInfoService;
    }

    public PrivatelyFundHeadInfoVO headInfo(HeadInfoReq params) {
        return new PrivatelyFundHeadInfoVO();
    }

    public CommonInfoVO commonInfo(CommonInfoReq params) {
//        CommonInfoVO res = new CommonInfoVO();
//        IStrategy strategy = StrategyHandleUtils.getStrategy(params.getStrategy());
//        String secId = params.getSecId();
//        Byte raiseType = null;
//        List<Frequency> result = CollectionUtil.newArrayList();
//        String fundType = this.baseInfoService.getFundType(secId);
//        List<IStrategy> originStrategy = CollectionUtil.newArrayList();
//        // 获取频率
//        CmBigDataFundInfoDo fundHeadInfo = fundHeadInfoBaseService.getFundHeadInfo(secId);
//        if (fundHeadInfo == null) {
//            throw new APIException("");
//        }
//        raiseType = fundHeadInfo.getRaiseType();
//        Frequency navFrequency = fundInformationDao.getNavFrequency(secId);
//        result.add(navFrequency);
//        // 将频率转换一道 取最小频率
//        List<Frequency> frequencies = fundInformationDao.toFrequencyList(result);
//        if (raiseType == 2) {
//            frequencies.removeIf(p -> p.getId() == Frequency.Default.getId());
//        }
//        List<ValueLabelVO> frequencyValueLabel = trans2ValueLabel(frequencies);
//        res.setFrequency(frequencyValueLabel);
//        // 净值类型
//        List<ValueLabelVO> navType = Arrays.stream(NavType.values())
//                .filter(p -> !NavType.All.equals(p) && !NavType.UnitAndCumulativeNav.equals(p))
//                .map(p -> new ValueLabelVO(p.name(), p.getDesc())).collect(Collectors.toList());
//        res.setNavType(navType);
//        // 策略曲线
//        RaiseType realRaiseType = RaiseType.getRaiseType(raiseType);
//        handleTimeRange(strategy, res, secId, companyOrManager, realRaiseType);
//        // 获取常用指数
////        List<ValueLabelVO> benchmarkList = userCommonIndexService.getByUser(-1);
////        handleBenchmark(strategy, res, secId, fundType, originStrategy, companyOrManager, benchmarkList);
        return new CommonInfoVO();
    }

    @SuppressWarnings("unchecked")
    public HeadIndicatorVO headIndicator(HeadIndicatorReq req) {
        IndicatorParams params = req.convert();
        Performance<IndicatorParams, Map<String, Object>> instance = this.factory.getInstance(PerformanceConstants.INDICATOR);
        Map<String, Object> dataset = instance.execute(params);
        Map<String, Object> indicatorMap = MapUtil.get(dataset, req.getSecId(), Map.class);
        HeadIndicatorVO vo = new HeadIndicatorVO();
        vo.setAnnualReturn(MapUtil.getStr(indicatorMap, Indicator.AnnualReturn.name()));
        vo.setAnnualStdDev(MapUtil.getStr(indicatorMap, Indicator.AnnualStdDev.name()));
        vo.setMaxDrawdown(MapUtil.getStr(indicatorMap, Indicator.MaxDrawdown.name()));
        vo.setSharpeRatio(MapUtil.getStr(indicatorMap, Indicator.SharpeRatio.name()));
        return vo;
    }

    public List<FundSimilarVO> getFundSimilarList(FundSimilarReq req) {
        List<String> refIds = ListUtil.toLinkedList(req.getRefId());
        FundSimilarParams params = req.convert();
        List<FundSimilarDo> dataList = this.baseInfoService.getFundSimilarList(params);
        List<String> fundIds = dataList.stream().map(FundSimilarDo::getFundId).collect(Collectors.toList());
        if (CollUtil.isNotEmpty(fundIds)) {
            refIds.addAll(fundIds);
        }
//        Map<String, List<IndicatorCalcPropertyDto>> multiSecRetListNew = this.baseIndicatorServiceV2.getMultiSecRetListNew(
//                secIds, ListUtil.empty(), Frequency.Monthly, Frequency.Daily, startDate, endDate, false, Consts.BENCHMARK,
//                params.getRaiseType(), Strategy.All, Visibility.Visible, NavType.CumulativeNav);
//        List<Map<String, Object>> initRetList = NavUtil.handleNextStepNew(secIds, ListUtil.empty(), multiSecRetListNew, false);
//        Map<String, Map<String, Double>> retMap = RateConvertorUtils.transfer2TwoDimensionMap(initRetList, "fund_id", "end_date", "ret_range");
//        Double[][] corArr = CalcUtils.getCorrelationMatrix(retMap, ListUtil.toList(params.getRefId()), secIds);
//        for (FundSimilarDo map : dataList) {
//            FundSimilarVO similarVO = BeanUtil.copyProperties(map, FundSimilarVO.class);
//            int i = fundIds.indexOf(map.getFundId()) + 1; // 因为第一个基金是当前基金，所以需要 + 1 来取该基金与当前基金的相关性
//            similarVO.setSimilar(corArr[0][i]);
//            similarVO.setStrategy(StrategyHandleUtils.getStrategy(map.getStrategy()).getStrategyNameDesc());
//            similarVO.setSubStrategy(this.getStrategyName(map.getSubStrategy()));
//            similarVO.setThirdStrategy(this.getStrategyName(map.getThirdStrategy()));
//            resultList.add(similarVO);
//        }
//        resultList.stream().filter(e -> e.getSimilar() != null && e.getSimilar() >= Double.parseDouble(params.getThreshold())).collect(Collectors.toList());
        return null;
    }
}
