package com.smppw.analysis.application.service.performance;

import com.smppw.analysis.application.dto.performance.*;
import com.smppw.analysis.domain.dto.performance.*;
import com.smppw.analysis.domain.manager.performance.Performance;
import com.smppw.analysis.domain.manager.performance.PerformanceFactory;
import org.springframework.stereotype.Service;

import java.util.Map;

import static com.smppw.analysis.domain.manager.performance.PerformanceConstants.*;

@Service
public class FundPerformanceService {
    private final PerformanceFactory factory;

    public FundPerformanceService(PerformanceFactory factory) {
        this.factory = factory;
    }

    public Map<String, Object> calcIndicator(IndicatorReq req) {
        IndicatorParams params = req.convert();
        Performance<IndicatorParams, Map<String, Object>> instance = this.factory.getInstance(INDICATOR);
        return instance.execute(params);
    }

    public Map<String, Object> trend(TrendReq req) {
        TrendParams params = req.convert();
        Performance<TrendParams, Map<String, Object>> instance = this.factory.getInstance(TREND);
        return instance.execute(params);
    }

    public Map<String, Object> dynamicDown(DrawdownTrendReq req) {
        TrendParams params = req.convert();
        Performance<TrendParams, Map<String, Object>> instance = this.factory.getInstance(TREND);
        return instance.execute(params);
    }

    public Map<String, Object> cor(CorrelationReq req) {
        CorrelationParams params = req.convert();
        Performance<CorrelationParams, Map<String, Object>> instance = this.factory.getInstance(CORRELATION);
        return instance.execute(params);
    }

    public WinVO win(WinReq req) {
        WinParams params = req.convert();
        Performance<WinParams, WinVO> instance = this.factory.getInstance(WIN);
        return instance.execute(params);
    }

    public Map<String, Object> revenue(RevenueReq req) {
        RevenueParams params = req.convert();
        Performance<RevenueParams, Map<String, Object>> instance = this.factory.getInstance(CORRELATION);
        return instance.execute(params);
    }

    public Map<String, Object> imfTrend(ImfTrendReq req) {
        TrendParams params = req.convert();
        Performance<TrendParams, Map<String, Object>> instance = this.factory.getInstance(IMF);
        return instance.execute(params);
    }

    public Map<String, Object> interval(IntervalReq req) {
        IntervalParams params = req.convert();
        Performance<IntervalParams, Map<String, Object>> instance = this.factory.getInstance(INTERVAL);
        return instance.execute(params);
    }

    public Map<String, Object> rolling(RollingReq req) {
        RollingParams params = req.convert();
        Performance<RollingParams, Map<String, Object>> instance = this.factory.getInstance(ROLLING);
        return instance.execute(params);
    }

    public Map<String, Object> rank(RankReq req) {
        RankParams params = req.convert();
        Performance<RankParams, Map<String, Object>> instance = this.factory.getInstance(RANK);
        return instance.execute(params);
    }
}
