package com.smppw.analysis.application.service.position;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import com.smppw.analysis.application.dto.position.MarginalRiskContributionReq;
import com.smppw.analysis.domain.dto.position.RefMarketValueRatio;
import com.smppw.analysis.domain.dto.position.future.MarginalRiskContribution;
import com.smppw.analysis.domain.dto.position.future.MarginalRiskContributionParams;
import com.smppw.analysis.domain.dto.position.future.MarginalRiskContributionVO;
import com.smppw.analysis.domain.manager.position.BizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerConstants;
import com.smppw.analysis.domain.manager.position.BizHandlerFactory;
import com.smppw.common.pojo.FundFuturesOption;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/6 14:41
 * @description 基金持仓分析-期货tab
 */
@Component
public class FuturePositionAnalysis {
    private final BizHandlerFactory factor;

    public FuturePositionAnalysis(BizHandlerFactory factor) {
        this.factor = factor;
    }

    public Map<String, Object> riskCont(MarginalRiskContributionReq req) {
        MarginalRiskContributionParams params = req.convert();
        BizHandler<MarginalRiskContributionParams, List<MarginalRiskContributionVO>> bizHandler = this.factor.getBizHandlerInstance(BizHandlerConstants.MARGINAL_RISK_CONTRIBUTION);
        List<MarginalRiskContributionVO> dataList = bizHandler.bizHandle(params);
        if (CollUtil.isEmpty(dataList)) {
            return MapUtil.empty();
        }
        Map<String, String> mapping = MapUtil.newHashMap(true);
        mapping.put(FundFuturesOption.Bond.getNameEnDesc(), FundFuturesOption.Bond.getNameDesc());
        mapping.put(FundFuturesOption.Product.getNameEnDesc(), FundFuturesOption.Product.getNameDesc());
        mapping.put(FundFuturesOption.StockIndex.getNameEnDesc(), FundFuturesOption.StockIndex.getNameDesc());
        Map<String, Object> dataset = MapUtil.newHashMap();
        mapping.forEach((k, v) -> {
            List<Map<String, Object>> tempList = ListUtil.list(true);
            for (MarginalRiskContributionVO vo : dataList) {
                List<MarginalRiskContribution> collect = vo.getRiskContList().stream().filter(e -> k.equals(e.getAsset().getValue())).collect(Collectors.toList());
                Map<String, Object> data = MapUtil.newHashMap();
                data.put("date", vo.getDate());
                data.put("ratio", collect.stream().map(RefMarketValueRatio::getRatio).reduce(BigDecimal::add).orElse(BigDecimal.ZERO));
                data.put("riskCont", collect.stream().map(MarginalRiskContribution::getRiskCont).reduce(BigDecimal::add).orElse(BigDecimal.ZERO));
                tempList.add(data);
            }
            dataset.put(k, tempList);
        });
        Map<String, List<Map<String, Object>>> tableObj = MapUtil.newHashMap(true);
        for (MarginalRiskContributionVO vo : dataList) {
            List<Map<String, Object>> tempList = vo.getRiskContList().stream().map(e -> {
                Map<String, Object> data = MapUtil.newHashMap(true);
                String label = e.getRef().getLabel();
                if (label.contains("票面利率") && label.contains("名义") && label.contains("国债")) {
                    label = label.split("名义")[1];
                }
                data.put("name", label);
                data.put("ratio", e.getRatio());
                data.put("riskCont", e.getRiskCont());
                return data;
            }).collect(Collectors.toList());
            tempList = tempList.stream().limit(10).collect(Collectors.toList());
            tableObj.put(vo.getDate(), tempList);
        }
        return MapUtil.<String, Object>builder().put("dataset", dataset)
                .put("productNameMapping", mapping).put("tableObj", tableObj).build();
    }
}
