package com.smppw.analysis.client;

import com.smppw.analysis.application.service.style.StyleService;
import com.smppw.analysis.domain.dto.style.*;
import com.smppw.common.pojo.ResultVo;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

/**
 * @author wangzaijun
 * @date 2023/8/7 15:52
 * @description 风格评测api
 */
@RestController
@RequestMapping("/v1/api/style")
public class FundStyleApi {
    private final StyleService service;

    public FundStyleApi(StyleService service) {
        this.service = service;
    }

    /**
     * 股票业绩归因（五因子归因分析）-前端组件：风格评测-股票业绩归因组件
     *
     * @param params 请求参数
     * @return /
     */
    @GetMapping("stock-attribution")
    public ResultVo<Map<String, Object>> stockAttribution(StockAttributionParams params) {
        return ResultVo.ok(this.service.stockAttribution(params));
    }

    /**
     * 债券业绩归因-前端组件：风格评测-债券业绩归因
     *
     * @param params 请求参数
     * @return /
     */
    @GetMapping("bond-attribution")
    public ResultVo<Map<String, Object>> bondAttribution(BondAttributionParams params) {
        return ResultVo.ok(this.service.bondAttribution(params));
    }

    /**
     * 相对价值业绩归因-前端组件：风格评测-相对价值业绩归因
     *
     * @param params 请求参数
     * @return /
     */
    @GetMapping("relative-attribution")
    public ResultVo<Map<String, Object>> relativeAttribution(RelativeAttributionParams params) {
        return ResultVo.ok(this.service.relativeAttribution(params));
    }

    /**
     * 成长价值风格-前端组件：风格评测-成长价值风格
     *
     * @param params 请求参数
     * @return /
     */
    @GetMapping("growth-value")
    public ResultVo<FundStyleVO> growthValue(GrowthValueParams params) {
        return ResultVo.ok(this.service.growthValue(params));
    }

    /**
     * 行业配置-前端组件：风格评测-行业配置
     *
     * @param params 请求参数
     * @return /
     */
    @GetMapping("industry-value")
    public ResultVo<FundStyleVO> industryValue(IndustryValueParams params) {
        return ResultVo.ok(this.service.industryValue(params));
    }

    /**
     * 期货风格-前端组件：风格评测-期货风格
     *
     * @param params 请求参数
     * @return /
     */
    @GetMapping("future-factor")
    public ResultVo<Map<String, Object>> futureFactor(FutureFactorParams params) {
        return ResultVo.ok(this.service.futureFactor(params));
    }

    /**
     * 自定义风格-前端组件：风格评测-自定义风格
     *
     * @param params 请求参数
     * @return /
     */
    @GetMapping("custom-rbsa")
    public ResultVo<CustomRbsaVO> customRbsa(CustomRbsaParams params) {
        return ResultVo.ok(this.service.customRbsa(params));
    }

    /**
     * Barra风格概览
     */
    @GetMapping("/barra-factor-overview")
    public ResultVo<Map<String, Object>> barraOverview(BarraStyleParams params) {
        return ResultVo.ok(this.service.barraOverview(params));
    }

    /**
     * Barra 风格归因
     */
    @GetMapping("/barra-factor-risk-profit")
    public ResultVo<Map<String, Object>> barraRiskProfit(BarraStyleParams params) {
        return ResultVo.ok((this.service.barraRiskProfit(params)));
    }

    /**
     * 选股择时能力-前端组件：风格评测-选股择时能力
     *
     * @param params 请求参数
     * @return /
     */
    @GetMapping("selection-timing")
    public ResultVo<Map<String, Object>> selectionTiming(SelectionTimingParams params) {
        return ResultVo.ok(this.service.selectionTiming(params));
    }

    /**
     * 融智 风格总览-前端组件：风格评测-融智 风格总览
     *
     * @param params 请求参数
     * @return /
     */
    @GetMapping("ppw")
    public ResultVo<Map<String, Object>> rzStyle(RzStyleParams params) {
        return ResultVo.ok(this.service.rzStyle(params));
    }
}
