package com.smppw.analysis.domain.dao;

import cn.hutool.core.map.MapUtil;
import com.smppw.analysis.domain.dataobject.FundFrequencyDo;
import com.smppw.analysis.domain.dataobject.FundInformationDo;
import com.smppw.analysis.domain.dataobject.FundSimilarDo;
import com.smppw.analysis.domain.dataobject.MonetaryFundProfitDO;
import com.smppw.analysis.domain.mapper.FundFrequencyDoMapper;
import com.smppw.analysis.domain.mapper.FundInformationDoMapper;
import com.smppw.analysis.domain.mapper.MonetaryFundProfitMapper;
import com.smppw.common.pojo.enums.Frequency;
import com.smppw.common.pojo.enums.Indicator;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class FundInformationDao {
    private final FundFrequencyDoMapper fundFrequencyDoMapper;
    private final FundInformationDoMapper fundInformationDoMapper;
    private final MonetaryFundProfitMapper monetaryFundProfitMapper;

    public FundInformationDao(FundFrequencyDoMapper fundFrequencyDoMapper,
                              FundInformationDoMapper fundInformationDoMapper,
                              MonetaryFundProfitMapper monetaryFundProfitMapper) {
        this.fundFrequencyDoMapper = fundFrequencyDoMapper;
        this.fundInformationDoMapper = fundInformationDoMapper;
        this.monetaryFundProfitMapper = monetaryFundProfitMapper;
    }

    public String getNavFrequencyByFundId(String secId) {
        return this.fundFrequencyDoMapper.getNavFrequencyByFundId(secId);
    }

    public Map<String, Frequency> getNavFrequency(List<String> secIdList) {
        Map<String, Frequency> secFrequencyMap = new HashMap<>();
        List<FundFrequencyDo> fundFrequencyDos = fundFrequencyDoMapper.getNavFrequencyByFundIdList(secIdList);
        for (FundFrequencyDo fundFrequencyDo : fundFrequencyDos) {
            String frequencyStr = fundFrequencyDo.getNavFrequency();
            Frequency frequency = Frequency.Monthly;
            if ("天".equals(frequencyStr)) {
                frequency = Frequency.Daily;
            } else if ("周".equals(frequencyStr)) {
                frequency = Frequency.Weekly;
            }
            secFrequencyMap.put(fundFrequencyDo.getFundId(), frequency);
        }
        return secFrequencyMap;
    }

    public Map<String, String> getMarketFundIdNameMap(List<String> fundIdList) {
        List<FundInformationDo> fundInformationDos = fundInformationDoMapper.listFundInfo(fundIdList);
        Map<String, String> fundIdNameMap = new HashMap<>();
        for (FundInformationDo fundInformationDo : fundInformationDos) {
            fundIdNameMap.put(fundInformationDo.getFundId(), fundInformationDo.getFundShortName());
        }
        return fundIdNameMap;
    }

    public List<MonetaryFundProfitDO> queryMonetaryFund(String fundId) {
        return this.monetaryFundProfitMapper.queryByFundId(fundId);
    }

    public List<Map<String, Object>> getFundRank(String rankDate, String fundId, List<String> indexIds, Indicator indicator) {
        Map<String, Object> params = MapUtil.newHashMap(8);
        params.put("fundId", fundId);
        params.put("rankDate", rankDate);
        params.put("indexIds", String.join("_", indexIds));
        params.put("indicator", indicator.getId());
        return this.fundInformationDoMapper.getFundRankByCall(params);
    }

    public List<FundSimilarDo> getSameCompanyFundIds(Map<String, Object> params) {
        return null;
    }

    public List<FundSimilarDo> getSameStrategyFundIds(Map<String, Object> params) {
        return null;
    }
}
