package com.smppw.analysis.domain.dao;

import com.smppw.analysis.domain.dataobject.MarketIndexesDo;
import com.smppw.analysis.domain.mapper.MarketIndexesDoMapper;
import com.smppw.common.pojo.dto.NavDto;
import com.smppw.constants.DateConst;
import com.smppw.utils.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Repository
public class MarketIndexNavDao {

    @Autowired
    private MarketIndexesDoMapper marketIndexesDoMapper;

    public Map<String, List<NavDto>> getNav(List<String> indexIdList, String startDate, String endDate, boolean needUpdateAndCreateTime) {
        List<MarketIndexesDo> navDoList = marketIndexesDoMapper.listNavByTimeInterval(indexIdList, startDate, endDate, needUpdateAndCreateTime);
        Map<String, List<NavDto>> indexIdNavDtoListMap = navDoListToNavDtoMap(navDoList);
        return indexIdNavDtoListMap;
    }

    public List<MarketIndexesDo> getIndexNavByPriceDate(String indexId, List<String> priceDateList) {
        return marketIndexesDoMapper.selectIndexNavByPriceDate(indexId, priceDateList);
    }

    /**
     * 领域对象集合转为数据传输对象集合
     *
     * @param navDoList 领域对象集合
     * @return 数据传输对象集合  key-基金ID  value-数据传输对象集合
     */
    private Map<String, List<NavDto>> navDoListToNavDtoMap(List<MarketIndexesDo> navDoList) {
        Map<String, List<NavDto>> fundIdNavDtoListMap = new HashMap<>();

        if (navDoList == null) {
            return fundIdNavDtoListMap;
        }

        for (MarketIndexesDo navDo : navDoList) {
            NavDto navDto = new NavDto();
            navDto.setFundId(navDo.getIndexId());
            if (navDo.getPriceDate() == null || navDo.getClose() == null) {
                continue;
            }
            navDto.setPriceDate(DateUtil.DateToString(navDo.getPriceDate(), DateConst.YYYY_MM_DD));
            navDto.setOriginalNav(navDo.getClose());
            navDto.setCumulativeNav(navDo.getClose());
            navDto.setCumulativeNavWithdrawal(navDo.getClose());
            if (navDo.getUpdatetime() != null) {
                navDto.setUpdatetime(DateUtil.DateToString(navDo.getUpdatetime(), DateConst.YYYY_MM_DD_HH_MM_SS));
            }
            if (navDo.getCreatetime() != null) {
                navDto.setCreatetime(DateUtil.DateToString(navDo.getCreatetime(), DateConst.YYYY_MM_DD));
            }

            if (fundIdNavDtoListMap.containsKey(navDo.getIndexId())) {
                List<NavDto> navDtoList = fundIdNavDtoListMap.get(navDo.getIndexId());
                navDtoList.add(navDto);
            } else {
                List<NavDto> navDtoList = new ArrayList<>();
                navDtoList.add(navDto);
                fundIdNavDtoListMap.put(navDo.getIndexId(), navDtoList);
            }
        }
        return fundIdNavDtoListMap;
    }

}
