package com.smppw.analysis.domain.dao;


import com.smppw.analysis.domain.dataobject.*;
import com.smppw.analysis.domain.mapper.BaseUnderlyingMapper;
import com.smppw.analysis.domain.mapper.IndexesTradeDateDoMapper;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author wangzaijun
 * @date 2023/7/24 11:39
 * @description 基金持仓分析需要的所有mapper接口
 */
@Service
public class PositionAnalysisDao {
    private final BaseUnderlyingMapper baseUnderlyingMapper;
    private final IndexesTradeDateDoMapper indexesTradeDateDoMapper;

    public PositionAnalysisDao(BaseUnderlyingMapper baseUnderlyingMapper,
                               IndexesTradeDateDoMapper indexesTradeDateDoMapper) {
        this.baseUnderlyingMapper = baseUnderlyingMapper;
        this.indexesTradeDateDoMapper = indexesTradeDateDoMapper;
    }

    /**
     * 获取最近的交日期，往前找
     *
     * @param tradeDate /
     * @return /
     */
    public String getLatestTradeDate(String tradeDate) {
        return this.indexesTradeDateDoMapper.queryLatestTradeDate(tradeDate);
    }

    /**
     * 获取多个日期的流动性标准
     *
     * @param dateList /
     * @return /
     */
    public List<SecLiquidityInfoDO> listSecLiquidityInfo(List<String> dateList) {
        return this.baseUnderlyingMapper.querySecLiquidityInfo(dateList);
    }

    /**
     * 多个日期的5日平均换手率
     *
     * @param dateList /
     * @return /
     */
    public List<SecTurnoverInfoDO> listSecTurnoverInfo(List<String> dateList) {
        return this.baseUnderlyingMapper.querySecTurnoverInfo(dateList);
    }

    /**
     * 获取所有股票对应行业
     *
     * @return /
     */
    public List<SwSecIndustryInfoDO> listSecIndustryInfo() {
        return this.baseUnderlyingMapper.querySecIndustryInfo();
    }

    /**
     * 获取所有股票对应风格
     *
     * @param dateList 风格截止日集合
     * @return /
     */
    public List<SecStyleInfoDO> listSecStyleInfo(List<String> dateList) {
        return this.baseUnderlyingMapper.querySecStyleInfo(dateList);
    }

    /**
     * 获取当前日期之后最近的权重数据
     *
     * @param indexId 基准id
     * @param endDate 当前日期
     * @return /
     */
    public List<IndexSecWeightInfoDO> listIndexSecWeight(String indexId, String endDate) {
        return this.baseUnderlyingMapper.queryIndexSecWeight(indexId, endDate);
    }

    /**
     * 获取股票行情
     *
     * @param endDate 当前时间
     * @return /
     */
    public List<SecClosePriceDO> listSecClosePrice(String startDate, String endDate) {
        return this.baseUnderlyingMapper.listSecClosePrice(startDate, endDate);
    }

    public List<SecClosePriceDO> listSecClosePrice(List<String> dateList) {
        return this.baseUnderlyingMapper.listDateSecClosePrice(dateList);
    }

    /**
     * 获取科创版股票行情
     *
     * @param endDate 当前时间
     * @return /
     */
    public List<SecClosePriceDO> listSciTechSecClosePrice(String startDate, String endDate) {
        return this.baseUnderlyingMapper.listSciTechSecClosePrice(startDate, endDate);
    }

    public List<SecClosePriceDO> listSciTechSecClosePrice(List<String> dateList) {
        return this.baseUnderlyingMapper.listDateSciTechSecClosePrice(dateList);
    }
}
