package com.smppw.analysis.domain.dao;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dataobject.FundPositionBaseInfoDO;
import com.smppw.analysis.domain.dataobject.FundPositionDetailDO;
import com.smppw.analysis.domain.dataobject.PubliclyFundHolderInfoDO;
import com.smppw.analysis.domain.dataobject.PubliclyFundStockChangeDO;
import com.smppw.analysis.domain.mapper.PubliclyFundExtInfoMapper;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

@Component
public class PubliclyFundPositionDao {
    /**
     * 公募基金估值表合法报告日期，去除 03和09
     */
    private static final String[] PUBLICLY_VALID_DATES = new String[]{"-12-", "-06-"};
    private final PubliclyFundExtInfoMapper publiclyFundExtInfoMapper;

    public PubliclyFundPositionDao(PubliclyFundExtInfoMapper publiclyFundExtInfoMapper) {
        this.publiclyFundExtInfoMapper = publiclyFundExtInfoMapper;
    }

    public List<FundPositionBaseInfoDO> fundPositionBaseInfos(String fundId, String startDate, String endDate) {
        List<FundPositionBaseInfoDO> dataList = this.publiclyFundExtInfoMapper.queryPositionBaseInfos(fundId, startDate, endDate);
        if (CollUtil.isNotEmpty(dataList)) {
            dataList = dataList.stream().filter(e -> StrUtil.containsAny(DateUtil.formatDate(e.getReportDate()),
                    PUBLICLY_VALID_DATES)).collect(Collectors.toList());
        }
        return dataList;
    }

    public List<FundPositionDetailDO> positionDetailList(String fundId) {
        List<FundPositionDetailDO> dataList = this.publiclyFundExtInfoMapper.queryPositionDetails(fundId);
        if (CollUtil.isNotEmpty(dataList)) {
            dataList = dataList.stream().filter(e -> StrUtil.containsAny(DateUtil.formatDate(e.getValuationDate()),
                    PUBLICLY_VALID_DATES)).collect(Collectors.toList());
        }
        return dataList;
    }

    public List<PubliclyFundHolderInfoDO> mfHolderInfoList(String fundId, String startDate, String endDate) {
        return this.publiclyFundExtInfoMapper.mfHolderInfoList(fundId, startDate, endDate);
    }

    public List<PubliclyFundStockChangeDO> mfStockChangeList(String fundId, String startDate, String endDate) {
        return this.publiclyFundExtInfoMapper.mfStockChangeList(fundId, startDate, endDate);
    }
}
