package com.smppw.analysis.domain.dao;

import com.smppw.analysis.domain.dataobject.PublicNavDo;
import com.smppw.analysis.infrastructure.consts.NavConst;
import com.smppw.analysis.domain.mapper.PublicNavDoMapper;
import com.smppw.common.pojo.dto.NavDto;
import com.smppw.common.pojo.enums.NavType;
import com.smppw.constants.DateConst;
import com.smppw.utils.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Repository
public class PubliclyOfferedFundNavDao {

    @Autowired
    private PublicNavDoMapper publicNavDoMapper;

    public static Map<String, List<NavDto>> convertToMap(List<PublicNavDo> navDoList) {
        Map<String, List<NavDto>> fundIdNavDtoListMap = new HashMap<>();

        if (navDoList == null) {
            return fundIdNavDtoListMap;
        }

        for (PublicNavDo navDo : navDoList) {
            NavDto navDto = new NavDto();
            navDto.setFundId(navDo.getFundId());
            if (navDo.getPriceDate() == null) {
                continue;
            }
            navDto.setPriceDate(DateUtil.DateToString(navDo.getPriceDate(), DateConst.YYYY_MM_DD));
            navDto.setOriginalNav(navDo.getNav());
            navDto.setCumulativeNav(navDo.getCumulativeNav());
            navDto.setCumulativeNavWithdrawal(navDo.getCumulativeNavWithdrawal());
            if (navDo.getUpdatetime() != null) {
                navDto.setUpdatetime(DateUtil.DateToString(navDo.getUpdatetime(), DateConst.YYYY_MM_DD_HH_MM_SS));
            }
            if (navDo.getCreatetime() != null) {
                navDto.setCreatetime(DateUtil.DateToString(navDo.getCreatetime(), DateConst.YYYY_MM_DD));
            }

            if (fundIdNavDtoListMap.containsKey(navDo.getFundId())) {
                List<NavDto> navDtoList = fundIdNavDtoListMap.get(navDo.getFundId());
                navDtoList.add(navDto);
            } else {
                List<NavDto> navDtoList = new ArrayList<>();
                navDtoList.add(navDto);
                fundIdNavDtoListMap.put(navDo.getFundId(), navDtoList);
            }
        }
        return fundIdNavDtoListMap;
    }

    public Map<String, List<NavDto>> getNav(List<String> fundIdList, String startDate, String endDate, NavType navType, boolean needUpdateAndCreateTime) {
        String navFieldSql = "";
        List<PublicNavDo> navDoList = null;
        if (NavType.CumulativeNav == navType) {
            navFieldSql = NavConst.CUMULATIVE_NAV_SQL_FIELD;
        } else if (NavType.OriginalNav == navType) {
            navFieldSql = NavConst.NAV_SQL_FIELD;
        } else if (NavType.WithdrawalNav == navType) {
            navFieldSql = NavConst.CUMULATIVE_NAV_WITHDRAWAL_SQL_FIELD;
        } else if (NavType.All == navType) {
            navFieldSql = NavConst.CUMULATIVE_NAV_SQL_FIELD + "," + NavConst.NAV_SQL_FIELD + "," + NavConst.CUMULATIVE_NAV_WITHDRAWAL_SQL_FIELD;
        }
        navDoList = publicNavDoMapper.listNavByTimeInterval(fundIdList, startDate, endDate, navFieldSql, needUpdateAndCreateTime);
        Map<String, List<NavDto>> fundIdNavDtoListMap = convertToMap(navDoList);
        return fundIdNavDtoListMap;
    }

}
