package com.smppw.analysis.domain.dao;

import com.smppw.analysis.domain.dataobject.DepositNavDo;
import com.smppw.analysis.domain.mapper.DepositNavDoMapper;
import com.smppw.common.pojo.dto.NavDto;
import com.smppw.constants.DateConst;
import com.smppw.utils.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Repository
public class RiskOfFreeNavDao {
    @Autowired
    private DepositNavDoMapper depositNavMasterMapper;

    public Map<String, List<NavDto>> getNav(List<String> indexIdList, String startDate, String endDate, boolean needUpdateAndCreateTime) {
        List<DepositNavDo> navDoList = depositNavMasterMapper.listNavByTimeInterval(indexIdList, startDate, endDate, needUpdateAndCreateTime);
        return navDoListToNavDtoMap(navDoList);
    }

    /**
     * 领域对象集合转为数据传输对象集合
     *
     * @param navDoList 领域对象集合
     * @return 数据传输对象集合  key-基金ID  value-数据传输对象集合
     */
    private Map<String, List<NavDto>> navDoListToNavDtoMap(List<DepositNavDo> navDoList) {
        Map<String, List<NavDto>> fundIdNavDtoListMap = new HashMap<>();

        if (navDoList == null) {
            return fundIdNavDtoListMap;
        }

        for (DepositNavDo navDo : navDoList) {
            NavDto navDto = new NavDto();
            navDto.setFundId(navDo.getFundId());
            if (navDo.getPriceDate() == null) {
                continue;
            }
            navDto.setPriceDate(DateUtil.DateToString(navDo.getPriceDate(), DateConst.YYYY_MM_DD));
            navDto.setOriginalNav(navDo.getNav());
            navDto.setCumulativeNav(navDo.getNav());
            navDto.setCumulativeNavWithdrawal(navDo.getNav());
            if (navDo.getUpdatetime() != null) {
                navDto.setUpdatetime(DateUtil.DateToString(navDo.getUpdatetime(), DateConst.YYYY_MM_DD));
            }
            if (navDo.getCreatetime() != null) {
                navDto.setCreatetime(DateUtil.DateToString(navDo.getCreatetime(), DateConst.YYYY_MM_DD));
            }

            if (fundIdNavDtoListMap.containsKey(navDo.getFundId())) {
                List<NavDto> navDtoList = fundIdNavDtoListMap.get(navDo.getFundId());
                navDtoList.add(navDto);
            } else {
                List<NavDto> navDtoList = new ArrayList<>();
                navDtoList.add(navDto);
                fundIdNavDtoListMap.put(navDo.getFundId(), navDtoList);
            }
        }
        return fundIdNavDtoListMap;
    }
}
