package com.smppw.analysis.domain.dao;

import com.smppw.analysis.domain.dataobject.IndexesRzIndexDo;
import com.smppw.analysis.domain.mapper.IndexesRzIndexDoMapper;
import com.smppw.common.pojo.dto.NavDto;
import com.smppw.constants.DateConst;
import com.smppw.utils.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.*;

@Repository
public class RongzhiIndexNavDao {

    @Autowired
    private IndexesRzIndexDoMapper indexesRzIndexDoMapper;

    public Boolean isExist(String indexId) {
        Integer exist = this.indexesRzIndexDoMapper.isExist(indexId);
        return Objects.equals(1, exist);
    }

    public Map<String, List<NavDto>> getNav(List<String> indexIdList, String startDate, String endDate, boolean needUpdateAndCreateTime) {
        List<IndexesRzIndexDo> navDoList = indexesRzIndexDoMapper.listNavByTimeInterval(indexIdList, startDate, endDate, needUpdateAndCreateTime);
        return navDoListToNavDtoMap(navDoList);
    }

    /**
     * 领域对象集合转为数据传输对象集合
     *
     * @param navDoList 领域对象集合
     * @return 数据传输对象集合  key-基金ID  value-数据传输对象集合
     */
    private Map<String, List<NavDto>> navDoListToNavDtoMap(List<IndexesRzIndexDo> navDoList) {
        Map<String, List<NavDto>> fundIdNavDtoListMap = new HashMap<>();

        if (navDoList == null) {
            return fundIdNavDtoListMap;
        }

        for (IndexesRzIndexDo navDo : navDoList) {
            NavDto navDto = new NavDto();
            navDto.setFundId(navDo.getIndexId());
            if (navDo.getEndDate() == null) {
                continue;
            }
            navDto.setPriceDate(DateUtil.DateToString(navDo.getEndDate(), DateConst.YYYY_MM_DD));
            navDto.setOriginalNav(navDo.getIndexValue());
            navDto.setCumulativeNav(navDo.getIndexValue());
            navDto.setCumulativeNavWithdrawal(navDo.getIndexValue());
            if (navDo.getUpdatetime() != null) {
                navDto.setUpdatetime(DateUtil.DateToString(navDo.getUpdatetime(), DateConst.YYYY_MM_DD_HH_MM_SS));
            }
            if (navDo.getCreatetime() != null) {
                navDto.setCreatetime(DateUtil.DateToString(navDo.getCreatetime(), DateConst.YYYY_MM_DD));
            }

            if (fundIdNavDtoListMap.containsKey(navDo.getIndexId())) {
                List<NavDto> navDtoList = fundIdNavDtoListMap.get(navDo.getIndexId());
                navDtoList.add(navDto);
            } else {
                List<NavDto> navDtoList = new ArrayList<>();
                navDtoList.add(navDto);
                fundIdNavDtoListMap.put(navDo.getIndexId(), navDtoList);
            }
        }
        return fundIdNavDtoListMap;
    }

}
