package com.smppw.analysis.domain.dataobject;

import java.math.BigDecimal;
import java.util.Date;

/**
    * 基金基本信息表
    */
public class FundInformationDo {
    /**
    * 主键ID
    */
    private Integer id;

    /**
    * 父级产品ID
    */
    private String pFundId;

    /**
    * 基金id，'HF'开头(后加36进制编码格式,不足8位长度左补零） 例：HF00000001
    */
    private String fundId;

    /**
    * 基金中文全称
    */
    private String fundName;

    /**
    * 基金中文简称
    */
    private String fundShortName;

    /**
    * 基金形式：1-公司型，2-合伙型，3-契约型，-1-其他
    */
    private Integer fundStructure;

    /**
    * 基金类型:1-信托计
划，2-有限合伙，3-券商资管，4-公募专户，5-单账户，6-证券投资基金，7-海外基金，8-期货资管，9-保险资管、10-创业投
资基金、11-股权投资基金、12-银行理财、13-类固收信托 、 14-私募资产配置基金 15公募基金 -1其他投资基金
    */
    private Byte fundType;

    /**
    * （即将弃用）公募基金类型：1-混合型、2-债券型、3-定开债券、4-联接基金、5-货币型、6-债券指数、7-保本型、8-理财型、9-QDII、10-股票指数、11-QDII-指数、12-股票型、13-固定收益、14-分级杠杆、15-ETF-场内、16-QDII-ETF、17-债券创新-场内、18-封闭式
    */
    private Byte publicFundType;

    /**
    * (已废弃)公募基金类型-基础参数，见fund_parameters:1-股票型、2-混合型、3-债券型、4-货币型、5-商品型、6-市场中性型、7-FOF、8-海外型
    */
    private Byte pubFundType;

    /**
    * (已废弃)公募基金子类型-基础参数，见fund_parameters:1-主动型、2-被动型、3-偏股型、4-平衡型、5-偏债型、6-普通债券型、7-可转债型、8-指数型、9-商品型、10-市场中性型、11-货币型、12-理财型
    */
    private Byte pubSubFundType;

    /**
    * 券商资管产品特点
    */
    private String fundCharacteristic;

    /**
    * 基础货币,1-人民币，2-港币，3-美元，4-份，-1-其他
    */
    private Integer baseCurrency;

    /**
    * 成立日期
    */
    private Date inceptionDate;

    /**
    * 注册国家,1-中国大陆、2-香港、3-新加坡、4-开曼群岛、5-台湾、6-英属维尔京群岛BVI、-1-其他
    */
    private Integer domicile;

    /**
    * 指数id，以'IN'开头(后加36进制编码格式,不足8位长度左补零） 例：IN00000001
    */
    private String primaryBenchmarkId;

    /**
    * 业绩基准指数及其说明
    */
    private String secondaryBenchmark;

    /**
    * 封闭期，单位：月，-1：不确定，0：无封闭期
    */
    private Integer lockupPeriod;

    /**
    * 封闭期单位:1-天 2-月
    */
    private Byte lockupPeriodUnit;

    /**
    * 准封闭期
    */
    private String lockupPeriod2;

    /**
    * 锁定期，单位：月，-1：不确定，0：无锁定期
    */
    private Integer lockPeriod;

    /**
    * 锁定期单位:1-天 2-月
    */
    private Byte lockPeriodUnit;

    /**
    * 开放日
    */
    private String openDay;

    /**
    * 赎回日
    */
    private String redemptionDay;

    /**
    * 产品存续期，单位：月。-1，不确定，0，无固定期限，999999永续
    */
    private Integer duration;

    /**
    * 单位面值
    */
    private BigDecimal initialUnitValue;

    /**
    * 基金投资范围
    */
    private String investmentScope;

    /**
    * 投资目标
    */
    private String investmentObjective;

    /**
    * 基金资产配置比例
    */
    private String fundPortfolio;

    /**
    * 投资限制
    */
    private String investmentRestriction;

    /**
    * 投资理念
    */
    private String fundInvestmentPhilosophy;

    /**
    * 投资策略
    */
    private String fundStrategyDescription;

    /**
    * 投资决策流程
    */
    private String investmentProcess;

    /**
    * 投资顾问Id
    */
    private String advisorId;

    /**
    * 托管银行Id
    */
    private String custodianId;

    /**
    * 证券经纪人Id
    */
    private String brokerId;

    /**
    * 期货经纪人id
    */
    private String brokerFutureId;

    /**
    * 外包机构Id
    */
    private String liquidationAgencyId;

    /**
    * 基金管理公司Id
    */
    private String trustId;

    /**
    * 行政管理人Id
    */
    private String administratorId;

    /**
    * 法律顾问Id
    */
    private String legalCounselId;

    /**
    * 审计机构
    */
    private String auditorId;

    /**
    * 综合服务商id
    */
    private String generalServiceId;

    /**
    * 发行人id
    */
    private String issuerId;

    /**
    * 净值披露频率
    */
    private String navFrequency;

    /**
    * 产品业绩披露标识:1-A,2-B,3-C
    */
    private Byte performanceDisclosureMark;

    /**
    * 产品业绩披露方式:1-净值,2-万份收益
    */
    private Byte performanceDisclosureType;

    /**
    * 基金在前台是否可见
    */
    private Integer isvisible;

    /**
    * 管理类型：1-顾问管理，2-受托管理，3-自我管理
    */
    private Integer managerType;

    /**
    * 成立时参与人户数
    */
    private Integer beginMemberCnt;

    /**
    * 1-暂行办法实施前成立的基金，2-暂行办法实施后的基金
    */
    private Byte registerPeriod;

    /**
    * 中基协最后更新时间
    */
    private String zjxLastInfoUpdateTime;

    /**
    * 基金协会特别提示
    */
    private String specialTips;

    /**
    * 中基协页面地址
    */
    private String amacUrl;

    /**
    * 募集方式：1-私募，2-公募
    */
    private Byte raiseType;

    /**
    * 风险收益特征
    */
    private String riskReturnDesc;

    /**
    * 创建者Id,默认第一次创建者名称，创建后不变更
    */
    private Integer creatorid;

    /**
    * 创建时间,默认第一次创建的getdate()时间
    */
    private Date createtime;

    /**
    * 修改者Id;第一次创建时与Creator值相同，修改时与修改人值相同
    */
    private Integer updaterid;

    /**
    * 修改时间;第一次创建时与CreatTime值相同，修改时与修改时间相同
    */
    private Date updatetime;

    /**
    * 记录的有效性；1-有效；0-无效；
    */
    private Integer isvalid;

    private String registerNumber;

    /**
    * 是否分级：1-分级，0-不分级;
    */
    private Byte istiered;

    /**
    * 分级说明:0-空 1-优先份额 2-中间份额 3-劣后份额
    */
    private Integer istieredDesc;

    /**
    * 备案日期
    */
    private Date registerDate;

    /**
    * 是否参与排名，1-参与排名 0-不参与排名
    */
    private Integer isRanking;

    /**
    * 是否参与评级，1-参与评级 0-不参与评级
    */
    private Integer isRating;

    /**
    * 基金净值来源地址
    */
    private String navSource;

    /**
    * 合同文件名称
    */
    private String contractName;

    /**
    * 合同文件存储路径
    */
    private String contractPath;

    /**
    * 是否授权：1-是，0-否，-1-未确定(默认)
    */
    private Integer isAuthorized;

    /**
    * 是否调查中：1-是，0-否
    */
    private Integer isInResearch;

    /**
    * 展示控制(业务场景),结合d_fund_visible_ctrl_set设置前台是否可见、净值是否可见、净值是否可见(B端)
    */
    private Integer visibleCtrl;

    /**
    * 净值是否可见：1-是，0-否
    */
    private Integer isNavVisible;

    /**
    * 净值是否可见(B端)：1-是，0-否
    */
    private Integer isNavVisibleOrg;

    /**
    * 是否费前:1-是 0-否
    */
    private Byte isFeeBefore;

    /**
    * 是否扣除业绩报酬:1-是 0-否("是否费前"为"否"才会有值,否则为空)
    */
    private Byte isDeductReward;

    /**
    * 估值机构id
    */
    private String valuationInstitutionId;

    /**
    * 信托登记系统产品编码
    */
    private String trustRegisterNumber;

    /**
    * 是否代销产品: 1-是 , 0-否
    */
    private Byte isConsignment;

    /**
    * 是否qdii:1-是,0-否
    */
    private Byte isQdii;

    /**
    * 是否独家
    */
    private Byte isExclusive;

    /**
    * 是否独立展示(用于子基金)
    */
    private Byte isShowIndependent;

    /**
    * 申请登记日期(中信登)
    */
    private Date trustApplyRegisterDate;

    /**
    * 公示日期(中信登)
    */
    private Date trustPublicityDate;

    /**
    * 主要投向行业
    */
    private String trustMainIndustry;

    /**
    * 财产运用方式
    */
    private String trustPropertyApplication;

    /**
    * 信托功能：1-融资类 2-投资类 3-事务管理
    */
    private Byte trustFunction;

    /**
    * 当前是否备案展示：1-是 0-否 -1-不详
    */
    private Byte isAmacShow;

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public String getpFundId() {
        return pFundId;
    }

    public void setpFundId(String pFundId) {
        this.pFundId = pFundId;
    }

    public String getFundId() {
        return fundId;
    }

    public void setFundId(String fundId) {
        this.fundId = fundId;
    }

    public String getFundName() {
        return fundName;
    }

    public void setFundName(String fundName) {
        this.fundName = fundName;
    }

    public String getFundShortName() {
        return fundShortName;
    }

    public void setFundShortName(String fundShortName) {
        this.fundShortName = fundShortName;
    }

    public Integer getFundStructure() {
        return fundStructure;
    }

    public void setFundStructure(Integer fundStructure) {
        this.fundStructure = fundStructure;
    }

    public Byte getFundType() {
        return fundType;
    }

    public void setFundType(Byte fundType) {
        this.fundType = fundType;
    }

    public Byte getPublicFundType() {
        return publicFundType;
    }

    public void setPublicFundType(Byte publicFundType) {
        this.publicFundType = publicFundType;
    }

    public Byte getPubFundType() {
        return pubFundType;
    }

    public void setPubFundType(Byte pubFundType) {
        this.pubFundType = pubFundType;
    }

    public Byte getPubSubFundType() {
        return pubSubFundType;
    }

    public void setPubSubFundType(Byte pubSubFundType) {
        this.pubSubFundType = pubSubFundType;
    }

    public String getFundCharacteristic() {
        return fundCharacteristic;
    }

    public void setFundCharacteristic(String fundCharacteristic) {
        this.fundCharacteristic = fundCharacteristic;
    }

    public Integer getBaseCurrency() {
        return baseCurrency;
    }

    public void setBaseCurrency(Integer baseCurrency) {
        this.baseCurrency = baseCurrency;
    }

    public Date getInceptionDate() {
        return inceptionDate;
    }

    public void setInceptionDate(Date inceptionDate) {
        this.inceptionDate = inceptionDate;
    }

    public Integer getDomicile() {
        return domicile;
    }

    public void setDomicile(Integer domicile) {
        this.domicile = domicile;
    }

    public String getPrimaryBenchmarkId() {
        return primaryBenchmarkId;
    }

    public void setPrimaryBenchmarkId(String primaryBenchmarkId) {
        this.primaryBenchmarkId = primaryBenchmarkId;
    }

    public String getSecondaryBenchmark() {
        return secondaryBenchmark;
    }

    public void setSecondaryBenchmark(String secondaryBenchmark) {
        this.secondaryBenchmark = secondaryBenchmark;
    }

    public Integer getLockupPeriod() {
        return lockupPeriod;
    }

    public void setLockupPeriod(Integer lockupPeriod) {
        this.lockupPeriod = lockupPeriod;
    }

    public Byte getLockupPeriodUnit() {
        return lockupPeriodUnit;
    }

    public void setLockupPeriodUnit(Byte lockupPeriodUnit) {
        this.lockupPeriodUnit = lockupPeriodUnit;
    }

    public String getLockupPeriod2() {
        return lockupPeriod2;
    }

    public void setLockupPeriod2(String lockupPeriod2) {
        this.lockupPeriod2 = lockupPeriod2;
    }

    public Integer getLockPeriod() {
        return lockPeriod;
    }

    public void setLockPeriod(Integer lockPeriod) {
        this.lockPeriod = lockPeriod;
    }

    public Byte getLockPeriodUnit() {
        return lockPeriodUnit;
    }

    public void setLockPeriodUnit(Byte lockPeriodUnit) {
        this.lockPeriodUnit = lockPeriodUnit;
    }

    public String getOpenDay() {
        return openDay;
    }

    public void setOpenDay(String openDay) {
        this.openDay = openDay;
    }

    public String getRedemptionDay() {
        return redemptionDay;
    }

    public void setRedemptionDay(String redemptionDay) {
        this.redemptionDay = redemptionDay;
    }

    public Integer getDuration() {
        return duration;
    }

    public void setDuration(Integer duration) {
        this.duration = duration;
    }

    public BigDecimal getInitialUnitValue() {
        return initialUnitValue;
    }

    public void setInitialUnitValue(BigDecimal initialUnitValue) {
        this.initialUnitValue = initialUnitValue;
    }

    public String getInvestmentScope() {
        return investmentScope;
    }

    public void setInvestmentScope(String investmentScope) {
        this.investmentScope = investmentScope;
    }

    public String getInvestmentObjective() {
        return investmentObjective;
    }

    public void setInvestmentObjective(String investmentObjective) {
        this.investmentObjective = investmentObjective;
    }

    public String getFundPortfolio() {
        return fundPortfolio;
    }

    public void setFundPortfolio(String fundPortfolio) {
        this.fundPortfolio = fundPortfolio;
    }

    public String getInvestmentRestriction() {
        return investmentRestriction;
    }

    public void setInvestmentRestriction(String investmentRestriction) {
        this.investmentRestriction = investmentRestriction;
    }

    public String getFundInvestmentPhilosophy() {
        return fundInvestmentPhilosophy;
    }

    public void setFundInvestmentPhilosophy(String fundInvestmentPhilosophy) {
        this.fundInvestmentPhilosophy = fundInvestmentPhilosophy;
    }

    public String getFundStrategyDescription() {
        return fundStrategyDescription;
    }

    public void setFundStrategyDescription(String fundStrategyDescription) {
        this.fundStrategyDescription = fundStrategyDescription;
    }

    public String getInvestmentProcess() {
        return investmentProcess;
    }

    public void setInvestmentProcess(String investmentProcess) {
        this.investmentProcess = investmentProcess;
    }

    public String getAdvisorId() {
        return advisorId;
    }

    public void setAdvisorId(String advisorId) {
        this.advisorId = advisorId;
    }

    public String getCustodianId() {
        return custodianId;
    }

    public void setCustodianId(String custodianId) {
        this.custodianId = custodianId;
    }

    public String getBrokerId() {
        return brokerId;
    }

    public void setBrokerId(String brokerId) {
        this.brokerId = brokerId;
    }

    public String getBrokerFutureId() {
        return brokerFutureId;
    }

    public void setBrokerFutureId(String brokerFutureId) {
        this.brokerFutureId = brokerFutureId;
    }

    public String getLiquidationAgencyId() {
        return liquidationAgencyId;
    }

    public void setLiquidationAgencyId(String liquidationAgencyId) {
        this.liquidationAgencyId = liquidationAgencyId;
    }

    public String getTrustId() {
        return trustId;
    }

    public void setTrustId(String trustId) {
        this.trustId = trustId;
    }

    public String getAdministratorId() {
        return administratorId;
    }

    public void setAdministratorId(String administratorId) {
        this.administratorId = administratorId;
    }

    public String getLegalCounselId() {
        return legalCounselId;
    }

    public void setLegalCounselId(String legalCounselId) {
        this.legalCounselId = legalCounselId;
    }

    public String getAuditorId() {
        return auditorId;
    }

    public void setAuditorId(String auditorId) {
        this.auditorId = auditorId;
    }

    public String getGeneralServiceId() {
        return generalServiceId;
    }

    public void setGeneralServiceId(String generalServiceId) {
        this.generalServiceId = generalServiceId;
    }

    public String getIssuerId() {
        return issuerId;
    }

    public void setIssuerId(String issuerId) {
        this.issuerId = issuerId;
    }

    public String getNavFrequency() {
        return navFrequency;
    }

    public void setNavFrequency(String navFrequency) {
        this.navFrequency = navFrequency;
    }

    public Byte getPerformanceDisclosureMark() {
        return performanceDisclosureMark;
    }

    public void setPerformanceDisclosureMark(Byte performanceDisclosureMark) {
        this.performanceDisclosureMark = performanceDisclosureMark;
    }

    public Byte getPerformanceDisclosureType() {
        return performanceDisclosureType;
    }

    public void setPerformanceDisclosureType(Byte performanceDisclosureType) {
        this.performanceDisclosureType = performanceDisclosureType;
    }

    public Integer getIsvisible() {
        return isvisible;
    }

    public void setIsvisible(Integer isvisible) {
        this.isvisible = isvisible;
    }

    public Integer getManagerType() {
        return managerType;
    }

    public void setManagerType(Integer managerType) {
        this.managerType = managerType;
    }

    public Integer getBeginMemberCnt() {
        return beginMemberCnt;
    }

    public void setBeginMemberCnt(Integer beginMemberCnt) {
        this.beginMemberCnt = beginMemberCnt;
    }

    public Byte getRegisterPeriod() {
        return registerPeriod;
    }

    public void setRegisterPeriod(Byte registerPeriod) {
        this.registerPeriod = registerPeriod;
    }

    public String getZjxLastInfoUpdateTime() {
        return zjxLastInfoUpdateTime;
    }

    public void setZjxLastInfoUpdateTime(String zjxLastInfoUpdateTime) {
        this.zjxLastInfoUpdateTime = zjxLastInfoUpdateTime;
    }

    public String getSpecialTips() {
        return specialTips;
    }

    public void setSpecialTips(String specialTips) {
        this.specialTips = specialTips;
    }

    public String getAmacUrl() {
        return amacUrl;
    }

    public void setAmacUrl(String amacUrl) {
        this.amacUrl = amacUrl;
    }

    public Byte getRaiseType() {
        return raiseType;
    }

    public void setRaiseType(Byte raiseType) {
        this.raiseType = raiseType;
    }

    public String getRiskReturnDesc() {
        return riskReturnDesc;
    }

    public void setRiskReturnDesc(String riskReturnDesc) {
        this.riskReturnDesc = riskReturnDesc;
    }

    public Integer getCreatorid() {
        return creatorid;
    }

    public void setCreatorid(Integer creatorid) {
        this.creatorid = creatorid;
    }

    public Date getCreatetime() {
        return createtime;
    }

    public void setCreatetime(Date createtime) {
        this.createtime = createtime;
    }

    public Integer getUpdaterid() {
        return updaterid;
    }

    public void setUpdaterid(Integer updaterid) {
        this.updaterid = updaterid;
    }

    public Date getUpdatetime() {
        return updatetime;
    }

    public void setUpdatetime(Date updatetime) {
        this.updatetime = updatetime;
    }

    public Integer getIsvalid() {
        return isvalid;
    }

    public void setIsvalid(Integer isvalid) {
        this.isvalid = isvalid;
    }

    public String getRegisterNumber() {
        return registerNumber;
    }

    public void setRegisterNumber(String registerNumber) {
        this.registerNumber = registerNumber;
    }

    public Byte getIstiered() {
        return istiered;
    }

    public void setIstiered(Byte istiered) {
        this.istiered = istiered;
    }

    public Integer getIstieredDesc() {
        return istieredDesc;
    }

    public void setIstieredDesc(Integer istieredDesc) {
        this.istieredDesc = istieredDesc;
    }

    public Date getRegisterDate() {
        return registerDate;
    }

    public void setRegisterDate(Date registerDate) {
        this.registerDate = registerDate;
    }

    public Integer getIsRanking() {
        return isRanking;
    }

    public void setIsRanking(Integer isRanking) {
        this.isRanking = isRanking;
    }

    public Integer getIsRating() {
        return isRating;
    }

    public void setIsRating(Integer isRating) {
        this.isRating = isRating;
    }

    public String getNavSource() {
        return navSource;
    }

    public void setNavSource(String navSource) {
        this.navSource = navSource;
    }

    public String getContractName() {
        return contractName;
    }

    public void setContractName(String contractName) {
        this.contractName = contractName;
    }

    public String getContractPath() {
        return contractPath;
    }

    public void setContractPath(String contractPath) {
        this.contractPath = contractPath;
    }

    public Integer getIsAuthorized() {
        return isAuthorized;
    }

    public void setIsAuthorized(Integer isAuthorized) {
        this.isAuthorized = isAuthorized;
    }

    public Integer getIsInResearch() {
        return isInResearch;
    }

    public void setIsInResearch(Integer isInResearch) {
        this.isInResearch = isInResearch;
    }

    public Integer getVisibleCtrl() {
        return visibleCtrl;
    }

    public void setVisibleCtrl(Integer visibleCtrl) {
        this.visibleCtrl = visibleCtrl;
    }

    public Integer getIsNavVisible() {
        return isNavVisible;
    }

    public void setIsNavVisible(Integer isNavVisible) {
        this.isNavVisible = isNavVisible;
    }

    public Integer getIsNavVisibleOrg() {
        return isNavVisibleOrg;
    }

    public void setIsNavVisibleOrg(Integer isNavVisibleOrg) {
        this.isNavVisibleOrg = isNavVisibleOrg;
    }

    public Byte getIsFeeBefore() {
        return isFeeBefore;
    }

    public void setIsFeeBefore(Byte isFeeBefore) {
        this.isFeeBefore = isFeeBefore;
    }

    public Byte getIsDeductReward() {
        return isDeductReward;
    }

    public void setIsDeductReward(Byte isDeductReward) {
        this.isDeductReward = isDeductReward;
    }

    public String getValuationInstitutionId() {
        return valuationInstitutionId;
    }

    public void setValuationInstitutionId(String valuationInstitutionId) {
        this.valuationInstitutionId = valuationInstitutionId;
    }

    public String getTrustRegisterNumber() {
        return trustRegisterNumber;
    }

    public void setTrustRegisterNumber(String trustRegisterNumber) {
        this.trustRegisterNumber = trustRegisterNumber;
    }

    public Byte getIsConsignment() {
        return isConsignment;
    }

    public void setIsConsignment(Byte isConsignment) {
        this.isConsignment = isConsignment;
    }

    public Byte getIsQdii() {
        return isQdii;
    }

    public void setIsQdii(Byte isQdii) {
        this.isQdii = isQdii;
    }

    public Byte getIsExclusive() {
        return isExclusive;
    }

    public void setIsExclusive(Byte isExclusive) {
        this.isExclusive = isExclusive;
    }

    public Byte getIsShowIndependent() {
        return isShowIndependent;
    }

    public void setIsShowIndependent(Byte isShowIndependent) {
        this.isShowIndependent = isShowIndependent;
    }

    public Date getTrustApplyRegisterDate() {
        return trustApplyRegisterDate;
    }

    public void setTrustApplyRegisterDate(Date trustApplyRegisterDate) {
        this.trustApplyRegisterDate = trustApplyRegisterDate;
    }

    public Date getTrustPublicityDate() {
        return trustPublicityDate;
    }

    public void setTrustPublicityDate(Date trustPublicityDate) {
        this.trustPublicityDate = trustPublicityDate;
    }

    public String getTrustMainIndustry() {
        return trustMainIndustry;
    }

    public void setTrustMainIndustry(String trustMainIndustry) {
        this.trustMainIndustry = trustMainIndustry;
    }

    public String getTrustPropertyApplication() {
        return trustPropertyApplication;
    }

    public void setTrustPropertyApplication(String trustPropertyApplication) {
        this.trustPropertyApplication = trustPropertyApplication;
    }

    public Byte getTrustFunction() {
        return trustFunction;
    }

    public void setTrustFunction(Byte trustFunction) {
        this.trustFunction = trustFunction;
    }

    public Byte getIsAmacShow() {
        return isAmacShow;
    }

    public void setIsAmacShow(Byte isAmacShow) {
        this.isAmacShow = isAmacShow;
    }

    @Override
    public String toString() {
        return "FundInformationDo{" +
                "id=" + id +
                ", pFundId='" + pFundId + '\'' +
                ", fundId='" + fundId + '\'' +
                ", fundName='" + fundName + '\'' +
                ", fundShortName='" + fundShortName + '\'' +
                ", fundStructure=" + fundStructure +
                ", fundType=" + fundType +
                ", publicFundType=" + publicFundType +
                ", pubFundType=" + pubFundType +
                ", pubSubFundType=" + pubSubFundType +
                ", fundCharacteristic='" + fundCharacteristic + '\'' +
                ", baseCurrency=" + baseCurrency +
                ", inceptionDate=" + inceptionDate +
                ", domicile=" + domicile +
                ", primaryBenchmarkId='" + primaryBenchmarkId + '\'' +
                ", secondaryBenchmark='" + secondaryBenchmark + '\'' +
                ", lockupPeriod=" + lockupPeriod +
                ", lockupPeriodUnit=" + lockupPeriodUnit +
                ", lockupPeriod2='" + lockupPeriod2 + '\'' +
                ", lockPeriod=" + lockPeriod +
                ", lockPeriodUnit=" + lockPeriodUnit +
                ", openDay='" + openDay + '\'' +
                ", redemptionDay='" + redemptionDay + '\'' +
                ", duration=" + duration +
                ", initialUnitValue=" + initialUnitValue +
                ", investmentScope='" + investmentScope + '\'' +
                ", investmentObjective='" + investmentObjective + '\'' +
                ", fundPortfolio='" + fundPortfolio + '\'' +
                ", investmentRestriction='" + investmentRestriction + '\'' +
                ", fundInvestmentPhilosophy='" + fundInvestmentPhilosophy + '\'' +
                ", fundStrategyDescription='" + fundStrategyDescription + '\'' +
                ", investmentProcess='" + investmentProcess + '\'' +
                ", advisorId='" + advisorId + '\'' +
                ", custodianId='" + custodianId + '\'' +
                ", brokerId='" + brokerId + '\'' +
                ", brokerFutureId='" + brokerFutureId + '\'' +
                ", liquidationAgencyId='" + liquidationAgencyId + '\'' +
                ", trustId='" + trustId + '\'' +
                ", administratorId='" + administratorId + '\'' +
                ", legalCounselId='" + legalCounselId + '\'' +
                ", auditorId='" + auditorId + '\'' +
                ", generalServiceId='" + generalServiceId + '\'' +
                ", issuerId='" + issuerId + '\'' +
                ", navFrequency='" + navFrequency + '\'' +
                ", performanceDisclosureMark=" + performanceDisclosureMark +
                ", performanceDisclosureType=" + performanceDisclosureType +
                ", isvisible=" + isvisible +
                ", managerType=" + managerType +
                ", beginMemberCnt=" + beginMemberCnt +
                ", registerPeriod=" + registerPeriod +
                ", zjxLastInfoUpdateTime='" + zjxLastInfoUpdateTime + '\'' +
                ", specialTips='" + specialTips + '\'' +
                ", amacUrl='" + amacUrl + '\'' +
                ", raiseType=" + raiseType +
                ", riskReturnDesc='" + riskReturnDesc + '\'' +
                ", creatorid=" + creatorid +
                ", createtime=" + createtime +
                ", updaterid=" + updaterid +
                ", updatetime=" + updatetime +
                ", isvalid=" + isvalid +
                ", registerNumber='" + registerNumber + '\'' +
                ", istiered=" + istiered +
                ", istieredDesc=" + istieredDesc +
                ", registerDate=" + registerDate +
                ", isRanking=" + isRanking +
                ", isRating=" + isRating +
                ", navSource='" + navSource + '\'' +
                ", contractName='" + contractName + '\'' +
                ", contractPath='" + contractPath + '\'' +
                ", isAuthorized=" + isAuthorized +
                ", isInResearch=" + isInResearch +
                ", visibleCtrl=" + visibleCtrl +
                ", isNavVisible=" + isNavVisible +
                ", isNavVisibleOrg=" + isNavVisibleOrg +
                ", isFeeBefore=" + isFeeBefore +
                ", isDeductReward=" + isDeductReward +
                ", valuationInstitutionId='" + valuationInstitutionId + '\'' +
                ", trustRegisterNumber='" + trustRegisterNumber + '\'' +
                ", isConsignment=" + isConsignment +
                ", isQdii=" + isQdii +
                ", isExclusive=" + isExclusive +
                ", isShowIndependent=" + isShowIndependent +
                ", trustApplyRegisterDate=" + trustApplyRegisterDate +
                ", trustPublicityDate=" + trustPublicityDate +
                ", trustMainIndustry='" + trustMainIndustry + '\'' +
                ", trustPropertyApplication='" + trustPropertyApplication + '\'' +
                ", trustFunction=" + trustFunction +
                ", isAmacShow=" + isAmacShow +
                '}';
    }
}