package com.smppw.analysis.domain.dto.position;

import cn.hutool.core.collection.ListUtil;
import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.constants.SecType;

import java.util.List;

/**
 * @author wangzaijun
 * @date 2023/6/19 11:26
 * @description 基金持仓的常量类
 */
public final class PositionConstants {
    public static final String ASSET_MONEY = "MONEY";
    public static final String ASSET_MONEY_DESC = "存款及备付金";

    public static final String ASSET_RETURN_SALE = "RETURN_SALE";
    public static final String ASSET_RETURN_SALE_CODE = "2202";
    public static final String ASSET_RETURN_SALE_DESC = "逆回购";

    public static final String ASSET_SALE = "SALE";
    public static final String ASSET_SALE_CODE = "1202";
    public static final String ASSET_SALE_DESC = "正回购";

    public static final String[] SALE_CODES = {ASSET_RETURN_SALE_CODE, ASSET_SALE_CODE};

    public static final String ASSET_OTHER = "OTHER";
    public static final String ASSET_OTHER_DESC = "其他";

    /**
     * 其他类
     */
    public static final ValueLabelVO OTHER_ASSET = new ValueLabelVO(ASSET_OTHER, ASSET_OTHER_DESC);
    /**
     * 私募基金支持的类型
     */
    public static final List<String> PRIVATELY_TYPES = ListUtil.of(SecType.PRIVATELY_OFFERED_FUND, SecType.PRIVATE_FUND);
    /**
     * 国债期货和股指期货,商品期货
     */
    public static final List<Integer> FUTURE_SECOND_TYPES = ListUtil.of(HoldingType.IndexFuture.getId(),
            HoldingType.BondFuture.getId(), HoldingType.CmdtFuture.getId());

    /**
     * 股指
     * IF - 沪深300指数
     * IH - 上证50指数
     * IC - 中证500指数
     * IM - 中证1000指数
     */
    public static final List<String> STOCK_INDEX_BREEDS = ListUtil.of("IF", "IH", "IC", "IM");
    /**
     * 国债期货
     * 票面利率3%的2年期名义中短期国债 - TS
     * 票面利率3%的5年期名义中期国债 - TF
     * 票面利率3%的名义超长期国债 - TL
     * 票面利率3%的名义长期国债 - T
     */
    public static final String[]  BOND_BREED_ARR = {"TS", "TF", "TL", "T"};
    /**
     * 国债期货
     * 票面利率3%的2年期名义中短期国债 - TS
     * 票面利率3%的5年期名义中期国债 - TF
     * 票面利率3%的名义超长期国债 - TL
     * 票面利率3%的名义长期国债 - T
     */
    public static final List<String> BOND_BREEDS = ListUtil.of(BOND_BREED_ARR);
}
