package com.smppw.analysis.domain.dto.position;

import cn.hutool.core.text.CharSequenceUtil;

import java.util.Arrays;

/**
 * @author wangzaijun
 * @date 2023/6/9 17:16
 * @description 基金持仓分析 风格枚举
 */
public enum PositionStyleEnum {
    MARKET_GROWTH(201, 101, "大盘成长型"),
    MARKET_VALUE(201, 102, "大盘价值型"),
    MARKET_BALANCE(201, 103, "大盘平衡型"),
    MID_MARKET_GROWTH(202, 101, "中盘成长型"),
    MID_MARKET_VALUE(202, 102, "中盘价值型"),
    MID_MARKET_BALANCE(202, 103, "中盘平衡型"),
    SMALL_MARKET_GROWTH(203, 101, "小盘成长型"),
    SMALL_MARKET_VALUE(203, 102, "小盘价值型"),
    SMALL_MARKET_BALANCE(203, 103, "小盘平衡型");

    private final int market;
    private final int balance;
    private final String name;

    PositionStyleEnum(int market, int balance, String name) {
        this.market = market;
        this.balance = balance;
        this.name = name;
    }

    public static PositionStyleEnum getStyle(Integer market, Integer balance) {
        if (market == null || balance == null) {
            return null;
        }
        return Arrays.stream(PositionStyleEnum.values()).filter(e -> market.equals(e.getMarket()))
                .filter(e -> balance.equals(e.getBalance())).findFirst().orElse(null);
    }

    public static PositionStyleEnum getStyle(String code) {
        if (CharSequenceUtil.isEmpty(code)) {
            return null;
        }
        return Arrays.stream(PositionStyleEnum.values()).filter(e -> e.name().equals(code)).findFirst().orElse(null);
    }

    public int getMarket() {
        return market;
    }

    public int getBalance() {
        return balance;
    }

    public String getName() {
        return name;
    }
}
