package com.smppw.analysis.domain.dto.position.bond;

import com.smppw.analysis.domain.dto.position.RefMarketValueRatio;
import lombok.Getter;
import lombok.Setter;

import java.util.List;

/**
 * @author wangzaijun
 * @date 2023/6/6 18:30
 * @description 债券 收益风险 接口返回结构
 */
@Setter
@Getter
public class ProfitRiskVO {
    /**
     * 估值日期
     */
    private String date;
    /**
     * 每个日期每个债券的收益风险，基于此基本数据可以求出整体的数据
     * 包括：信用利差、剩余期限、dvbp、凸性、修正久期、到期收益率=∑个券市值占净值比*个券各项指标
     */
    private List<ProfitRisk> profitRisk;

    /**
     * @author wangzaijun
     * @date 2023/6/6 18:39
     * @description 债券收益风险 实体对象
     */
    @Setter
    @Getter
    public static class ProfitRisk extends RefMarketValueRatio {
        /**
         * 凸性
         */
        private String convexity;
        /**
         * 修正久期
         */
        private String correctDuration;
        /**
         * 信用利差
         */
        private String creditSpread;
        /**
         * DVBP
         */
        private String dvbp;
        /**
         * 到期收益率
         */
        private String yieldMaturity;
    }
}
