package com.smppw.analysis.domain.dto.position.stock;

import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.analysis.domain.dto.position.RefMarketValueRatio;
import lombok.Getter;
import lombok.Setter;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author wangzaijun
 * @date 2023/6/6 16:31
 * @description 股票业绩归因 接口请求参数
 */
@Setter
@Getter
public class StockPerformanceAttributionVO {
    /**
     * 估值日期
     */
    private String date;
    /**
     * 每个行业的效应、市值和占净值比
     */
    private List<Effect> effect;

    /**
     * @author wangzaijun
     * @date 2023/6/6 16:53
     * @description 每个行业的效应、市值和占净值比数据
     */
    @Setter
    @Getter
    public static class Effect extends RefMarketValueRatio {
        /**
         * 超额收益=(1+持仓组合收益)/(1+基准指数收益)-1
         * 仓组合收益=估值日单位净值/期初单位净值-1
         * 超额=配置+选股+择时
         */
        private Double exactRet;
        /**
         * 配置效应
         */
        private Double allocation;
        /**
         * 选股效应
         */
        private Double stock;
        /**
         * 择时效应
         */
        private Double timing;

        public Effect() {
            super();
        }

        public Effect(ValueLabelVO ref, BigDecimal marketValue, BigDecimal ratio) {
            super(ref, marketValue, ratio);
        }
    }
}
