package com.smppw.analysis.domain.event;

import cn.hutool.core.util.StrUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.util.function.Function;

/**
 * @author wangzaijun
 * @date 2023/8/9 14:13
 * @description 领域层缓存事件监听
 */
@Component
public class CacheEventListener {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @EventListener(value = {SaveCacheEvent.class})
    public void saveCache(SaveCacheEvent<Object> event) {
        Object source = event.getSource();
        Function<Object, Boolean> function = event.getFunction();
        Boolean apply = function.apply(event.getTarget());
        if (apply == null || apply == Boolean.FALSE) {
            logger.warn(StrUtil.format("{} 缓存保存失败！", source));
        }
        if (logger.isDebugEnabled()) {
            long time = System.currentTimeMillis() - event.getTimestamp();
            logger.debug(StrUtil.format("{} 缓存保存耗时 {} ms.", source, time));
        }
    }
}
