package com.smppw.analysis.domain.event;

import org.springframework.context.ApplicationEvent;

import java.util.function.Function;

/**
 * @author wangzaijun
 * @date 2023/8/9 14:09
 * @description 领域层缓存保存事件, source建议为设置缓存key
 */
public class SaveCacheEvent<T> extends ApplicationEvent {
    /**
     * 保存对象
     */
    private final T target;
    /**
     * 如何保存缓存的操作，返回一个bool类型标识是否成功
     */
    private final Function<T, Boolean> function;

    public SaveCacheEvent(Object source, T target, Function<T, Boolean> function) {
        super(source);
        this.target = target;
        this.function = function;
    }

    public T getTarget() {
        return target;
    }

    public Function<T, Boolean> getFunction() {
        return function;
    }
}
