package com.smppw.analysis.domain.manager.performance;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Pair;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.performance.HistoryRetTableSingleData;
import com.smppw.analysis.domain.dto.performance.HistoryRetTableSingleResp;
import com.smppw.analysis.domain.dto.performance.HistoryRetTableSingleRet;
import com.smppw.common.cache.CaffeineLocalCache;
import com.smppw.common.pojo.dto.calc.IndicatorCalcPropertyDto;
import com.smppw.common.pojo.dto.time.TimeIntervalDto;
import com.smppw.common.pojo.enums.Frequency;
import com.smppw.common.pojo.enums.Indicator;
import com.smppw.constants.DateConst;
import org.apache.commons.lang3.time.DateUtils;

import java.util.*;
import java.util.stream.Collectors;

public class HistoryRetTableUtil {
    public static final String EXTRA = "extra";
    public static final String YEAR_ROLL_DEFAULT_ID = "YearRoll";
    public static final String MONTH_ROLL_DEFAULT_ID = "MonthRoll";
    public static final List<String> YEAR_MONTH_ROLL_LIST = ListUtil.of(YEAR_ROLL_DEFAULT_ID, MONTH_ROLL_DEFAULT_ID);

    private static final Map<Frequency, List<Integer>> CATE_LIST_MAP = MapUtil.newHashMap();
    private static final Map<Frequency, List<String>> MONTH_WEEK_LIST_MAP = MapUtil.newHashMap();
    private static final Map<String, Pair<String, String>> MONTH_QUARTERLY_MAP = MapUtil.newHashMap();

    static {
        CATE_LIST_MAP.put(Frequency.Monthly, ListUtil.toLinkedList(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12));
        CATE_LIST_MAP.put(Frequency.Weekly, ListUtil.toLinkedList(1, 2, 3, 4, 5));
        CATE_LIST_MAP.put(Frequency.Quarterly, ListUtil.toLinkedList(1, 2, 3, 4));
        CATE_LIST_MAP.put(Frequency.Annually, ListUtil.list(true));
        MONTH_WEEK_LIST_MAP.put(Frequency.Monthly, ListUtil.toLinkedList("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12"));
        MONTH_WEEK_LIST_MAP.put(Frequency.Weekly, ListUtil.toLinkedList("1", "2", "3", "4", "5"));
        MONTH_WEEK_LIST_MAP.put(Frequency.Quarterly, ListUtil.toLinkedList("03", "06", "09", "12"));
        MONTH_WEEK_LIST_MAP.put(Frequency.Annually, ListUtil.list(true));

        MONTH_QUARTERLY_MAP.put("01", Pair.of("01", "03"));
        MONTH_QUARTERLY_MAP.put("02", Pair.of("01", "03"));
        MONTH_QUARTERLY_MAP.put("03", Pair.of("01", "03"));
        MONTH_QUARTERLY_MAP.put("04", Pair.of("03", "06"));
        MONTH_QUARTERLY_MAP.put("05", Pair.of("03", "06"));
        MONTH_QUARTERLY_MAP.put("06", Pair.of("03", "06"));
        MONTH_QUARTERLY_MAP.put("07", Pair.of("06", "09"));
        MONTH_QUARTERLY_MAP.put("08", Pair.of("06", "09"));
        MONTH_QUARTERLY_MAP.put("09", Pair.of("06", "09"));
        MONTH_QUARTERLY_MAP.put("10", Pair.of("09", "12"));
        MONTH_QUARTERLY_MAP.put("11", Pair.of("09", "12"));
        MONTH_QUARTERLY_MAP.put("12", Pair.of("09", "12"));
    }

    public static HistoryRetTableSingleResp getHistoryRetTableSingleResp(List<String> secIds, List<String> indexIdList, Map<String, List<IndicatorCalcPropertyDto>> indicatorCalcPropertyDtoListMap,
                                                                         Indicator indicator, boolean ifExcessReturn, Frequency rollingFrequency) {
        TimeIntervalDto timeIntervalDto = new TimeIntervalDto();
        List<String> yearOrYearMonthList = ListUtil.list(true);
        List<Integer> cates = CATE_LIST_MAP.get(rollingFrequency);
        List<String> monthOrWeekList = MONTH_WEEK_LIST_MAP.get(rollingFrequency);

        // Map<secId, Map<年或者年月, Map<月或者周, value>>>
        Map<String, Map<String, Map<String, String>>> secYearMonthValueMapMapMap = new HashMap<>(16);
        // Map<secId, Map<年或者年月, value>>
        Map<String, Map<String, String>> secYearValueMapMap = new HashMap<>(16);

        for (String secId : secIds) {
            List<IndicatorCalcPropertyDto> indicatorCalcPropertyDtoList = indicatorCalcPropertyDtoListMap.get(secId);
            for (IndicatorCalcPropertyDto indicatorCalcPropertyDto : indicatorCalcPropertyDtoList) {
                String id = indicatorCalcPropertyDto.getIndicatorCalcReq().getIndicatorCalcTimeRangeDto().getId();
                if (Frequency.Monthly == rollingFrequency || Frequency.Quarterly == rollingFrequency) {
                    if (YEAR_ROLL_DEFAULT_ID.equals(id)) {
                        prepareSingleYearDate(secId, indexIdList, indicatorCalcPropertyDto, indicator, secYearValueMapMap, ifExcessReturn);
                    } else {
                        prepareSingleMonthDate(secId, indexIdList, indicatorCalcPropertyDto, indicator, timeIntervalDto, yearOrYearMonthList, secYearMonthValueMapMapMap, ifExcessReturn);
                    }
                } else if (Frequency.Weekly == rollingFrequency) {
                    if (MONTH_ROLL_DEFAULT_ID.equals(id)) {
                        prepareSingleYearMonthDate(secId, indexIdList, indicatorCalcPropertyDto, indicator, secYearValueMapMap, ifExcessReturn);
                    } else {
                        prepareSingleWeekDate(secId, indexIdList, indicatorCalcPropertyDto, indicator, timeIntervalDto, yearOrYearMonthList, secYearMonthValueMapMapMap, ifExcessReturn);
                    }
                } else if (Frequency.Annually == rollingFrequency) {
                    prepareSingleYearDate(secId, indexIdList, indicatorCalcPropertyDto, indicator, secYearValueMapMap, ifExcessReturn);
                    prepareSingleMonthDate(secId, indexIdList, indicatorCalcPropertyDto, indicator, timeIntervalDto, yearOrYearMonthList, secYearMonthValueMapMapMap, ifExcessReturn);
                }
            }
        }

        Collections.sort(yearOrYearMonthList);

        List<HistoryRetTableSingleData> data = new ArrayList<>();
        for (int i = yearOrYearMonthList.size() - 1; i >= 0; i--) {
            List<HistoryRetTableSingleRet> rets = new ArrayList<>();
            String yearOrYearMonth = yearOrYearMonthList.get(i);
            HistoryRetTableSingleData historyRetTableSingleData = getHistoryRetTableSingleData(secIds, monthOrWeekList,
                    secYearMonthValueMapMapMap, secYearValueMapMap, rets, yearOrYearMonth, rollingFrequency);

            if (ifExcessReturn) {
                for (String secId : secIds) {
                    HistoryRetTableSingleRet extraRet = new HistoryRetTableSingleRet();
                    List<String> secExtraRetList = new ArrayList<>();
                    String yearValue = null;
                    if (secYearValueMapMap.get(secId + EXTRA) != null) {
                        yearValue = secYearValueMapMap.get(secId + EXTRA).get(yearOrYearMonth);
                    }
                    secExtraRetList.add(yearValue);
                    Map<String, Map<String, String>> yearMonthValueMapMap = secYearMonthValueMapMapMap.get(secId + EXTRA);
                    Map<String, String> monthValueMap = Optional.ofNullable(yearMonthValueMapMap).map(e -> e.get(yearOrYearMonth)).orElse(null);
                    List<String> monthList = null;
                    if (monthValueMap != null) {
                        monthList = new ArrayList<>(monthValueMap.keySet());
                    }
                    for (String monthOrWeek : monthOrWeekList) {
                        String value = null;
                        if (monthValueMap != null) {
                            value = monthValueMap.get(monthOrWeek);
                            if (value == null && rollingFrequency == Frequency.Quarterly) {
                                Pair<String, String> pair = MONTH_QUARTERLY_MAP.get(monthOrWeek);
                                String s = monthList.stream().filter(e -> e.compareTo(pair.getKey()) > 0).filter(e -> e.compareTo(pair.getValue()) < 0)
                                        .filter(e -> e.compareTo(monthOrWeek) <= 0).max(String::compareTo).orElse(null);
                                value = s == null ? null : monthValueMap.get(s);
                            }
                        }
                        secExtraRetList.add(value);
                    }
                    extraRet.setId(secId);
                    extraRet.setType("extraRet");
                    extraRet.setRet(secExtraRetList);
                    rets.add(extraRet);
                }
            }

            calcIndexRet(indexIdList, monthOrWeekList, secYearMonthValueMapMapMap, secYearValueMapMap, data, historyRetTableSingleData, yearOrYearMonth, rets, rollingFrequency);
        }

        HistoryRetTableSingleResp resp = new HistoryRetTableSingleResp();
        resp.setData(data);
        resp.setDatetime(cates);
        resp.setStartDate(timeIntervalDto.getStartDate());
        resp.setEndDate(timeIntervalDto.getEndDate());
        return resp;
    }

    /**
     * 构建年度统计表格数据
     *
     * @param productNameMapping 名称映射
     * @param resp               数据
     * @return /
     */
    public static Map<String, Object> buildTableData(Map<String, String> productNameMapping, HistoryRetTableSingleResp resp, boolean excess, boolean flag) {
        List<Map<String, Object>> data = ListUtil.list(true);
        for (HistoryRetTableSingleData datum : resp.getData()) {
            String year = datum.getYear();
            if (excess && flag) {
                List<HistoryRetTableSingleRet> collect = datum.getRets().stream().filter(e -> "extraRet".equals(e.getType())).collect(Collectors.toList());
                for (HistoryRetTableSingleRet ret : collect) {
                    Map<String, Object> temp = MapUtil.<String, Object>builder("year", year)
                            .put("name", productNameMapping.get(ret.getId()))
                            .put("ret", ret.getRet()).build();
                    data.add(temp);
                }
            } else {
                for (HistoryRetTableSingleRet ret : datum.getRets()) {
                    Map<String, Object> temp = MapUtil.<String, Object>builder("year", year)
                            .put("name", "ret".equals(ret.getType()) ? productNameMapping.get(ret.getId()) : productNameMapping.get(ret.getId()) + "-超额收益")
                            .put("ret", ret.getRet()).build();
                    data.add(temp);
                }
            }
        }
        return MapUtil.<String, Object>builder("datetime", resp.getDatetime()).put("data", data).build();
    }

    private static HistoryRetTableSingleData getHistoryRetTableSingleData(List<String> secIds, List<String> monthOrWeekList, Map<String, Map<String, Map<String, String>>> secYearMonthValueMapMapMap,
                                                                          Map<String, Map<String, String>> secYearValueMapMap, List<HistoryRetTableSingleRet> rets, String yearOrYearMonth, Frequency rollingFrequency) {
        HistoryRetTableSingleData historyRetTableSingleData = new HistoryRetTableSingleData();
        historyRetTableSingleData.setYear(yearOrYearMonth);

        for (String secId : secIds) {
            HistoryRetTableSingleRet ret = new HistoryRetTableSingleRet();
            List<String> secRetList = new ArrayList<>();
            String yearValue = null;
            if (secYearValueMapMap.get(secId) != null) {
                yearValue = secYearValueMapMap.get(secId).get(yearOrYearMonth);
            }

            secRetList.add(yearValue);
            Map<String, Map<String, String>> yearMonthValueMapMap = secYearMonthValueMapMapMap.get(secId);
            Map<String, String> monthValueMap = Optional.ofNullable(yearMonthValueMapMap).map(e -> e.get(yearOrYearMonth)).orElse(null);
            List<String> monthList = null;
            if (monthValueMap != null) {
                monthList = new ArrayList<>(monthValueMap.keySet());
            }
            for (String monthOrWeek : monthOrWeekList) {
                String value = null;
                if (monthValueMap != null) {
                    value = monthValueMap.get(monthOrWeek);
                    if (value == null && rollingFrequency == Frequency.Quarterly) {
                        Pair<String, String> pair = MONTH_QUARTERLY_MAP.get(monthOrWeek);
                        String s = monthList.stream().filter(e -> e.compareTo(pair.getKey()) > 0).filter(e -> e.compareTo(pair.getValue()) < 0)
                                .filter(e -> e.compareTo(monthOrWeek) <= 0).max(String::compareTo).orElse(null);
                        value = s == null ? null : monthValueMap.get(s);
                    }
                }
                secRetList.add(value);
            }
            ret.setId(secId);
            ret.setType("ret");
            ret.setRet(secRetList);
            rets.add(ret);
        }
        return historyRetTableSingleData;
    }

    private static void prepareSingleWeekDate(String secId, List<String> indexIdList, IndicatorCalcPropertyDto indicatorCalcPropertyDto, Indicator indicator, TimeIntervalDto timeIntervalDto,
                                              List<String> yearMonthList, Map<String, Map<String, Map<String, String>>> secYearMonthWeekValueMapMapMap, boolean ifExtraRet) {
        String startDate1 = indicatorCalcPropertyDto.getIndicatorCalcReq().getIndicatorCalcTimeRangeDto().getStartDate();
        String endDate1 = indicatorCalcPropertyDto.getIndicatorCalcReq().getIndicatorCalcTimeRangeDto().getEndDate();
        if (StrUtil.isBlank(timeIntervalDto.getStartDate()) || startDate1.compareTo(timeIntervalDto.getStartDate()) < 0) {
            timeIntervalDto.setStartDate(startDate1);
        }
        if (StrUtil.isBlank(timeIntervalDto.getEndDate()) || endDate1.compareTo(timeIntervalDto.getEndDate()) > 0) {
            timeIntervalDto.setEndDate(endDate1);
        }
        String calcEndDate = indicatorCalcPropertyDto.getIndicatorCalcReq().getIndicatorCalcTimeRangeDto().getEndDate();
        String yearMonthWeek = CaffeineLocalCache.getDateMonthWeek(calcEndDate);
        String yearMonth = yearMonthWeek.substring(0, 7);
        String week = yearMonthWeek.substring(8);
        if (!yearMonthList.contains(yearMonth)) {
            yearMonthList.add(yearMonth);
        }
        handleMonthWeekYearData(secId, indexIdList, indicatorCalcPropertyDto, indicator, secYearMonthWeekValueMapMapMap, ifExtraRet, yearMonth, week);
    }

    private static void prepareSingleYearDate(String secId, List<String> indexIdList, IndicatorCalcPropertyDto indicatorCalcPropertyDto, Indicator indicator,
                                              Map<String, Map<String, String>> secYearValueMapMap, boolean ifExcessRet) {
        String endDate1 = indicatorCalcPropertyDto.getIndicatorCalcReq().getIndicatorCalcTimeRangeDto().getEndDate();
        Date monthDate = DateUtils.addDays(DateUtil.parse(endDate1, DateConst.YYYY_MM_DD), -5);
        String date = DateUtil.format(monthDate, DateConst.YYYY_MM_DD);
        String year = date.substring(0, 4);
        String value = indicatorCalcPropertyDto.getSecData().getIndicatorValueMap().get(indicator.name());
        if (secYearValueMapMap.containsKey(secId)) {
            secYearValueMapMap.get(secId).put(year, value);
        } else {
            Map<String, String> yearValueMap = new HashMap<>(16);
            yearValueMap.put(year, value);
            secYearValueMapMap.put(secId, yearValueMap);
        }
        if (Indicator.ExcessReturn == indicator || ifExcessRet) {
            String extraValue = indicatorCalcPropertyDto.getSecData().getExtraGeoIndicatorValueMap().get(indicator.name());
            if (secYearValueMapMap.containsKey(secId + EXTRA)) {
                secYearValueMapMap.get(secId + EXTRA).put(year, extraValue);
            } else {
                Map<String, String> yearValueMap = new HashMap<>(16);
                yearValueMap.put(year, extraValue);
                secYearValueMapMap.put(secId + EXTRA, yearValueMap);
            }
        }

        for (String indexId : indexIdList) {
            String indexValue = indicatorCalcPropertyDto.getIndexData().getIndexIndicatorValueMap().get(indexId).get(indicator.name());
            if (secYearValueMapMap.containsKey(indexId)) {
                secYearValueMapMap.get(indexId).put(year, indexValue);
            } else {
                Map<String, String> yearValueMap = new HashMap<>(16);
                yearValueMap.put(year, indexValue);
                secYearValueMapMap.put(indexId, yearValueMap);
            }
        }
    }

    private static void prepareSingleMonthDate(String secId, List<String> indexIdList, IndicatorCalcPropertyDto indicatorCalcPropertyDto, Indicator indicator,
                                               TimeIntervalDto timeInterval, List<String> yearList, Map<String, Map<String, Map<String, String>>> secYearMonthValueMapMapMap, boolean ifExcessRet) {
        String startDate1 = indicatorCalcPropertyDto.getIndicatorCalcReq().getIndicatorCalcTimeRangeDto().getStartDate();
        String endDate1 = indicatorCalcPropertyDto.getIndicatorCalcReq().getIndicatorCalcTimeRangeDto().getEndDate();
        if (StrUtil.isBlank(timeInterval.getStartDate()) || startDate1.compareTo(timeInterval.getStartDate()) < 0) {
            timeInterval.setStartDate(startDate1);
        }
        if (StrUtil.isBlank(timeInterval.getEndDate()) || endDate1.compareTo(timeInterval.getEndDate()) > 0) {
            timeInterval.setEndDate(endDate1);
        }
        String yearMonth = indicatorCalcPropertyDto.getIndicatorCalcReq().getIndicatorCalcTimeRangeDto().getId();
        String year = yearMonth.substring(0, 4);
        if (!yearList.contains(year)) {
            yearList.add(year);
        }
        String month = yearMonth.substring(5);
        handleMonthWeekYearData(secId, indexIdList, indicatorCalcPropertyDto, indicator, secYearMonthValueMapMapMap, ifExcessRet, year, month);
    }

    private static void handleMonthWeekYearData(String secId, List<String> indexIdList, IndicatorCalcPropertyDto indicatorCalcPropertyDto,
                                                Indicator indicator, Map<String, Map<String, Map<String, String>>> secYearMonthValueMapMapMap,
                                                boolean ifExcessRet, String year, String month) {
        String value = indicatorCalcPropertyDto.getSecData().getIndicatorValueMap().get(indicator.name());
        if (secYearMonthValueMapMapMap.containsKey(secId)) {
            Map<String, Map<String, String>> yearMonthValueMapMap = secYearMonthValueMapMapMap.get(secId);
            if (yearMonthValueMapMap.containsKey(year)) {
                Map<String, String> monthValueMap = yearMonthValueMapMap.get(year);
                monthValueMap.put(month, value);
            } else {
                Map<String, String> monthValueMap = new HashMap<>(16);
                monthValueMap.put(month, value);
                yearMonthValueMapMap.put(year, monthValueMap);
            }
        } else {
            Map<String, Map<String, String>> yearMonthValueMapMap = new HashMap<>(16);
            Map<String, String> monthValueMap = new HashMap<>(16);
            monthValueMap.put(month, value);
            yearMonthValueMapMap.put(year, monthValueMap);
            secYearMonthValueMapMapMap.put(secId, yearMonthValueMapMap);
        }
        if (Indicator.ExcessReturn == indicator || ifExcessRet) {
            String extraValue = indicatorCalcPropertyDto.getSecData().getExtraGeoIndicatorValueMap().get(indicator.name());
            if (secYearMonthValueMapMapMap.containsKey(secId + EXTRA)) {
                Map<String, Map<String, String>> yearMonthValueMapMap = secYearMonthValueMapMapMap.get(secId + EXTRA);
                if (yearMonthValueMapMap.containsKey(year)) {
                    Map<String, String> monthValueMap = yearMonthValueMapMap.get(year);
                    monthValueMap.put(month, extraValue);
                } else {
                    Map<String, String> monthValueMap = new HashMap<>(16);
                    monthValueMap.put(month, extraValue);
                    yearMonthValueMapMap.put(year, monthValueMap);
                }
            } else {
                Map<String, Map<String, String>> yearMonthValueMapMap = new HashMap<>(16);
                Map<String, String> monthValueMap = new HashMap<>(16);
                monthValueMap.put(month, extraValue);
                yearMonthValueMapMap.put(year, monthValueMap);
                secYearMonthValueMapMapMap.put(secId + EXTRA, yearMonthValueMapMap);
            }
        }
        for (String index : indexIdList) {
            String indexValue = indicatorCalcPropertyDto.getIndexData().getIndexIndicatorValueMap().get(index).get(indicator.name());
            if (secYearMonthValueMapMapMap.containsKey(index)) {
                Map<String, Map<String, String>> yearMonthValueMapMap = secYearMonthValueMapMapMap.get(index);
                if (yearMonthValueMapMap.containsKey(year)) {
                    Map<String, String> monthValueMap = yearMonthValueMapMap.get(year);
                    monthValueMap.put(month, indexValue);
                } else {
                    Map<String, String> monthValueMap = new HashMap<>(16);
                    monthValueMap.put(month, indexValue);
                    yearMonthValueMapMap.put(year, monthValueMap);
                }
            } else {
                Map<String, Map<String, String>> yearMonthValueMapMap = new HashMap<>(16);
                Map<String, String> monthValueMap = new HashMap<>(16);
                monthValueMap.put(month, indexValue);
                yearMonthValueMapMap.put(year, monthValueMap);
                secYearMonthValueMapMapMap.put(index, yearMonthValueMapMap);
            }
        }
    }

    private static void prepareSingleYearMonthDate(String secId, List<String> indexIdList, IndicatorCalcPropertyDto indicatorCalcPropertyDto, Indicator indicator,
                                                   Map<String, Map<String, String>> secMonthValueMapMap, boolean ifExtraRet) {
        String endDate1 = indicatorCalcPropertyDto.getIndicatorCalcReq().getIndicatorCalcTimeRangeDto().getEndDate();
        Date monthDate = DateUtils.addDays(DateUtil.parse(endDate1, DateConst.YYYY_MM_DD), -5);
        String date = DateUtil.format(monthDate, DateConst.YYYY_MM_DD);
        String yearMonth = date.substring(0, 7);
        String value = indicatorCalcPropertyDto.getSecData().getIndicatorValueMap().get(indicator.name());

        if (secMonthValueMapMap.containsKey(secId)) {
            secMonthValueMapMap.get(secId).put(yearMonth, value);
        } else {
            Map<String, String> yearValueMap = new HashMap<>(16);
            yearValueMap.put(yearMonth, value);
            secMonthValueMapMap.put(secId, yearValueMap);
        }

        if (Indicator.ExcessReturn == indicator || ifExtraRet) {
            String extraValue = indicatorCalcPropertyDto.getSecData().getExtraGeoIndicatorValueMap().get(indicator.name());
            if (secMonthValueMapMap.containsKey(secId + EXTRA)) {
                secMonthValueMapMap.get(secId + EXTRA).put(yearMonth, extraValue);
            } else {
                Map<String, String> yearValueMap = new HashMap<>(16);
                yearValueMap.put(yearMonth, extraValue);
                secMonthValueMapMap.put(secId + EXTRA, yearValueMap);
            }
        }

        //指数的
        for (String indexId : indexIdList) {
            String indexValue = indicatorCalcPropertyDto.getIndexData().getIndexIndicatorValueMap().get(indexId).get(indicator.name());
            if (secMonthValueMapMap.containsKey(indexId)) {
                secMonthValueMapMap.get(indexId).put(yearMonth, indexValue);
            } else {
                Map<String, String> yearValueMap = new HashMap<>(16);
                yearValueMap.put(yearMonth, indexValue);
                secMonthValueMapMap.put(indexId, yearValueMap);
            }
        }
    }

    private static void calcIndexRet(List<String> indexIdList, List<String> monthOrWeekList, Map<String, Map<String, Map<String, String>>> secYearMonthValueMapMapMap,
                                     Map<String, Map<String, String>> secYearValueMapMap, List<HistoryRetTableSingleData> data,
                                     HistoryRetTableSingleData historyRetTableSingleData, String yearOrYearMonth, List<HistoryRetTableSingleRet> rets, Frequency rollingFrequency) {
        for (String indexId : indexIdList) {
            HistoryRetTableSingleRet indexRet = new HistoryRetTableSingleRet();
            List<String> indexRetList = new ArrayList<>();
            String yearValue = null;
            if (secYearValueMapMap.get(indexId) != null) {
                yearValue = secYearValueMapMap.get(indexId).get(yearOrYearMonth);
            }
            indexRetList.add(yearValue);
            Map<String, Map<String, String>> yearMonthValueMapMap = secYearMonthValueMapMapMap.get(indexId);
            Map<String, String> monthValueMap = Optional.ofNullable(yearMonthValueMapMap).map(e -> e.get(yearOrYearMonth)).orElse(null);
            List<String> monthList = null;
            if (monthValueMap != null) {
                monthList = new ArrayList<>(monthValueMap.keySet());
            }
            for (String monthOrWeek : monthOrWeekList) {
                String value = null;
                if (monthValueMap != null) {
                    value = monthValueMap.get(monthOrWeek);
                    if (value == null && rollingFrequency == Frequency.Quarterly) {
                        Pair<String, String> pair = MONTH_QUARTERLY_MAP.get(monthOrWeek);
                        String s = monthList.stream().filter(e -> e.compareTo(pair.getKey()) > 0).filter(e -> e.compareTo(pair.getValue()) < 0)
                                .filter(e -> e.compareTo(monthOrWeek) <= 0).max(String::compareTo).orElse(null);
                        value = s == null ? null : monthValueMap.get(s);
                    }
                }
                indexRetList.add(value);
            }
            indexRet.setId(indexId);
            indexRet.setType("ret");
            indexRet.setRet(indexRetList);
            rets.add(indexRet);
        }
        historyRetTableSingleData.setRets(rets);
        data.add(historyRetTableSingleData);
    }
}
