package com.smppw.analysis.domain.manager.performance;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.smppw.analysis.domain.dto.performance.Params;
import org.springframework.stereotype.Component;

import java.util.Map;

@Component
public class PerformanceFactory {
    private static final Performance<Params, Object> EMPTY = p -> null;
    private static final Map<String, Performance<? extends Params, ?>> PERFORMANCE_STRATEGY = MapUtil.newHashMap(16);

    public PerformanceFactory(Map<String, Performance<? extends Params, ?>> component) {
        PERFORMANCE_STRATEGY.putAll(component);
    }

    @SuppressWarnings("unchecked")
    public <P extends Params, R> Performance<P, R> getInstance(String strategyKey) {
        Performance<P, R> empty = (Performance<P, R>) EMPTY;
        if (CharSequenceUtil.isBlank(strategyKey)) {
            return empty;
        }
        Performance<? extends Params, ?> performance = PERFORMANCE_STRATEGY.get(strategyKey);
        if (performance == null) {
            return empty;
        }
        try {
            return (Performance<P, R>) performance;
        } catch (Exception e) {
            return empty;
        }
    }
}
