package com.smppw.analysis.domain.manager.performance.handler;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.performance.CorrelationParams;
import com.smppw.analysis.domain.manager.performance.AbstractSingleSecPerformance;
import com.smppw.analysis.domain.manager.performance.PerformanceConstants;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.analysis.domain.service.NavService;
import com.smppw.analysis.infrastructure.exception.APIException;
import com.smppw.common.pojo.dto.DateRange;
import com.smppw.common.pojo.dto.DateValue;
import com.smppw.common.pojo.dto.calc.IndicatorCalcPropertyDto;
import com.smppw.common.pojo.dto.calc.IndicatorCalcTimeRangeDto;
import com.smppw.common.pojo.enums.Frequency;
import com.smppw.common.pojo.enums.TimeRange;
import com.smppw.common.pojo.enums.Visibility;
import com.smppw.common.rollrange.RollingSpliter;
import com.smppw.core.IndicatorService;
import com.smppw.utils.CalcUtils;
import com.smppw.utils.NavUtil;
import com.smppw.utils.StrategyHandleUtils;
import org.springframework.stereotype.Component;

import java.util.*;

@Component(PerformanceConstants.CORRELATION)
public class CorrelationHandler extends AbstractSingleSecPerformance<CorrelationParams, Map<String, Object>> {
    private final NavService navService;

    public CorrelationHandler(BaseInfoService baseInfoService, BaseIndicatorServiceV2 baseIndicatorServiceV2, NavService navService) {
        super(baseInfoService, baseIndicatorServiceV2);
        this.navService = navService;
    }

    @Override
    protected Map<String, Object> bizSecHandle(CorrelationParams params) {
        String startDate = params.getStartDate();
        String endDate = params.getEndDate();
        List<String> refIds = params.getRefIds();
        String fundId = refIds.get(0);

        Map<String, List<DateValue>> allNavMap = this.navService.getSecIdDateValueNavListMapByDb(refIds, params.getStartDate(), params.getEndDate(), Visibility.Both, params.getNavType(), MapUtil.empty());
        List<IndicatorCalcTimeRangeDto> secTimeRanges = IndicatorService.getInstance().getSecTimeRange(fundId, params.getFrequency(), allNavMap);
        List<String> secIds = this.getSecIdsByParams(params);
        List<String> indexIds = CollectionUtil.subtractToList(secIds, refIds);
        List<DateRange> dateRangeList = new RollingSpliter(startDate, endDate, params.getRollingWindow(), params.getRollingWindow(), false, Frequency.Monthly).split();
        if (dateRangeList.isEmpty()) {
            throw new APIException("净值数量小于窗口期，请尝试调整取值区间");
        }
        Map<String, List<Double>> corrData = MapUtil.newHashMap();
        List<String> dates = ListUtil.list(true);
        List<Map<String, Object>> corRetList = ListUtil.list(false);
        List<List<Map<String, Object>>> matrixList = this.getMatrixList(params, dateRangeList, dates, corrData, corRetList, allNavMap);
        List<Map<String, Object>> dataList = ListUtil.list(true);
        for (int i = 0; i < dates.size(); i++) {
            Object values = matrixList.get(i).get(0).get("values");
            dataList.add(MapUtil.<String, Object>builder("date", dates.get(i)).put("matrix", values).build());
        }
        Map<String, Object> dataset = MapUtil.<String, Object>builder(fundId, dataList).build();
        for (String indexId : indexIds) {
            List<Map<String, Object>> indexDataList = ListUtil.list(true);
            for (int i = 0; i < dates.size(); i++) {
                Object values = matrixList.get(i).stream().filter(e -> Objects.equals(indexId, e.get("id"))).findFirst()
                        .map(e -> e.get("values")).orElse(null);
                indexDataList.add(MapUtil.<String, Object>builder("date", dates.get(i))
                        .put("value", corrData.get(indexId).get(i)).put("matrix", values).build());
            }
            dataset.put(indexId, indexDataList);
        }
        String endDate1 = null;
        Map<String, String> timeRangeMap = MapUtil.newHashMap();
        for (IndicatorCalcTimeRangeDto timeRange : secTimeRanges) {
            if (StrUtil.isBlank(endDate1)) {
                endDate1 = timeRange.getEndDate();
            }
            timeRangeMap.put(timeRange.getTimeRange().name(), timeRange.getStartDate());
        }
        List<Map<String, Object>> resultList = ListUtil.list(true);
        Map<TimeRange, String> yearOptions = TimeRange.COR_DATES;
        yearOptions.forEach((k, v) -> {
            Map<String, Object> result = MapUtil.newHashMap();
            result.put("id", v);
            result.put("range", k);
            result.put("startDate", timeRangeMap.get(k.name()) == null ? null : timeRangeMap.get(k.name()));
            resultList.add(result);
        });
        List<Map<String, Object>> dataset2 = getDataset(params, refIds, indexIds, corRetList, endDate1, resultList);
        Map<String, String> productNameMapping = this.baseInfoService.querySecName(secIds);
        Map<String, Object> extInfos = MapUtil.<String, Object>builder().put("table", dataset2).build();
        return MapUtil.<String, Object>builder().put("dataset", dataset).put("productNameMapping", productNameMapping).putAll(extInfos).build();
    }

    private List<List<Map<String, Object>>> getMatrixList(CorrelationParams params, List<DateRange> dateRangeList, List<String> dates, Map<String,
            List<Double>> corrData, List<Map<String, Object>> corRetList, Map<String, List<DateValue>> allNavMap) {
        List<List<Map<String, Object>>> matrixList = ListUtil.list(false);
        List<String> refIds = params.getRefIds();
        String fundId = refIds.get(0);
        List<String> secIds = this.getSecIdsByParams(params);
        List<String> indexIds = CollectionUtil.subtractToList(secIds, refIds);
        List<IndicatorCalcTimeRangeDto> secTimeRanges = IndicatorService.getInstance().getSecTimeRange(fundId, params.getFrequency(), allNavMap);
        String incepStart = null;
        String incepEnd = null;
        if (CollUtil.isNotEmpty(secTimeRanges)) {
            for (IndicatorCalcTimeRangeDto secTimeRange : secTimeRanges) {
                if (TimeRange.FromSetup == secTimeRange.getTimeRange()) {
                    incepStart = secTimeRange.getStartDate();
                    incepEnd = secTimeRange.getEndDate();
                }
            }
        }
        Frequency frequency = params.getFrequency();
        if (frequency == null || frequency == Frequency.Default) {
            frequency = Frequency.Daily;
        }
        Map<String, List<IndicatorCalcPropertyDto>> multiSecRetListNew = this.baseIndicatorServiceV2.getMultiSecRetListNew(
                secIds, ListUtil.empty(), frequency, Frequency.Daily, incepStart, incepEnd, false, params.getBenchmarkId(),
                params.getRaiseType(), StrategyHandleUtils.getStrategy(params.getStrategy()), Visibility.Visible, params.getNavType(), allNavMap);
        if (CollectionUtil.isEmpty(indexIds)) {
            throw new APIException("指数不能为空");
        }
        List<Map<String, Object>> mapList = NavUtil.handleNextStepNew(secIds, indexIds, multiSecRetListNew, false);
        corRetList.addAll(mapList);
        for (DateRange dr : dateRangeList) {
            String start = dr.getStartDate();
            String end = dr.getEndDate();
            dates.add(end);
            Map<String, Map<String, Double>> retMap = findRetData(corRetList, start, end);
            Double[][] correlationMatrix = CalcUtils.getCorrelationMatrix(retMap, refIds, indexIds);
            Double[] values = correlationMatrix[0];
            for (int i = 0; i < values.length; i++) {
                String idx = indexIds.get(i);
                List<Double> cList = corrData.getOrDefault(idx, new ArrayList<>());
                cList.add(values[i]);
                corrData.put(idx, cList);
            }
            // 相关性矩�
            Double[][] matrix = CalcUtils.getCorrelationMatrix(retMap, secIds, secIds);
            List<Map<String, Object>> rowList = new ArrayList<>(matrix.length);
            for (int i = 0; i < matrix.length; i++) {
                rowList.add(MapUtil.<String, Object>builder("id", secIds.get(i)).put("values", matrix[i]).build());
            }
            matrixList.add(rowList);
        }
        return matrixList;
    }

    private List<Map<String, Object>> getDataset(CorrelationParams params, List<String> refIds, List<String> indexIds, List<Map<String, Object>> corRetList,
                                                 String endDate1, List<Map<String, Object>> resultList) {
        // 除了成立以来  今年以来 其他都需要加上过滤条�只要开始日期大于等�前端传参的开始日�
        List<Map<String, Object>> dataset2 = ListUtil.list(true);
        for (Map<String, Object> timeRange : resultList) {
            Map<String, Object> temp = MapUtil.newHashMap();
            String start = MapUtil.getStr(timeRange, "startDate");
            if (StrUtil.isBlank(start)) {
                continue;
            }
            TimeRange range = (TimeRange) timeRange.get("range");
            boolean isTime = range != TimeRange.FromSetup && range != TimeRange.FromThisYear;
            if (isTime && params.getStartDate().compareTo(start) > 0) {
                continue;
            }

            Map<String, Map<String, Double>> retMap = this.findRetData(corRetList, start, endDate1);
            Double[][] correlationMatrix = CalcUtils.getCorrelationMatrix(retMap, refIds, indexIds);
            Double[] values = correlationMatrix[0];
            temp.put("timeRange", timeRange.get("id"));
            for (int i = 0; i < indexIds.size(); i++) {
                temp.put(indexIds.get(i), values[i]);
            }
            dataset2.add(temp);
        }
        return dataset2;
    }

    private Map<String, Map<String, Double>> findRetData(List<Map<String, Object>> dataList, String start, String end) {
        Map<String, Map<String, Double>> map = MapUtil.newHashMap();
        for (Map<String, Object> m : dataList) {
            if (null == m) {
                continue;
            }
            String endDate = m.get("end_date") + "";
            if (start.compareTo(endDate) <= 0 && end.compareTo(endDate) >= 0) {
                String fundId = m.get("fund_id") + "";
                Object retRange = m.get("ret_range");
                Double ret = null;
                if (null != retRange) {
                    try {
                        ret = Double.parseDouble(retRange + "");
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                } else {
                    continue;
                }
                Map<String, Double> dataMap = map.getOrDefault(fundId, new LinkedHashMap<>());
                dataMap.put(endDate, ret);
                map.put(fundId, dataMap);
            }
        }
        return map;
    }
}
