package com.smppw.analysis.domain.manager.performance.handler;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import com.smppw.analysis.domain.dataobject.MonetaryFundProfitDO;
import com.smppw.analysis.domain.dto.performance.TrendParams;
import com.smppw.analysis.domain.manager.performance.PerformanceConstants;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.BaseInfoService;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.Optional;

@Component(PerformanceConstants.IMF)
public class ImfTrendHandler extends TrendHandler {
    public ImfTrendHandler(BaseInfoService baseInfoService, BaseIndicatorServiceV2 baseIndicatorServiceV2) {
        super(baseInfoService, baseIndicatorServiceV2);
    }

    @Override
    @SuppressWarnings("unchecked")
    protected Map<String, Object> bizSecHandle(TrendParams params) {
        String fundId = params.getRefIds().get(0);
        Map<String, Object> result = super.bizSecHandle(params);
        Map<String, List<Map<String, Object>>> dataset = MapUtil.get(result, "dataset", Map.class);
        List<MonetaryFundProfitDO> doList = this.baseInfoService.queryMonetaryFund(fundId);
        dataset.forEach((k, v) -> {
            if (fundId.equals(k)) {
                for (Map<String, Object> map : v) {
                    String date = MapUtil.getStr(map, "date");
                    MonetaryFundProfitDO aDo = doList.stream().filter(e -> date.equals(DateUtil.formatDate(e.getPriceDate()))).findFirst().orElse(null);
                    map.put("profit", Optional.ofNullable(aDo).map(MonetaryFundProfitDO::getProfitPerMillion).orElse(null));
                    map.put("serven", Optional.ofNullable(aDo).map(MonetaryFundProfitDO::getProfitServenDayAnnual).orElse(null));
                }
            }
        });
        result.put("dataset", dataset);
        return result;
    }
}
