package com.smppw.analysis.domain.manager.performance.handler;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.smppw.analysis.domain.dto.performance.IntervalParams;
import com.smppw.analysis.domain.manager.performance.AbstractSingleSecPerformance;
import com.smppw.analysis.domain.manager.performance.PerformanceConstants;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.common.pojo.dto.calc.IndicatorCalcIndexDataDto;
import com.smppw.common.pojo.dto.calc.IndicatorCalcPropertyDto;
import com.smppw.common.pojo.dto.calc.IndicatorCalcReqPropertyDto;
import com.smppw.common.pojo.dto.calc.IndicatorCalcTimeRangeDto;
import com.smppw.common.pojo.dto.indicator.CalcMultipleSecMultipleTimeRangeIndicatorReq;
import com.smppw.common.pojo.enums.DateIntervalType;
import com.smppw.common.pojo.enums.Indicator;
import com.smppw.common.pojo.enums.TimeRange;
import com.smppw.constants.Consts;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.Optional;

@Component(PerformanceConstants.INTERVAL)
public class IntervalHandler extends AbstractSingleSecPerformance<IntervalParams, Map<String, Object>> {
    public IntervalHandler(BaseInfoService baseInfoService, BaseIndicatorServiceV2 baseIndicatorServiceV2) {
        super(baseInfoService, baseIndicatorServiceV2);
    }

    @Override
    protected CalcMultipleSecMultipleTimeRangeIndicatorReq buildCalcReq(IntervalParams params, DateIntervalType dateIntervalType) {
        CalcMultipleSecMultipleTimeRangeIndicatorReq req = super.buildCalcReq(params, dateIntervalType);
        Indicator indicator = params.getIndicator();
        List<Indicator> indicators = ListUtil.toList(Indicator.MaxDrawdown);
        List<Indicator> geoIndicators = ListUtil.list(false);
        boolean excess = params.getExcess() != null && params.getExcess();
        if (excess) {
            geoIndicators.add(indicator);
        } else {
            indicators.add(indicator);
        }
        req.setIndicatorList(indicators);
        req.setGeoExtraindicatorList(geoIndicators);
        req.setRiskOfFreeId(Consts.RISK_OF_FREE);
        return req;
    }

    @Override
    protected Map<String, Object> bizSecHandle(IntervalParams params) {
        List<String> refIds = params.getRefIds();
        List<String> secIds = this.getSecIdsByParams(params);
        List<String> indexIds = CollectionUtil.subtractToList(secIds, refIds);
        Indicator indicator = params.getIndicator();
        boolean excess = params.getExcess() != null && params.getExcess();
        CalcMultipleSecMultipleTimeRangeIndicatorReq req = this.buildCalcReq(params, DateIntervalType.DefaultInterval);
        Map<String, List<IndicatorCalcPropertyDto>> indicatorMap = this.baseIndicatorServiceV2.calcMultipleSecMultipleTimeRangeIndicator(req);
        Map<TimeRange, Boolean> timeRangeFlag = MapUtil.newHashMap();
        Map<String, Object> valuesMap = MapUtil.newHashMap(true);
        Map<String, Object> indexValuesMap = MapUtil.newHashMap(true);
        for (String refId : refIds) {
            List<IndicatorCalcPropertyDto> dtos = Optional.ofNullable(indicatorMap.get(refId)).filter(e -> !e.isEmpty()).orElse(ListUtil.empty());
            // 当前基金的数据处�
            List<Map<String, Object>> fundIndicatorList = ListUtil.list(true);
            for (IndicatorCalcPropertyDto dto : dtos) {
                TimeRange timeRange = Optional.ofNullable(dto.getIndicatorCalcReq()).map(IndicatorCalcReqPropertyDto::getIndicatorCalcTimeRangeDto)
                        .map(IndicatorCalcTimeRangeDto::getTimeRange).orElse(null);
                if (timeRange == null) {
                    continue;
                }
                if (params.getTimeRange() != null) {
                    String start = Optional.ofNullable(dto.getIndicatorCalcReq()).map(IndicatorCalcReqPropertyDto::getIndicatorCalcTimeRangeDto).map(IndicatorCalcTimeRangeDto::getStartDate).orElse(null);
                    boolean isFrom = timeRange != TimeRange.FromSetup && timeRange != TimeRange.FromThisYear;
                    boolean isDate = CharSequenceUtil.compare(params.getStartDate(), start, true) > 0;
                    if (isFrom && isDate) {
                        timeRangeFlag.put(timeRange, true);
                        continue;
                    }
                }
                String value = Optional.ofNullable(dto.getSecData()).map(e -> excess ? e.getExtraGeoIndicatorValueMap() : e.getIndicatorValueMap())
                        .map(e -> e.get(indicator.name())).orElse(null);
                fundIndicatorList.add(MapUtil.<String, Object>builder("date", handleStrDate(timeRange.name(), null)).put("value", value).build());
            }
            valuesMap.put(refId, fundIndicatorList);
            // 指数的数据处�
            for (String indexId : indexIds) {
                boolean excessBench = (excess && indexId.equals(params.getBenchmarkId()));
                if (indexValuesMap.containsKey(indexId) || excessBench) {
                    continue;
                }
                List<Map<String, Object>> indexIndicatorList = ListUtil.list(true);
                for (IndicatorCalcPropertyDto dto : dtos) {
                    TimeRange timeRange = Optional.ofNullable(dto.getIndicatorCalcReq()).map(IndicatorCalcReqPropertyDto::getIndicatorCalcTimeRangeDto)
                            .map(IndicatorCalcTimeRangeDto::getTimeRange).orElse(null);
                    if (timeRange == null || timeRangeFlag.getOrDefault(timeRange, false)) {
                        continue;
                    }
                    String value = Optional.ofNullable(dto.getIndexData()).map(IndicatorCalcIndexDataDto::getIndexIndicatorValueMap).map(e -> e.get(indexId)).map(e -> e.get(indicator.name())).orElse(null);
                    indexIndicatorList.add(MapUtil.<String, Object>builder("date", handleStrDate(timeRange.name(), null)).put("value", value).build());
                }
                indexValuesMap.put(indexId, indexIndicatorList);
            }
        }
        Map<String, Object> dataset = MapUtil.<String, Object>builder().putAll(valuesMap).putAll(indexValuesMap).build();

        Map<String, String> productNameMapping = this.baseInfoService.querySecName(secIds);
        if (excess) {
            for (String indexId : indexIds) {
                productNameMapping.remove(indexId);
            }
        }
        Map<String, Object> extInfos = MapUtil.<String, Object>builder("endDate", params.getEndDate()).put("startDate", params.getStartDate()).build();
        return MapUtil.<String, Object>builder().put("dataset", dataset).put("productNameMapping", productNameMapping).putAll(extInfos).build();
    }
}
