package com.smppw.analysis.domain.manager.performance.handler;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.performance.RankParams;
import com.smppw.analysis.domain.manager.performance.AbstractPerformance;
import com.smppw.analysis.domain.manager.performance.PerformanceConstants;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.common.pojo.IStrategy;
import com.smppw.utils.StrategyHandleUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@Component(PerformanceConstants.RANK)
public class RankHandler extends AbstractPerformance<RankParams, Map<String, Object>> {
    public RankHandler(BaseInfoService baseInfoService, BaseIndicatorServiceV2 baseIndicatorServiceV2) {
        super(baseInfoService, baseIndicatorServiceV2);
    }

    @Override
    protected Map<String, Object> bizHandle(RankParams params) {
        String fundId = params.getRefIds().get(0);
        List<String> secIds = params.getRefIds();
        CollUtil.addAllIfNotContains(secIds, ListUtil.of(params.getBenchmarkId()));
        List<String> indexIds = params.getSecIds();
        CollUtil.addAllIfNotContains(secIds, indexIds);
        IStrategy strategy = StrategyHandleUtils.getStrategy(params.getStrategy());
        String rankDate = this.baseInfoService.getLatestRankRat();
        List<Map<String, Object>> tempList = this.baseInfoService.getFundRank(rankDate, fundId, indexIds, params.getIndicator());
        Map<String, String> mapper = this.baseInfoService.querySecName(secIds);
        Map<String, String> productMapper = MapUtil.<String, String>builder(MapUtil.newHashMap(true)).putAll(mapper)
                .put("rank", "同策略排名")
                .put("strategy_avg", "同策略平均")
                .put("strategy_avg_99", "同策略平均(1%-99%)")
                .put("strategy_best", "同策略最好")
                .put("strategy_worst", "同策略最差")
                .put("percent5", "5%分位数")
                .put("percent25", "25%分位数")
                .put("percent50", "50%分位数")
                .put("percent75", "75%分位数")
                .put("percent95", "95%分位数").build();
        Map<String, Object> extInfos = MapUtil.<String, Object>builder("strategy", strategy.getStrategyNameDesc()).build();
        List<Map<String, Object>> dataset = this.getRankDataset(tempList);
        return MapUtil.<String, Object>builder().put("dataset", dataset).put("productNameMapping", productMapper).putAll(extInfos).build();
    }

    private List<Map<String, Object>> getRankDataset(List<Map<String, Object>> tempList) {
        String rankFiled = "fund_rank";
        Map<String, Map<String, Object>> resMap = new LinkedHashMap<>();
        if (CollUtil.isEmpty(tempList)) {
            return CollUtil.newArrayList();
        }
        for (Map<String, Object> unit : tempList) {
            String classification = MapUtil.getStr(unit, "classification");
            String name = MapUtil.getStr(unit, "name");
            if (name == null && classification == null) {
                continue;
            }
            List<String> keys = new ArrayList<>(unit.keySet());
            keys.remove("name");
            keys.remove("classification");
            for (String key : keys) {
                if (!resMap.containsKey(key)) {
                    resMap.put(key, MapUtil.newHashMap(false));
                }
                if (classification.startsWith("strategy") || classification.startsWith("percen") || classification.equals(rankFiled)) {
                    name = classification;
                }
                resMap.get(key).put(name, unit.get(key));
            }
        }
        List<Map<String, Object>> dataset = ListUtil.list(false);
        resMap.forEach((k, v) -> dataset.add(MapUtil.<String, Object>builder("date", k).putAll(v).build()));
        dataset.forEach(e -> {
            String rank = MapUtil.getStr(e, rankFiled);
            String rankCnt = MapUtil.getStr(e, "strategy_rank_cnt");
            String strRank = null;
            if (CharSequenceUtil.isAllNotBlank(rank, rankCnt)) {
                strRank = StrUtil.subBefore(rank, ".", false) + "/"
                        + StrUtil.subBefore(rankCnt, ".", false);
            }
            e.put("rank", strRank);
            e.remove(rankFiled);
            e.remove("strategy_rank_cnt");
        });
        dataset.sort((o1, o2) -> MapUtil.getStr(o2, "date").compareTo(MapUtil.getStr(o1, "date")));
        return dataset;
    }
}
