package com.smppw.analysis.domain.manager.position;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import com.smppw.analysis.domain.dto.position.BaseParams;
import com.smppw.analysis.domain.dto.position.CategoryConstraint;
import com.smppw.analysis.domain.dto.position.FundPositionBaseInfo;
import com.smppw.analysis.domain.dto.position.FundPositionDetail;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.constants.SecType;
import com.smppw.utils.BigDecimalUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/15 19:13
 * @description 非综合类的组件抽象
 */
public abstract class AbstractNonSynthesizeBizHandler<P extends BaseParams, R> extends AbstractBizHandler<P, R> {
    public AbstractNonSynthesizeBizHandler(AnalysisProperty property, CacheFactory cacheFactory, PositionLoadFactory factory) {
        super(property, cacheFactory, factory);
    }

    @Override
    protected R afterLoadData(P params, List<FundPositionBaseInfo> baseInfos, List<FundPositionDetail> positionInfos) {
        if (CollUtil.isNotEmpty(positionInfos)) {
            // 过滤掉没有识别到大类的、多空轧差后数量为null的和私募基金的subType！=20的
            positionInfos = positionInfos.stream().filter(e -> e.getAsset() != null).filter(e -> e.getNumber() != null)
                    .filter(e -> SecType.PUBLICLY_OFFERED_FUNDS.equals(e.getFundType()) || (e.getSubType() != null && e.getSubType() != 20))
                    .collect(Collectors.toList());
        }
        return this.afterNonSynthesizeLoadData(params, baseInfos, positionInfos);
    }

    protected abstract R afterNonSynthesizeLoadData(P params, List<FundPositionBaseInfo> baseInfos, List<FundPositionDetail> positionInfos);

    @Deprecated
    protected CategoryConstraint calcCategoryWeight(BigDecimal total, Map<ValueLabelVO, List<Map<String, Object>>> indexStyleMap,
                                                    ValueLabelVO category, List<Map<String, Object>> categoryList) {
        // 同风格（行业、风格或流动性）指数成分权重汇总
        BigDecimal indexWeight = BigDecimal.ZERO;
        if (indexStyleMap != null && indexStyleMap.get(category) != null) {
            indexWeight = indexStyleMap.get(category).stream().map(e -> MapUtil.get(e, "weight", BigDecimal.class)).reduce(BigDecimal::add)
                    .map(e -> e.divide(new BigDecimal("100"), 18, RoundingMode.HALF_UP)).orElse(BigDecimal.ZERO);
        }
        // 同风格（行业、风格或流动性）多头市值汇总
        BigDecimal bullMv = categoryList.stream().filter(e -> Objects.equals("1", MapUtil.getStr(e, "sharesNature")))
                .map(e -> MapUtil.get(e, "marketValue", BigDecimal.class)).reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
        // 同风格（行业、风格或流动性）空头市值汇总，并取负数
        BigDecimal bearMv = categoryList.stream().filter(e -> Objects.equals("2", MapUtil.getStr(e, "sharesNature")))
                .map(e -> MapUtil.get(e, "marketValue", BigDecimal.class)).reduce(BigDecimal::add)
                .map(e -> BigDecimalUtils.toBigDecimal("-1").multiply(e)).orElse(BigDecimal.ZERO);
        CategoryConstraint temp = new CategoryConstraint();
        temp.setCategory(category);
        // 求权重
        BigDecimal bull = total == null || total.compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO : bullMv.divide(total, 6, RoundingMode.HALF_UP);
        BigDecimal bear = total == null || total.compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO : bearMv.divide(total, 6, RoundingMode.HALF_UP);
        temp.setBull(bull);
        temp.setBear(bear);
        // 多空
        temp.setPupil(bull.subtract(bear.abs()));
        temp.setBenchmark(indexWeight.setScale(6, RoundingMode.HALF_UP));
        return temp;
    }
}
