package com.smppw.analysis.domain.manager.position;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import com.smppw.analysis.domain.dto.position.BaseParams;
import com.smppw.analysis.domain.dto.position.FundPositionBaseInfo;
import com.smppw.analysis.domain.dto.position.FundPositionDetail;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.constants.SecType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @author wangzaijun
 * @date 2023/6/7 14:04
 * @description 价值持仓数据的工厂
 */
@Component
public class PositionLoadFactory {
    private static final Logger LOGGER = LoggerFactory.getLogger(PositionLoadFactory.class);
    private static final Map<String, PositionLoad> LOAD_STRATEGY = MapUtil.newHashMap(8);
    private final BaseInfoService secTypeService;

    public PositionLoadFactory(Map<String, PositionLoad> loadMap, BaseInfoService secTypeService) {
        LOAD_STRATEGY.putAll(loadMap);
        this.secTypeService = secTypeService;
    }

    /**
     * 提供一个根据基金id获取持仓数据加载接口的方法，没有找到的话返回一个空数据的策略接口并记录日志
     * 仅支持私有基金和公募基金，不支持私募基金了
     *
     * @param fundId 基金id
     * @return 数据加载策略
     */
    public PositionLoad getPositionLoadInstance(String fundId) {
        List<String> supportTypes = ListUtil.of(SecType.PUBLICLY_OFFERED_FUNDS, SecType.PRIVATE_FUND);
        String secType = this.secTypeService.getSecType(fundId);
        if (secType == null || !supportTypes.contains(secType)) {
            LOGGER.warn(String.format("secType[%s-%s] not find positionLoad strategy.", fundId, secType));
            return new PositionLoad() {
                @Override
                public <P extends BaseParams> List<FundPositionBaseInfo> getFundPositionBaseInfo(P params) {
                    return ListUtil.empty();
                }
                @Override
                public <P extends BaseParams> List<FundPositionDetail> loadPositionDetail(P params) {
                    return ListUtil.empty();
                }
            };
        } else {
            return LOAD_STRATEGY.get(secType);
        }
    }
}
