package com.smppw.analysis.domain.manager.position.bond;

import cn.hutool.core.collection.ListUtil;
import com.smppw.analysis.domain.dto.position.AssetCategoryEnum;
import com.smppw.analysis.domain.dto.position.FundPositionBaseInfo;
import com.smppw.analysis.domain.dto.position.FundPositionDetail;
import com.smppw.analysis.domain.dto.position.RefMarketValueRatio;
import com.smppw.analysis.domain.dto.position.bond.BondSortAllocationParam;
import com.smppw.analysis.domain.dto.position.stock.ConcentrationVO;
import com.smppw.analysis.domain.manager.position.AbstractNonSynthesizeBizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerConstants;
import com.smppw.analysis.domain.manager.position.PositionLoadFactory;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/8 14:07
 * @description 股票集中度业务数据转换
 */
@Component(BizHandlerConstants.BOND_CONCENTRATION)
public class ConcentrationBizBondHandler extends AbstractNonSynthesizeBizHandler<BondSortAllocationParam, List<ConcentrationVO>> {
    public ConcentrationBizBondHandler(AnalysisProperty property, CacheFactory cacheFactory, PositionLoadFactory factory) {
        super(property, cacheFactory, factory);
    }

    @Override
    protected List<ConcentrationVO> afterNonSynthesizeLoadData(BondSortAllocationParam params, List<FundPositionBaseInfo> baseInfos, List<FundPositionDetail> positionInfos) {
        String fundId = params.getFundId();
        if(StringUtils.isNotBlank(fundId)){
            if(fundId.startsWith("MF")){
                positionInfos = positionInfos.stream().filter(e -> AssetCategoryEnum.BOND.name().equals(e.getAsset().getValue()))
                        .collect(Collectors.toList());
            }else {
                positionInfos = positionInfos.stream().filter(e -> AssetCategoryEnum.BOND.name().equals(e.getAsset().getValue()))
                        .filter(p->Objects.nonNull(p.getLevel()))
                        .filter(p->p.getLevel()>=4)
                        .collect(Collectors.toList());
            }
        }

        List<ConcentrationVO> resultList = ListUtil.list(false);
        // 按日期分组处理
        Map<String, List<FundPositionDetail>> collect = positionInfos.stream().collect(Collectors.groupingBy(FundPositionDetail::getDate));
        collect.forEach((k, v) -> {
            // 基金在当前日期的净资产值
            BigDecimal bigDecimal = baseInfos.stream().filter(e -> k.equals(e.getDate())).findFirst().map(FundPositionBaseInfo::getAssetNv).orElse(null);
            // 得到每个日期下的所有债券成分，按占比降序
            List<RefMarketValueRatio> position = v.stream().map(e -> {
                BigDecimal ratioNv = e.getRatioNv();
                if (ratioNv == null) {
                    return null;
                }
                return new RefMarketValueRatio(e.getRef(), e.getMarketValue(), ratioNv, e.getSecId());
            }).filter(Objects::nonNull).sorted((o1, o2) -> o2.getRatio().compareTo(o1.getRatio())).collect(Collectors.toList());
            ConcentrationVO vo = new ConcentrationVO();
            vo.setDate(k);
            vo.setAssetNv(bigDecimal);
            vo.setPosition(position);
            resultList.add(vo);
        });
        // 按日期升序
        resultList.sort(Comparator.comparing(ConcentrationVO::getDate));
        return resultList;
    }
}