package com.smppw.analysis.domain.manager.position.future;


import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import com.smppw.analysis.domain.dataobject.FundPositionDetailDO;
import com.smppw.analysis.domain.mapper.BaseUnderlyingMapper;
import com.smppw.analysis.domain.mapper.PubliclyFundExtInfoMapper;
import com.smppw.common.pojo.FundFuturesOption;
import com.smppw.common.pojo.ValueLabelVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * FileName: FundFuturesOptionBaseService
 * Author:   chenjianhua
 * Date:     2023/6/12 17:36
 * Description: ${DESCRIPTION}
 */
@Service
public class FundFuturesOptionBaseService {

    private static final Logger logger = LoggerFactory.getLogger(FundFuturesOptionBaseService.class);
    private static Map<String, String> bondNameMapping = new HashMap<>();

    static {
        bondNameMapping.put("票面利率3%的2年期名义中短期国债", "2年期国债");
        bondNameMapping.put("票面利率3%的5年期名义中短期国债", "5年期国债");
        bondNameMapping.put("票面利率3%的名义超长期国债", "30年期国债");
        bondNameMapping.put("票面利率3%的名义长期国债", "10年期国债");
    }

    private final BaseUnderlyingMapper baseUnderlyingMapper;
    private final PubliclyFundExtInfoMapper publiclyFundExtInfoMapper;


    public FundFuturesOptionBaseService(BaseUnderlyingMapper baseUnderlyingMapper, PubliclyFundExtInfoMapper publiclyFundExtInfoMapper) {
        this.baseUnderlyingMapper = baseUnderlyingMapper;
        this.publiclyFundExtInfoMapper = publiclyFundExtInfoMapper;
    }

    public Map<String, Object> fundFuturesOption(Integer userId, String fundId, Integer optionType, Integer shType, String startDate, String endDate) {
        //套期：【二级科目】3201、3202以下“套期”下合约资产归属
        //投机：【二级科目】3101、3102、3103以下“投机”字段下合约资产归属
        Map<String, Object> result = new HashMap<>();
        Map<String, List<Map<String, Object>>> optionData = new LinkedHashMap<>();
        optionData.put(FundFuturesOption.Bond.getNameEnDesc(), new ArrayList<>());
        optionData.put(FundFuturesOption.StockIndex.getNameEnDesc(), new ArrayList<>());
        optionData.put(FundFuturesOption.Product.getNameEnDesc(), new ArrayList<>());
        optionData.put(FundFuturesOption.Future.getNameEnDesc(), new ArrayList<>());
        optionData.put(FundFuturesOption.Other.getNameEnDesc(), new ArrayList<>());
        Map<String, String> optionName = new HashMap<>();
        optionName.put(FundFuturesOption.Bond.getNameEnDesc(), FundFuturesOption.Bond.getNameDesc());
        optionName.put(FundFuturesOption.StockIndex.getNameEnDesc(), FundFuturesOption.StockIndex.getNameDesc());
        optionName.put(FundFuturesOption.Product.getNameEnDesc(), FundFuturesOption.Product.getNameDesc());
        optionName.put(FundFuturesOption.Future.getNameEnDesc(), FundFuturesOption.Future.getNameDesc());
        optionName.put(FundFuturesOption.Other.getNameEnDesc(), FundFuturesOption.Other.getNameDesc());
        List<Map<String, Object>> tableList = new ArrayList<>();
        if (fundId.startsWith("MF")) {
            List<FundPositionDetailDO> fundPositionDetailDOS = this.publiclyFundExtInfoMapper.getMFFundPositionDetail(fundId, startDate, endDate);
            List<Map<String, Object>> nvListMap = this.publiclyFundExtInfoMapper.getValuationDateNv(fundId, startDate, endDate);
            //按照时间分组
            Map<String, List<FundPositionDetailDO>> fundPositionDateMap = new LinkedHashMap<>();
            Map<String, BigDecimal> nvDateMap = new LinkedHashMap<>();
            for (FundPositionDetailDO detail : fundPositionDetailDOS) {
                String valuationDate = DateUtil.formatDate(detail.getValuationDate());
                if (fundPositionDateMap.containsKey(valuationDate)) {
                    List<FundPositionDetailDO> details = fundPositionDateMap.get(valuationDate);
                    details.add(detail);
                    fundPositionDateMap.put(valuationDate, details);
                } else {
                    List<FundPositionDetailDO> details = new ArrayList<>();
                    details.add(detail);
                    fundPositionDateMap.put(valuationDate, details);
                }
            }
            //基金净资产按时间分类
            for (Map<String, Object> nvMap : nvListMap) {
                nvDateMap.put(nvMap.get("valuationDate").toString(), (BigDecimal) nvMap.get("nv"));
            }
            //针对每个估值日期做数据处理
            for (String valuationDate : fundPositionDateMap.keySet()) {
                List<FundPositionDetailDO> details = fundPositionDateMap.get(valuationDate);
                if (nvDateMap.get(valuationDate) == null) {  //净资产为null，直接跳过
                    continue;
                }
                BigDecimal nv = nvDateMap.get(valuationDate);
                for (FundPositionDetailDO detail : details) {
                    String securitiesCode = detail.getSecuritiesCode();
                    String securitiesName = detail.getSecuritiesName();
                    if ((securitiesCode.length() >= 1 && securitiesCode.substring(0, 1).matches("[A-Za-z]"))) { //前两位字母
                        Map<String, String> optionTypeMap = null;
                        //根据前两位字母查询所属品种
                        String tradingCode = securitiesCode.substring(0, 2);
                        if ((securitiesCode.length() >= 2 && securitiesCode.substring(0, 2).matches("[A-Za-z]+"))) {
                            optionTypeMap = this.baseUnderlyingMapper.getOptionTypeMap(tradingCode);
                        } else {
                            optionTypeMap = this.baseUnderlyingMapper.getOptionTypeMap(tradingCode.substring(0, 1));
                        }
                        if (optionTypeMap == null || optionTypeMap.keySet().size() == 0 && (FundFuturesOption.getFundFuturesOption(optionType).getId() == 0 || optionType == 5)) {
                            String key = FundFuturesOption.Other.getNameEnDesc();
                            optionData = wrapperDetailData(optionData, detail, key, nv);
                            tableList.addAll(detailToMFMap(detail, key, securitiesName));
                            continue;
                        }
                        String firstOptionType = optionTypeMap.get("first_option_type");
                        if (FundFuturesOption.getFundFuturesOption(optionType).getNameDesc().equals(firstOptionType) || FundFuturesOption.getFundFuturesOption(optionType).getId() == 0) {
                            String firstTypeKey = FundFuturesOption.getFundFuturesOptionByName(firstOptionType).getNameEnDesc();
                            optionData = wrapperDetailData(optionData, detail, firstTypeKey, nv);
                            tableList.addAll(detailToMFMap(detail, firstTypeKey, securitiesName));
                        }
                    } else {
                        //期权
                        Map<String, String> dataMap = this.publiclyFundExtInfoMapper.getOptionContract(securitiesCode, securitiesName);
                        if (dataMap == null || dataMap.keySet().size() == 0) {
                            List<Map<String, String>> futureMapList = this.publiclyFundExtInfoMapper.getFundDetailForFuture(securitiesCode.substring(0, 4), fundId, valuationDate);
                            for (Map<String, String> futureMap : futureMapList) {
                                if (futureMap.get("securities_name") != null && futureMap.get("securities_name").contains("期权")) {
                                    dataMap = new HashMap<>();
                                    dataMap.put(futureMap.get("securities_code"), futureMap.get("securities_name"));
                                    break;
                                }
                            }
                        }
                        if ((dataMap == null || dataMap.keySet().size() == 0) && (FundFuturesOption.getFundFuturesOption(optionType).getId() == 0 || optionType == 5) && securitiesCode.length() != 4) {
                            //属于其他衍生品
                            String key = FundFuturesOption.Other.getNameEnDesc();
                            optionData = wrapperDetailData(optionData, detail, key, nv);
                            tableList.addAll(detailToMFMap(detail, key, securitiesName));
                            continue;
                        }
                        if (dataMap == null || dataMap.keySet().size() == 0) {
                            continue;
                        }
                        String firstOptionType = FundFuturesOption.Future.getNameDesc();
                        if (FundFuturesOption.getFundFuturesOption(optionType).getNameDesc().equals(firstOptionType) || FundFuturesOption.getFundFuturesOption(optionType).getId() == 0) {
                            String firstTypeKey = FundFuturesOption.getFundFuturesOptionByName(firstOptionType).getNameEnDesc();
                            optionData = wrapperDetailData(optionData, detail, firstTypeKey, nv);
                            tableList.addAll(detailToMFMap(detail, firstTypeKey, securitiesName));
                        }
                    }
                }

            }
        }
        //合并相同日期的数据
        Map<String, Object> data = new HashMap<>();
        for (String key : optionData.keySet()) {
            List<Map<String, Object>> optionList = optionData.get(key);
            if (optionList.size() < 1) {
                continue;
            }
            Map<String, Map<String, Object>> dateMap = new HashMap<>();
            for (Map<String, Object> dataMap : optionList) {
                String date = dataMap.get("date").toString();
                if (dateMap.containsKey(date)) {
                    Map<String, Object> map = dateMap.get(date);
                    map.put("ratio", ((BigDecimal) map.get("ratio")).add((BigDecimal) dataMap.get("ratio")));
                    map.put("type", FundFuturesOption.getFundFuturesOptionByEnName(key).getNameEnDesc());
                    dateMap.put(date, map);
                } else {
                    dateMap.put(date, dataMap);
                }
            }
            List<Map<String, Object>> dataList = dateMap.values().stream().sorted(Comparator.comparing(o -> MapUtil.getStr(o, "date"))).collect(Collectors.toList());
            data.put(key, dataList);
        }
        Map<String, List<Map<String, Object>>> tableListMap = new LinkedHashMap<>();
        for (Map<String, Object> tableMap : tableList) {
            String date = tableMap.get("valuationDate").toString();
            if (tableListMap.containsKey(date)) {
                List<Map<String, Object>> detaillist = tableListMap.get(date);
                detaillist.add(tableMap);
                tableListMap.put(date, detaillist);
            } else {
                List<Map<String, Object>> detaillist = new ArrayList<>();
                detaillist.add(tableMap);
                tableListMap.put(date, detaillist);
            }
        }
        for (String valuationDate : tableListMap.keySet()) {
            List<Map<String, Object>> tableListOrder = tableListMap.get(valuationDate);
            tableListOrder = tableListOrder.stream().sorted(Comparator.comparing(o -> (o.get("breedName")).toString())).collect(Collectors.toList());
            tableListMap.put(valuationDate, tableListOrder);
        }
        result.put("productNameMapping", optionName);
        result.put("dataset", data);
        result.put("table", tableListMap);
        return result;
    }

    private Map<String, List<Map<String, Object>>> wrapperDetailData(Map<String, List<Map<String, Object>>> optionData, FundPositionDetailDO detail, String key, BigDecimal nv) {
        if (optionData.containsKey(key)) {
            List<Map<String, Object>> options = optionData.get(key);
            Map<String, Object> data = new HashMap<>();
            data.put("type", key);
            BigDecimal marketValue = detail.getMarketValue();
            BigDecimal airdropMarketValue = detail.getAirdropMarketValue();
            if (airdropMarketValue == null && detail.getSharesNature() == 1) {
                data.put("ratio", marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setMarketValueRatio(marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setAirdropMarketValueRatio(null);
            } else if (airdropMarketValue == null && detail.getSharesNature() == 2) {
                data.put("ratio", marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setAirdropMarketValueRatio(marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setMarketValueRatio(marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
            } else {
                data.put("ratio", (marketValue.add(airdropMarketValue)).divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setMarketValueRatio((marketValue.add(airdropMarketValue)).divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setAirdropMarketValueRatio(airdropMarketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
            }
            data.put("date", DateUtil.formatDate(detail.getValuationDate()));
            data.put("securitiesCode", detail.getSecuritiesCode());
            options.add(data);
            optionData.put(key, options);
        } else {
            List<Map<String, Object>> options = new ArrayList<>();
            Map<String, Object> data = new HashMap<>();
            data.put("type", key);
            BigDecimal marketValue = detail.getMarketValue();
            BigDecimal airdropMarketValue = detail.getAirdropMarketValue();
            if (airdropMarketValue == null && detail.getSharesNature() == 1) {
                data.put("ratio", marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setMarketValueRatio(marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setAirdropMarketValueRatio(null);
            } else if (airdropMarketValue == null && detail.getSharesNature() == 2) {
                data.put("ratio", marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setAirdropMarketValueRatio(marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setMarketValueRatio(marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
            } else {
                data.put("ratio", (marketValue.add(airdropMarketValue)).divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setMarketValueRatio((marketValue.add(airdropMarketValue)).divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setAirdropMarketValueRatio(airdropMarketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
            }
            data.put("date", DateUtil.formatDate(detail.getValuationDate()));
            data.put("securitiesCode", detail.getSecuritiesCode());
            options.add(data);
            optionData.put(key, options);
        }
        return optionData;
    }

    private List<Map<String, Object>> detailToMFMap(FundPositionDetailDO detail, String caegoryName, String breedName) {
        List<Map<String, Object>> result = new ArrayList<>();
        Map<String, Object> tableMap = new HashMap<>();
        tableMap.put("categoryName", FundFuturesOption.getFundFuturesOptionByEnName(caegoryName).getNameDesc());
        tableMap.put("breedName", breedName);
        tableMap.put("securitiesName", detail.getSecuritiesName());
        tableMap.put("tradingDirection", detail.getSharesNature() == 1 ? "多头" : "空头");
        tableMap.put("securitiesAmount", detail.getSecuritiesAmount());
        tableMap.put("netCost", detail.getMarketValue());
        tableMap.put("marketValue", detail.getMarketValue());
        tableMap.put("marketValueRatio", detail.getMarketValueRatio());
        tableMap.put("valuationDate", DateUtil.formatDate(detail.getValuationDate()));
        result.add(tableMap);
        if (detail.getAirdropMarketValue() != null && detail.getSharesNature() != 2) {  //说明存在多空情况，此时需要多加一个数据用于后续导出
            Map<String, Object> tableMap2 = new HashMap<>();
            tableMap2.put("categoryName", FundFuturesOption.getFundFuturesOptionByEnName(caegoryName).getNameDesc());
            tableMap2.put("breedName", breedName);
            tableMap2.put("securitiesName", detail.getSecuritiesName());
            tableMap2.put("tradingDirection", "空头");
            tableMap2.put("securitiesAmount", detail.getAirdropSecuritiesAmount());
            tableMap2.put("netCost", detail.getAirdropMarketValue());
            tableMap2.put("marketValue", detail.getAirdropMarketValue());
            tableMap2.put("marketValueRatio", detail.getAirdropMarketValueRatio());
            tableMap2.put("valuationDate", DateUtil.formatDate(detail.getValuationDate()));
            result.add(tableMap2);
        }
        return result;
    }

    public Map<String, Object> fundPositionAnalysis(Integer userId, String fundId, Integer optionType, Integer shType, String startDate, String endDate) {
        //套期：【二级科目】3201、3202以下“套期”下合约资产归属
        //投机：【二级科目】3101、3102、3103以下“投机”字段下合约资产归属
        Map<String, Object> result = new HashMap<>();
        Map<String, List<Map<String, Object>>> optionData = new LinkedHashMap<>();
        optionData.put(FundFuturesOption.getFundFuturesOption(optionType).getNameEnDesc(), new ArrayList<>());
        Map<String, String> optionName = new HashMap<>();
        optionName.put(FundFuturesOption.getFundFuturesOption(optionType).getNameEnDesc() + "_long", FundFuturesOption.getFundFuturesOption(optionType).getNameDesc() + "多头占净值比");
        optionName.put(FundFuturesOption.getFundFuturesOption(optionType).getNameEnDesc() + "_short", FundFuturesOption.getFundFuturesOption(optionType).getNameDesc() + "空头占净值比");
        optionName.put(FundFuturesOption.getFundFuturesOption(optionType).getNameEnDesc() + "_netting", FundFuturesOption.getFundFuturesOption(optionType).getNameDesc() + "多空轧差");
        List<String> valuationDateList = new ArrayList<>();
        if (fundId.startsWith("MF")) {
            List<FundPositionDetailDO> fundPositionDetailDOS = this.publiclyFundExtInfoMapper.getMFFundPositionDetail(fundId, startDate, endDate);
            List<Map<String, Object>> nvListMap = this.publiclyFundExtInfoMapper.getValuationDateNv(fundId, startDate, endDate);
            //按照时间分组
            Map<String, List<FundPositionDetailDO>> fundPositionDateMap = new LinkedHashMap<>();
            Map<String, BigDecimal> nvDateMap = new LinkedHashMap<>();
            for (FundPositionDetailDO detail : fundPositionDetailDOS) {
                String valuationDate = DateUtil.formatDate(detail.getValuationDate());
                if (fundPositionDateMap.containsKey(valuationDate)) {
                    List<FundPositionDetailDO> details = fundPositionDateMap.get(valuationDate);
                    details.add(detail);
                    fundPositionDateMap.put(valuationDate, details);
                } else {
                    List<FundPositionDetailDO> details = new ArrayList<>();
                    details.add(detail);
                    fundPositionDateMap.put(valuationDate, details);
                }
            }
            //基金净资产按时间分类
            for (Map<String, Object> nvMap : nvListMap) {
                nvDateMap.put(nvMap.get("valuationDate").toString(), (BigDecimal) nvMap.get("nv"));
            }
            //针对每个估值日期做数据处理
            for (String valuationDate : fundPositionDateMap.keySet()) {
                List<FundPositionDetailDO> details = fundPositionDateMap.get(valuationDate);
                valuationDateList.add(valuationDate);
                for (FundPositionDetailDO detail : details) {
                    String securitiesCode = detail.getSecuritiesCode();
                    if (securitiesCode == null) {
                        continue;
                    }
                    BigDecimal nv = nvDateMap.get(valuationDate);
                    if ((securitiesCode.length() >= 1 && securitiesCode.substring(0, 1).matches("[A-Za-z]"))) { //前两位字母
                        List<Map<String, String>> optionTypeMapList = null;
                        //根据前两位字母查询所属品种
                        String tradingCode = securitiesCode.substring(0, 2);
                        if ((securitiesCode.length() >= 2 && securitiesCode.substring(0, 2).matches("[A-Za-z]+"))) {
                            optionTypeMapList = this.baseUnderlyingMapper.getSecondOptionTypeByCode(tradingCode);
                        } else {
                            optionTypeMapList = this.baseUnderlyingMapper.getSecondOptionTypeByCode(tradingCode.substring(0, 1));
                        }
                        if (optionTypeMapList == null || optionTypeMapList.size() == 0) {
                            continue;
                        }
                        for (Map<String, String> optionTypeMap : optionTypeMapList) {
                            String secondOptionType = optionTypeMap.get("second_option_type");
                            if (FundFuturesOption.getFundFuturesOption(optionType).getNameDesc().equals(secondOptionType) || FundFuturesOption.getFundFuturesOption(optionType).getId() == FundFuturesOption.All.getId()) {
                                String key = FundFuturesOption.getFundFuturesOptionByName(secondOptionType).getNameEnDesc();
                                optionData = wrapperFutureAnalysis(optionData, detail, key, nv);
                            }
                        }
                    }
                }
            }
        }

        Map<String, Map<String, Object>> dataMap = new LinkedHashMap<>();
        Map<String, List<Map<String, Object>>> resultData = new HashMap<>();
        for (String key : optionData.keySet()) {
            List<Map<String, Object>> options = optionData.get(key);
            if (options.size() < 1) {
                continue;
            }
            Map<String, Map<String, Object>> longDataMap = new LinkedHashMap<>();
            Map<String, Map<String, Object>> shortDataMap = new LinkedHashMap<>();
            Map<String, Map<String, Object>> nettingDataMap = new LinkedHashMap<>();
            for (Map<String, Object> option : options) {
                if (dataMap.containsKey(option.get("date").toString())) {
                    Map<String, Object> data = dataMap.get(option.get("date").toString());
                    Map<String, Object> longData = longDataMap.get(option.get("date").toString());
                    Map<String, Object> shortData = shortDataMap.get(option.get("date").toString());
                    Map<String, Object> nettingData = nettingDataMap.get(option.get("date").toString());
                    if (option.get("longShort").equals(1) && option.get("airdropMarketValueRatio") != null) {
                        longData.put("marketValueRatio", ((BigDecimal) longData.get("marketValueRatio")).add(((BigDecimal) option.get("marketValueRatio"))));
                        longData.put("date", option.get("date"));
                        shortData.put("marketValueRatio", ((BigDecimal) shortData.get("marketValueRatio")).add(((BigDecimal) option.get("airdropMarketValueRatio"))));
                        shortData.put("date", option.get("date"));
                        nettingData.put("marketValueRatio", ((BigDecimal) nettingData.get("marketValueRatio")).add(((BigDecimal) option.get("nettingMarketRatio"))));
                        nettingData.put("date", option.get("date"));
                    } else if (option.get("longShort").equals(1)) { //多头
                        longData.put("marketValueRatio", ((BigDecimal) longData.get("marketValueRatio")).add(((BigDecimal) option.get("marketValueRatio"))));
                        longData.put("date", option.get("date"));
                        shortData.put("marketValueRatio", shortData.get("marketValueRatio"));
                        shortData.put("date", option.get("date"));
                        nettingData.put("marketValueRatio", ((BigDecimal) nettingData.get("marketValueRatio")).add(((BigDecimal) option.get("nettingMarketRatio"))));
                        nettingData.put("date", option.get("date"));
                    } else if (option.get("longShort").equals(2)) {  //空头
                        longData.put("marketValueRatio", longData.get("marketValueRatio"));
                        longData.put("date", option.get("date"));
                        shortData.put("marketValueRatio", shortData.get("marketValueRatio") == null ? option.get("marketValueRatio") : ((BigDecimal) shortData.get("marketValueRatio")).add(((BigDecimal) option.get("marketValueRatio"))));
                        shortData.put("date", option.get("date"));
                        nettingData.put("marketValueRatio", nettingData.get("marketValueRatio") == null ? option.get("nettingMarketRatio") : ((BigDecimal) nettingData.get("marketValueRatio")).add(((BigDecimal) option.get("nettingMarketRatio"))));
                        nettingData.put("date", option.get("date"));
                    }
                    longDataMap.put(data.get("date").toString(), longData);
                    shortDataMap.put(data.get("date").toString(), shortData);
                    nettingDataMap.put(data.get("date").toString(), nettingData);
                    data.put("nettingMarketRatio", nettingData.get("marketValueRatio"));
                    data.put("nettingMarketValue", data.get("nettingMarketValue") == null ? option.get("nettingMarketValue") : ((BigDecimal) data.get("nettingMarketValue")).add(((BigDecimal) option.get("nettingMarketValue"))));
                    data.put("longMarketValueRatio", longData.get("marketValueRatio"));
                    data.put("shortMarketValueRatio", shortData.get("marketValueRatio"));
                    dataMap.put(option.get("date").toString(), data);
                } else {
                    option.put("type", FundFuturesOption.getFundFuturesOptionByEnName(key).getNameDesc());
                    option.put("longMarketValueRatio", option.get("marketValueRatio"));
                    option.put("shortMarketValueRatio", option.get("longShort").toString().equals("1") ? option.get("airdropMarketValueRatio") : null);
                    dataMap.put(option.get("date").toString(), option);
                    Map<String, Object> longData = new LinkedHashMap<>();
                    Map<String, Object> shortData = new LinkedHashMap<>();
                    Map<String, Object> nettingData = new LinkedHashMap<>();
                    if (option.get("longShort").equals(1) && option.get("airdropMarketValueRatio") != null) {
                        longData.put("marketValueRatio", option.get("marketValueRatio"));
                        longData.put("date", option.get("date"));
                        shortData.put("marketValueRatio", option.get("airdropMarketValueRatio"));
                        shortData.put("date", option.get("date"));
                        nettingData.put("marketValueRatio", option.get("nettingMarketRatio"));
                        nettingData.put("date", option.get("date"));
                    } else if (option.get("longShort").equals(1)) { //多头
                        longData.put("marketValueRatio", option.get("marketValueRatio"));
                        longData.put("date", option.get("date"));
                        shortData.put("marketValueRatio", new BigDecimal(0));
                        shortData.put("date", option.get("date"));
                        nettingData.put("marketValueRatio", option.get("nettingMarketRatio"));
                        nettingData.put("date", option.get("date"));
                    } else if (option.get("longShort").equals(2)) {  //空头
                        longData.put("marketValueRatio", new BigDecimal(0));
                        longData.put("date", option.get("date"));
                        shortData.put("marketValueRatio", option.get("marketValueRatio"));
                        shortData.put("date", option.get("date"));
                        nettingData.put("marketValueRatio", option.get("nettingMarketRatio"));
                        nettingData.put("date", option.get("date"));
                    }
                    longDataMap.put(option.get("date").toString(), longData);
                    shortDataMap.put(option.get("date").toString(), shortData);
                    nettingDataMap.put(option.get("date").toString(), nettingData);
                }
            }
            resultData.put(key + "_long", longDataMap.values().stream().collect(Collectors.toList()));
            resultData.put(key + "_short", shortDataMap.values().stream().collect(Collectors.toList()));
            resultData.put(key + "_netting", nettingDataMap.values().stream().collect(Collectors.toList()));
        }
        result.put("productNameMapping", optionName);
        result.put("dataset", resultData);
        result.put("table", dataMap);
        return result;
    }

    private Map<String, List<Map<String, Object>>> wrapperFutureAnalysis(Map<String, List<Map<String, Object>>> optionData, FundPositionDetailDO detail, String key, BigDecimal nv) {
        if (optionData.containsKey(key)) {
            List<Map<String, Object>> options = optionData.get(key);
            Map<String, Object> data = new HashMap<>();
            BigDecimal marketValue = detail.getMarketValue();
            BigDecimal airdropMarketValue = detail.getAirdropMarketValue();
            data.put("marketValue", marketValue);
            data.put("marketValueRatio", marketValue.divide(nv, 6, RoundingMode.HALF_DOWN));
            data.put("airdropMarketValue", airdropMarketValue);
            data.put("airdropMarketValueRatio", airdropMarketValue == null ? null : airdropMarketValue.divide(nv, 6, RoundingMode.HALF_DOWN));
            if (airdropMarketValue == null) {
                data.put("nettingMarketValue", marketValue);
            } else if (marketValue == null) {
                data.put("nettingMarketValue", airdropMarketValue);
            } else if (airdropMarketValue == null && marketValue == null) {
                data.put("nettingMarketValue", 0);
            } else {
                data.put("nettingMarketValue", marketValue.add(airdropMarketValue));
            }
            data.put("type", detail.getSecuritiesName());
            detail.setNettingMarketValue((BigDecimal) data.get("nettingMarketValue"));
            data.put("nettingMarketRatio", detail.getNettingMarketValue().divide(nv, 6, RoundingMode.HALF_DOWN));
            data.put("date", DateUtil.formatDate(detail.getValuationDate()));
            data.put("longShort", detail.getSharesNature());
            options.add(data);
            optionData.put(key, options);
        } else {
            List<Map<String, Object>> options = new ArrayList<>();
            Map<String, Object> data = new HashMap<>();
            BigDecimal marketValue = detail.getMarketValue();
            BigDecimal airdropMarketValue = detail.getAirdropMarketValue();
            data.put("marketValue", marketValue);
            data.put("marketValueRatio", marketValue.divide(nv, 6, RoundingMode.HALF_DOWN));
            data.put("airdropMarketValue", airdropMarketValue);
            data.put("airdropMarketValueRatio", airdropMarketValue == null ? null : airdropMarketValue.divide(nv, 6, RoundingMode.HALF_DOWN));
            if (airdropMarketValue == null) {
                data.put("nettingMarketValue", marketValue);
            } else if (marketValue == null) {
                data.put("nettingMarketValue", airdropMarketValue);
            } else if (airdropMarketValue == null && marketValue == null) {
                data.put("nettingMarketValue", 0);
            } else {
                data.put("nettingMarketValue", marketValue.add(airdropMarketValue));
            }
            data.put("type", detail.getSecuritiesName());
            detail.setNettingMarketValue((BigDecimal) data.get("nettingMarketValue"));
            data.put("nettingMarketRatio", detail.getNettingMarketValue().divide(nv, 6, RoundingMode.HALF_DOWN));
            data.put("date", DateUtil.formatDate(detail.getValuationDate()));
            data.put("longShort", detail.getSharesNature());
            options.add(data);
            optionData.put(key, options);
        }
        return optionData;
    }

    public Map<String, Object> fundDerivativeLeverageMargin(Integer userId, String fundId, Integer shType, String startDate, String endDate) {
        Map<String, Object> result = new HashMap<>();
        List<Map<String, Object>> dataList = new ArrayList<>();
        Map<String, Object> dataListMap = new LinkedHashMap<>();
        if (fundId.startsWith("MF")) {
            List<Map<String, Object>> fundMarginMapList = new ArrayList<>();
            List<Map<String, Object>> totalMarketValueMapList = new ArrayList<>();
            List<Map<String, Object>> leverageRatioMapList = new ArrayList<>();
            List<Map<String, Object>> rinOfMarginMapList = new ArrayList<>();

            List<Map<String, Object>> marginDataMapList = this.publiclyFundExtInfoMapper.getFundMargin(fundId, startDate, endDate);
            List<FundPositionDetailDO> fundPositionDetailDOS = this.publiclyFundExtInfoMapper.queryPositionDetailsFundMargin(fundId, startDate, endDate);
            //按照时间分组
            Map<String, List<FundPositionDetailDO>> fundPositionDateMap = new LinkedHashMap<>();
            for (FundPositionDetailDO detail : fundPositionDetailDOS) {
                String valuationDate = DateUtil.formatDate(detail.getValuationDate());
                if (fundPositionDateMap.containsKey(valuationDate)) {
                    List<FundPositionDetailDO> details = fundPositionDateMap.get(valuationDate);
                    details.add(detail);
                    fundPositionDateMap.put(valuationDate, details);
                } else {
                    List<FundPositionDetailDO> details = new ArrayList<>();
                    details.add(detail);
                    fundPositionDateMap.put(valuationDate, details);
                }
            }
            Map<String, Map<String, Object>> rinMarginMap = new LinkedHashMap<>();
            for (Map<String, Object> map : marginDataMapList) {
//                rinMarginMap.put(DateUtils.DateToString((Date) map.get("report_date"),"yyyy-MM-dd"),map);
                rinMarginMap.put(MapUtil.getStr(map, "report_date"), map);
            }
            for (String valuationDate : fundPositionDateMap.keySet()) {
                Map<String, Object> valuationMap = new HashMap<>();
                Map<String, Object> fundMarginMap = new HashMap<>();
                Map<String, Object> totalMarketValueMap = new HashMap<>();
                Map<String, Object> leverageRatioMap = new HashMap<>();
                Map<String, Object> rinOfMarginMap = new HashMap<>();
                fundMarginMap.put("valuationDate", valuationDate);
                totalMarketValueMap.put("valuationDate", valuationDate);
                leverageRatioMap.put("valuationDate", valuationDate);
                rinOfMarginMap.put("valuationDate", valuationDate);
                Map<String, Object> dataMap = rinMarginMap.get(valuationDate);
                if (dataMap == null) {
                    continue;
                }
                BigDecimal rinOfMargin = (BigDecimal) dataMap.get("rin_of_margin");
                BigDecimal fundMargin = (BigDecimal) dataMap.get("mv_of_margin");
                if (rinOfMargin == null) {
                    rinOfMargin = new BigDecimal(0);
                }
                if (fundMargin == null) {
                    fundMargin = new BigDecimal(0);
                }
                valuationMap.put("fundMargin", fundMargin.divide(new BigDecimal(10000)).setScale(2, RoundingMode.HALF_DOWN));
                fundMarginMap.put("totalMarketValue", fundMargin.divide(new BigDecimal(10000)).setScale(2, RoundingMode.HALF_DOWN));
                rinOfMarginMap.put("ratio", rinOfMargin.setScale(4, RoundingMode.HALF_DOWN));
                BigDecimal totalMarketValue = new BigDecimal(0);
                List<FundPositionDetailDO> details = fundPositionDateMap.get(valuationDate);
                for (FundPositionDetailDO detail : details) {
                    String securitiesCode = detail.getSecuritiesCode();
                    if (securitiesCode == null) {
                        continue;
                    }
                    if (detail.getMarketValue() == null) {
                        totalMarketValue = totalMarketValue.add(detail.getAirdropMarketValue() == null ? new BigDecimal(0) : detail.getAirdropMarketValue().abs());
                    } else if (detail.getAirdropMarketValue() == null) {
                        totalMarketValue = totalMarketValue.add(detail.getMarketValue() == null ? new BigDecimal(0) : detail.getMarketValue().abs());
                    } else {
                        totalMarketValue = totalMarketValue.add(detail.getMarketValue().abs().add(detail.getMarketValue().abs()));
                    }
                }
                valuationMap.put("totalMarketValue", totalMarketValue.divide(new BigDecimal(10000)));
                totalMarketValueMap.put("totalMarketValue", totalMarketValue.divide(new BigDecimal(10000)).setScale(2, RoundingMode.HALF_DOWN));
                BigDecimal leverageRatio = totalMarketValue.divide(fundMargin, 4, RoundingMode.HALF_DOWN);
                valuationMap.put("leverageRatio", leverageRatio.setScale(4, RoundingMode.HALF_DOWN));
                leverageRatioMap.put("ratio", leverageRatio);
                valuationMap.put("rinOfMargin", rinOfMargin);
                valuationMap.put("valuationDate", valuationDate);
                dataList.add(valuationMap);
                fundMarginMapList.add(fundMarginMap);
                rinOfMarginMapList.add(rinOfMarginMap);
                totalMarketValueMapList.add(totalMarketValueMap);
                leverageRatioMapList.add(leverageRatioMap);
                dataListMap.put("fundMargin", fundMarginMapList);
                dataListMap.put("rinOfMargin", rinOfMarginMapList);
                dataListMap.put("totalMarketValue", totalMarketValueMapList);
                dataListMap.put("leverageRatio", leverageRatioMapList);
            }
        }
        Map<String, String> optionName = new HashMap<>();
        optionName.put("fundMargin", "保证金");
        optionName.put("totalMarketValue", "期货总成本");
        optionName.put("leverageRatio", "杠杆比率");
        optionName.put("rinOfMargin", "保证金占比");
        result.put("productNameMapping", optionName);
        result.put("dataset", dataListMap);
        result.put("table", dataList);
        return result;
    }

    private List<Map<String, Object>> getRate(List<Map<String, Object>> closePriceList) {
        if (closePriceList.size() < 2) {
            return new ArrayList<>();
        }
        List<Map<String, Object>> rateList = new ArrayList<>();
        for (int idx = 0; idx + 1 < closePriceList.size(); idx++) {
            if (closePriceList.get(idx) == null) {
                continue;
            }
            Map<String, Object> closePriceMap = closePriceList.get(idx);
            Map<String, Object> nextClosePriceMap = closePriceList.get(idx + 1);
            BigDecimal closePrice = (BigDecimal) closePriceMap.get("close_price");
            BigDecimal nextClosePrice = (BigDecimal) nextClosePriceMap.get("close_price");
            BigDecimal rate = nextClosePrice.divide(closePrice, RoundingMode.HALF_DOWN).subtract(new BigDecimal(1));
            Map<String, Object> data = new HashMap<>();
            String priceDate = nextClosePriceMap.get("price_date").toString();
            data.put("price_date", priceDate);
            data.put("rate", rate);
            rateList.add(data);
        }
        return rateList;

    }

    public Map<String, Object> fundPositionAnalysisLine(Integer userId, String fundId, Integer optionType, Integer shType, String startDate, String endDate) {
        Map<String, Object> result = new HashMap<>();
        Map<String, List<Map<String, Object>>> optionData = new LinkedHashMap<>();
        optionData.put(FundFuturesOption.NonFerrousMetal.getNameEnDesc(), new ArrayList<>());
        optionData.put(FundFuturesOption.BlackMetal.getNameEnDesc(), new ArrayList<>());
        optionData.put(FundFuturesOption.Metal.getNameEnDesc(), new ArrayList<>());
        optionData.put(FundFuturesOption.Energization.getNameEnDesc(), new ArrayList<>());
        optionData.put(FundFuturesOption.IndustrialProducts.getNameEnDesc(), new ArrayList<>());
        optionData.put(FundFuturesOption.AgriculturalProducts.getNameEnDesc(), new ArrayList<>());
        optionData.put(FundFuturesOption.PreciousMetal.getNameEnDesc(), new ArrayList<>());
        optionData.put(FundFuturesOption.BondFuture.getNameEnDesc(), new ArrayList<>());
        optionData.put(FundFuturesOption.StockIndexFuture.getNameEnDesc(), new ArrayList<>());
        Map<String, String> optionName = new HashMap<>();
        optionName.put(FundFuturesOption.NonFerrousMetal.getNameEnDesc(), FundFuturesOption.NonFerrousMetal.getNameDesc() + "敞口");
        optionName.put(FundFuturesOption.BlackMetal.getNameEnDesc(), FundFuturesOption.BlackMetal.getNameDesc() + "敞口");
        optionName.put(FundFuturesOption.Metal.getNameEnDesc(), FundFuturesOption.Metal.getNameDesc() + "敞口");
        optionName.put(FundFuturesOption.Energization.getNameEnDesc(), FundFuturesOption.Energization.getNameDesc() + "敞口");
        optionName.put(FundFuturesOption.IndustrialProducts.getNameEnDesc(), FundFuturesOption.IndustrialProducts.getNameDesc() + "敞口");
        optionName.put(FundFuturesOption.AgriculturalProducts.getNameEnDesc(), FundFuturesOption.AgriculturalProducts.getNameDesc() + "敞口");
        optionName.put(FundFuturesOption.PreciousMetal.getNameEnDesc(), FundFuturesOption.PreciousMetal.getNameDesc() + "敞口");
        optionName.put(FundFuturesOption.BondFuture.getNameEnDesc(), FundFuturesOption.BondFuture.getNameDesc() + "敞口");
        optionName.put(FundFuturesOption.StockIndexFuture.getNameEnDesc(), FundFuturesOption.StockIndexFuture.getNameDesc() + "敞口");
        List<String> valuationDateList = new ArrayList<>();
        if (fundId.startsWith("MF")) {
            List<FundPositionDetailDO> fundPositionDetailDOS = this.publiclyFundExtInfoMapper.getMFFundPositionDetail(fundId, startDate, endDate);
            List<Map<String, Object>> nvListMap = this.publiclyFundExtInfoMapper.getValuationDateNv(fundId, startDate, endDate);
            //按照时间分组
            Map<String, List<FundPositionDetailDO>> fundPositionDateMap = new LinkedHashMap<>();
            Map<String, BigDecimal> nvDateMap = new LinkedHashMap<>();
            for (FundPositionDetailDO detail : fundPositionDetailDOS) {
                String valuationDate = DateUtil.formatDate(detail.getValuationDate());
                if (fundPositionDateMap.containsKey(valuationDate)) {
                    List<FundPositionDetailDO> details = fundPositionDateMap.get(valuationDate);
                    details.add(detail);
                    fundPositionDateMap.put(valuationDate, details);
                } else {
                    List<FundPositionDetailDO> details = new ArrayList<>();
                    details.add(detail);
                    fundPositionDateMap.put(valuationDate, details);
                }
            }
            //基金净资产按时间分类
            for (Map<String, Object> nvMap : nvListMap) {
                nvDateMap.put(nvMap.get("valuationDate").toString(), (BigDecimal) nvMap.get("nv"));
            }
            //针对每个估值日期做数据处理
            for (String valuationDate : fundPositionDateMap.keySet()) {
                valuationDateList.add(valuationDate);
                List<FundPositionDetailDO> details = fundPositionDateMap.get(valuationDate);
                BigDecimal nv = nvDateMap.get(valuationDate);
                for (FundPositionDetailDO detail : details) {
                    String securitiesCode = detail.getSecuritiesCode();
                    if (securitiesCode == null) {
                        continue;
                    }
                    if (nv == null) {
                        continue;
                    }
                    if ((securitiesCode.length() >= 1 && securitiesCode.substring(0, 1).matches("[A-Za-z]"))) { //前两位字母
                        List<Map<String, String>> optionTypeMapList = null;
                        //根据前两位字母查询所属品种
                        String tradingCode = securitiesCode.substring(0, 2);
                        if ((securitiesCode.length() >= 2 && securitiesCode.substring(0, 2).matches("[A-Za-z]+"))) {
                            optionTypeMapList = this.baseUnderlyingMapper.getSecondOptionTypeByCode(tradingCode);
                        } else {
                            optionTypeMapList = this.baseUnderlyingMapper.getSecondOptionTypeByCode(tradingCode.substring(0, 1));
                        }
                        if ((optionTypeMapList == null || optionTypeMapList.size() == 0)) {
                            continue;
                        }
                        for (Map<String, String> optionTypeMap : optionTypeMapList) {
                            String secondOptionType = optionTypeMap.get("second_option_type");
                            if (FundFuturesOption.PreciousMetal.getNameDesc().equals(secondOptionType) || FundFuturesOption.Metal.getNameDesc().equals(secondOptionType)
                                    || FundFuturesOption.BlackMetal.getNameDesc().equals(secondOptionType) || FundFuturesOption.NonFerrousMetal.getNameDesc().equals(secondOptionType)
                                    || FundFuturesOption.Energization.getNameDesc().equals(secondOptionType) || FundFuturesOption.IndustrialProducts.getNameDesc().equals(secondOptionType)
                                    || FundFuturesOption.AgriculturalProducts.getNameDesc().equals(secondOptionType) || FundFuturesOption.BondFuture.getNameDesc().equals(secondOptionType)
                                    || FundFuturesOption.StockIndexFuture.getNameDesc().equals(secondOptionType)) {
                                String key = FundFuturesOption.getFundFuturesOptionByName(secondOptionType).getNameEnDesc();
                                optionData = wrapperFutureAnalysisLine(optionData, detail, key, nv);
                            }
                        }
                    }
                }
            }
        }

        for (String key : optionData.keySet()) {
            List<Map<String, Object>> options = optionData.get(key);
            if (options.size() < 1) {
                continue;
            }
            Map<String, Map<String, Object>> dataMap = new LinkedHashMap<>();
            for (Map<String, Object> option : options) {
                if (dataMap.containsKey(option.get("date").toString())) {
                    Map<String, Object> data = dataMap.get(option.get("date").toString());
                    BigDecimal nettingMarketRatio = (BigDecimal) data.get("nettingMarketRatio");
                    BigDecimal airdropMarketValue = (BigDecimal) data.get("airdropMarketValue");
                    BigDecimal airdropMarketValueRatio = (BigDecimal) data.get("airdropMarketValueRatio");
                    BigDecimal nettingMarketValue = (BigDecimal) data.get("nettingMarketValue");
                    BigDecimal marketValue = (BigDecimal) data.get("marketValue");

                    BigDecimal optionNettingMarketRatio = (BigDecimal) option.get("nettingMarketRatio");
                    BigDecimal optionAirdropMarketValue = (BigDecimal) option.get("airdropMarketValue");
                    BigDecimal optionAirdropMarketValueRatio = (BigDecimal) option.get("airdropMarketValueRatio");
                    BigDecimal optionNettingMarketValue = (BigDecimal) option.get("nettingMarketValue");
                    BigDecimal optionMarketValue = (BigDecimal) option.get("marketValue");
                    data.put("nettingMarketRatio", nettingMarketRatio == null ? optionNettingMarketRatio : nettingMarketRatio.add(optionNettingMarketRatio == null ? new BigDecimal(0) : optionNettingMarketRatio));
                    data.put("airdropMarketValue", airdropMarketValue == null ? optionAirdropMarketValue : airdropMarketValue.add(optionAirdropMarketValue == null ? new BigDecimal(0) : optionAirdropMarketValue));
                    data.put("airdropMarketValueRatio", airdropMarketValueRatio == null ? optionAirdropMarketValueRatio : airdropMarketValueRatio.add(optionAirdropMarketValueRatio == null ? new BigDecimal(0) : optionAirdropMarketValueRatio));
                    data.put("nettingMarketValue", nettingMarketValue == null ? optionNettingMarketValue : nettingMarketValue.add(optionNettingMarketValue == null ? new BigDecimal(0) : optionNettingMarketValue));
                    data.put("marketValue", marketValue == null ? optionMarketValue : marketValue.add(optionMarketValue == null ? new BigDecimal(0) : optionMarketValue));
                } else {
                    option.put("type", key);
                    dataMap.put(option.get("date").toString(), option);
                }
            }
            for (String valuationDate : valuationDateList) {
                for (String optionKey : optionData.keySet()) {
                    List<Map<String, Object>> optionMapList = optionData.get(optionKey);
                    boolean flag = true;
                    for (Map<String, Object> optionMap : optionMapList) {
                        if (optionMap.get("date").toString().equals(valuationDate)) {
                            flag = false;
                            break;
                        }
                    }
                    if (flag) {
                        Map<String, Object> map = new HashMap<>();
                        map.put("date", valuationDate);
                        map.put("nettingMarketRatio", null);
                        map.put("airdropMarketValue", null);
                        map.put("airdropMarketValueRatio", null);
                        map.put("nettingMarketValue", null);
                        map.put("marketValue", null);
                        optionMapList.add(map);
                    }
                    optionMapList = optionMapList.stream()
                            .sorted(Comparator.comparing(o -> o.get("date").toString()))
                            .collect(Collectors.toList());
                    optionData.put(optionKey, optionMapList);
                }
            }
            optionData.put(key, new ArrayList<>(dataMap.values()));
        }
        result.put("productNameMapping", optionName);
        result.put("dataset", optionData);
        return result;
    }

    private Map<String, List<Map<String, Object>>> wrapperFutureAnalysisLine(Map<String, List<Map<String, Object>>> optionData, FundPositionDetailDO detail, String key, BigDecimal nv) {
        if (optionData.containsKey(key)) {
            List<Map<String, Object>> options = optionData.get(key);
            if (options == null) {
                options = new ArrayList<>();
            }
            Map<String, Object> data = new HashMap<>();
            BigDecimal marketValue = detail.getMarketValue();
            BigDecimal airdropMarketValue = detail.getAirdropMarketValue();
            data.put("marketValue", marketValue);
            data.put("marketValueRatio", marketValue.divide(nv, RoundingMode.HALF_DOWN));
            data.put("airdropMarketValue", airdropMarketValue);
            data.put("airdropMarketValueRatio", airdropMarketValue == null ? null : airdropMarketValue.divide(nv, RoundingMode.HALF_DOWN));
            if (airdropMarketValue == null) {
                data.put("nettingMarketValue", marketValue);
            } else if (marketValue == null) {
                data.put("nettingMarketValue", airdropMarketValue);
            } else if (airdropMarketValue == null && marketValue == null) {
                data.put("nettingMarketValue", 0);
            } else {
                data.put("nettingMarketValue", marketValue.add(airdropMarketValue));
            }
            data.put("type", detail.getSecuritiesName());
            detail.setNettingMarketValue((BigDecimal) data.get("nettingMarketValue"));
            data.put("nettingMarketRatio", detail.getNettingMarketValue().divide(nv, 4, RoundingMode.HALF_DOWN));
            data.put("date", DateUtil.formatDate(detail.getValuationDate()));
            options.add(data);
            optionData.put(key, options);
        } else {
            List<Map<String, Object>> options = new ArrayList<>();
            Map<String, Object> data = new HashMap<>();
            BigDecimal marketValue = detail.getMarketValue();
            BigDecimal airdropMarketValue = detail.getAirdropMarketValue();
            data.put("marketValue", marketValue);
            data.put("marketValueRatio", marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
            data.put("airdropMarketValue", airdropMarketValue);
            data.put("airdropMarketValueRatio", airdropMarketValue == null ? null : airdropMarketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
            if (airdropMarketValue == null) {
                data.put("nettingMarketValue", marketValue);
            } else if (marketValue == null) {
                data.put("nettingMarketValue", airdropMarketValue);
            } else if (airdropMarketValue == null && marketValue == null) {
                data.put("nettingMarketValue", 0);
            } else {
                data.put("nettingMarketValue", marketValue.add(airdropMarketValue));
            }
            data.put("type", detail.getSecuritiesName());
            detail.setNettingMarketValue((BigDecimal) data.get("nettingMarketValue"));
            data.put("nettingMarketRatio", detail.getNettingMarketValue().divide(nv, 4, RoundingMode.HALF_DOWN));
            data.put("date", DateUtil.formatDate(detail.getValuationDate()));
            options.add(data);
            optionData.put(key, options);
        }
        return optionData;
    }

    public Map<String, Object> fundFuturesOptionForBreed(Integer userId, String fundId, Integer optionType, Integer shType, String startDate, String endDate) {
        //套期：【二级科目】3201、3202以下“套期”下合约资产归属
        //投机：【二级科目】3101、3102、3103以下“投机”字段下合约资产归属
        Map<String, Object> result = new HashMap<>();
        Map<String, List<Map<String, Object>>> optionData = new LinkedHashMap<>();
        Map<String, String> optionName = new LinkedHashMap<>();
        List<String> valuationDateList = new ArrayList<>();
        if (fundId.startsWith("MF")) {
            List<FundPositionDetailDO> fundPositionDetailDOS = this.publiclyFundExtInfoMapper.getMFFundPositionDetail(fundId, startDate, endDate);
            List<Map<String, Object>> nvListMap = this.publiclyFundExtInfoMapper.getValuationDateNv(fundId, startDate, endDate);
            //按照时间分组
            Map<String, List<FundPositionDetailDO>> fundPositionDateMap = new LinkedHashMap<>();
            Map<String, BigDecimal> nvDateMap = new LinkedHashMap<>();
            for (FundPositionDetailDO detail : fundPositionDetailDOS) {
                String valuationDate = DateUtil.formatDate(detail.getValuationDate());
                if (fundPositionDateMap.containsKey(valuationDate)) {
                    List<FundPositionDetailDO> details = fundPositionDateMap.get(valuationDate);
                    details.add(detail);
                    fundPositionDateMap.put(valuationDate, details);
                } else {
                    List<FundPositionDetailDO> details = new ArrayList<>();
                    details.add(detail);
                    fundPositionDateMap.put(valuationDate, details);
                }
            }
            //基金净资产按时间分类
            for (Map<String, Object> nvMap : nvListMap) {
                nvDateMap.put(nvMap.get("valuationDate").toString(), (BigDecimal) nvMap.get("nv"));
            }
            //针对每个估值日期做数据处理
            for (String valuationDate : fundPositionDateMap.keySet()) {
                valuationDateList.add(valuationDate);
                List<FundPositionDetailDO> details = fundPositionDateMap.get(valuationDate);
                if (nvDateMap.get(valuationDate) == null) {  //净资产为null，直接跳过
                    continue;
                }
                BigDecimal nv = nvDateMap.get(valuationDate);
                for (FundPositionDetailDO detail : details) {
                    String securitiesCode = detail.getSecuritiesCode();
                    String securitiesName = detail.getSecuritiesName();
                    //针对国债期货做名称处理
                    if (securitiesCode.matches("^TS[1-9]\\d+") || securitiesCode.matches("^TF[1-9]\\d+") || securitiesCode.matches("^TL[1-9]\\d+")
                            || securitiesCode.matches("^T[1-9]\\d+")) {
                        securitiesName = securitiesCode;
                    }
                    if (securitiesCode == null) {
                        continue;
                    }
                    if ((securitiesCode.length() >= 1 && securitiesCode.substring(0, 1).matches("[A-Za-z]"))) { //前两位字母
                        Map<String, String> optionTypeMap = null;
                        //根据前两位字母查询所属品种
                        String tradingCode = securitiesCode.substring(0, 2);
                        String optioneKey = null;
                        if (securitiesCode.substring(0, 2).matches("[A-Za-z]+")) {
                            optionTypeMap = this.baseUnderlyingMapper.getOptionTypeMap(tradingCode);
                            optioneKey = tradingCode;
                        } else {
                            optionTypeMap = this.baseUnderlyingMapper.getOptionTypeMap(tradingCode.substring(0, 1));
                            optioneKey = tradingCode.substring(0, 1);
                        }
                        if (optionTypeMap == null || optionTypeMap.keySet().isEmpty() && (FundFuturesOption.getFundFuturesOption(optionType).getId() == 0 || optionType == 5)) {
                            optionData = wrapperDetailData(optionData, detail, securitiesCode, nv);
                            if (!optionName.containsKey(securitiesCode)) {
                                optionName.put(securitiesCode, securitiesName);
                            }
                            continue;
                        }
                        String firstOptionType = optionTypeMap.get("first_option_type");
                        if (FundFuturesOption.getFundFuturesOption(optionType).getNameDesc().equals(firstOptionType) || FundFuturesOption.getFundFuturesOption(optionType).getId() == 0) {
                            optionData = wrapperDetailDataForBeed(optionData, detail, optioneKey, nv, optionTypeMap.get("product_name"));
//                            if(!optionName.containsKey(securitiesCode)){
//                                optionName.put(securitiesCode,securitiesName);
//                            }
//                            optionData = wrapperCFDetailDataForBeed(optionData,detail,optioneKey,optionTypeMap.get("product_name"));
                            optionName.put(optioneKey, optionTypeMap.get("product_name"));
                        }
                    } else {  //期权
                        Map<String, String> dataMap = this.publiclyFundExtInfoMapper.getOptionContract(securitiesCode, securitiesName);
                        if (dataMap == null || dataMap.keySet().isEmpty()) {
                            List<Map<String, String>> futureMapList = this.publiclyFundExtInfoMapper.getFundDetailForFuture(securitiesCode.substring(0, 4), fundId, valuationDate);
                            for (Map<String, String> futureMap : futureMapList) {
                                if (futureMap.get("securities_name") != null && futureMap.get("securities_name").contains("期权")) {
                                    dataMap = new HashMap<>();
                                    dataMap.put(futureMap.get("securities_code"), futureMap.get("securities_name"));
                                    break;
                                }
                            }
                        }
                        if ((dataMap == null || dataMap.keySet().isEmpty()) && (FundFuturesOption.getFundFuturesOption(optionType).getId() == 0 || optionType == 5)) {
                            //属于其他衍生品
                            optionData = wrapperDetailData(optionData, detail, securitiesCode, nv);
                            if (!optionName.containsKey(securitiesCode)) {
                                optionName.put(securitiesCode, securitiesName);
                            }
                            continue;
                        }
                        if (dataMap == null || dataMap.keySet().isEmpty()) {
                            continue;
                        }
                        String firstOptionType = FundFuturesOption.Future.getNameDesc();
                        if (FundFuturesOption.getFundFuturesOption(optionType).getNameDesc().equals(firstOptionType) || FundFuturesOption.getFundFuturesOption(optionType).getId() == 0) {
                            optionData = wrapperDetailData(optionData, detail, securitiesCode, nv);
                            if (!optionName.containsKey(securitiesCode)) {
                                optionName.put(securitiesCode, securitiesName);
                            }
                        }
                    }
                }
            }
        }
        for (String valuationDate : valuationDateList) {
            for (String optionKey : optionData.keySet()) {
                List<Map<String, Object>> optionMapList = optionData.get(optionKey);
                if (optionMapList != null && optionMapList.isEmpty()) {
                    continue;
                }
                boolean flag = true;
                for (Map<String, Object> optionMap : optionMapList) {
                    if (optionMap.get("date").toString().equals(valuationDate)) {
                        flag = false;
                        break;
                    }
                }
                if (flag) {
                    Map<String, Object> map = new HashMap<>();
                    map.put("date", valuationDate);
                    map.put("ratio", null);
                    map.put("type", optionMapList.get(0).get("type"));
                    optionMapList.add(map);
                }
                optionMapList = optionMapList.stream()
                        .sorted(Comparator.comparing(o -> o.get("date").toString()))
                        .collect(Collectors.toList());
                optionData.put(optionKey, optionMapList);
            }
        }
        result.put("productNameMapping", optionName);
        result.put("dataset", optionData);
        return result;
    }

    private Map<String, List<Map<String, Object>>> wrapperDetailDataForBeed(Map<String, List<Map<String, Object>>> optionData,
                                                                            FundPositionDetailDO detail, String key, BigDecimal nv,
                                                                            String productName) {
        if (optionData.containsKey(key)) {
            List<Map<String, Object>> options = optionData.get(key);
            Map<String, Object> data = new HashMap<>();
            data.put("type", key);
            BigDecimal marketValue = detail.getMarketValue();
            BigDecimal airdropMarketValue = detail.getAirdropMarketValue();
            if (airdropMarketValue == null && detail.getSharesNature() == 1) {
                data.put("ratio", marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setMarketValueRatio(marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setAirdropMarketValueRatio(null);
            } else if (airdropMarketValue == null && detail.getSharesNature() == 2) {
                data.put("ratio", marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setAirdropMarketValueRatio(marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setMarketValueRatio(marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
            } else {
                data.put("ratio", (marketValue.add(airdropMarketValue)).divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setMarketValueRatio((marketValue.add(airdropMarketValue)).divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setAirdropMarketValueRatio(airdropMarketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
            }
            data.put("date", DateUtil.formatDate(detail.getValuationDate()));
            data.put("securitiesCode", detail.getSecuritiesCode());
            int idx = 0;
            for (Map<String, Object> dataMap : options) {
                if (dataMap.get("date").toString().equals(DateUtil.formatDate(detail.getValuationDate()))) {
                    dataMap.put("ratio", new BigDecimal(dataMap.get("ratio").toString()).add(new BigDecimal(data.get("ratio").toString())));
                    idx++;
                }
            }
            if (idx == 0) {
                options.add(data);
            }
            optionData.put(key, options);
        } else {
            List<Map<String, Object>> options = new ArrayList<>();
            Map<String, Object> data = new HashMap<>();
            data.put("type", key);
            BigDecimal marketValue = detail.getMarketValue();
            BigDecimal airdropMarketValue = detail.getAirdropMarketValue();
            if (airdropMarketValue == null && detail.getSharesNature() == 1) {
                data.put("ratio", marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setMarketValueRatio(marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setAirdropMarketValueRatio(null);
            } else if (airdropMarketValue == null && detail.getSharesNature() == 2) {
                data.put("ratio", marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setAirdropMarketValueRatio(marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setMarketValueRatio(marketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
            } else {
                data.put("ratio", (marketValue.add(airdropMarketValue)).divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setMarketValueRatio((marketValue.add(airdropMarketValue)).divide(nv, 4, RoundingMode.HALF_DOWN));
                detail.setAirdropMarketValueRatio(airdropMarketValue.divide(nv, 4, RoundingMode.HALF_DOWN));
            }
            data.put("date", DateUtil.formatDate(detail.getValuationDate()));
            data.put("securitiesCode", detail.getSecuritiesCode());
            options.add(data);
            optionData.put(key, options);
        }
        return optionData;
    }

    public Map<String, List<String>> fundFuturesOptionView() {
        List<Map<String, String>> productTradingMapList = new ArrayList<>();
        Map<String, List<String>> result = new LinkedHashMap<>();
        productTradingMapList = this.baseUnderlyingMapper.fundFuturesOptionView();
        for (Map<String, String> productMap : productTradingMapList) {
            String secondType = productMap.get("second_option_type");
            if (!result.containsKey(secondType)) {
                List<String> children = new ArrayList<>();
                children.add(productMap.get("product_name"));
                result.put(secondType, children);
            } else {
                List<String> children = result.get(secondType);
                children.add(productMap.get("product_name"));
                result.put(secondType, children);
            }
        }
        return result;
    }

    /**
     * 返回信息  key为code value为板块集合 "pp" ：["能化","工业品"]
     *
     * @return
     */
    public Map<String, List<String>> listCodeAndSecondOptionTypeMap() {
        List<ValueLabelVO> valueLabelDoList = this.baseUnderlyingMapper.getOptionTypeAndTradeCode("商品");
        // 转成code-->secondOptionType
        Map<String, List<String>> resultMap = valueLabelDoList.stream()
                .collect(Collectors.groupingBy(ValueLabelVO::getValue, Collectors.mapping(ValueLabelVO::getLabel, Collectors.toList())));
        return resultMap;

    }

    /**
     * key为secondOptionType value为List
     *
     * @return
     */
    public Map<String, List<String>> listSecondOptionTypeAndCode() {
        List<ValueLabelVO> valueLabelDoList = this.baseUnderlyingMapper.getOptionTypeAndTradeCode("商品");
        // 转成code-->secondOptionType
        Map<String, List<String>> resultMap = valueLabelDoList.stream()
                .collect(Collectors.groupingBy(ValueLabelVO::getLabel, Collectors.mapping(ValueLabelVO::getValue, Collectors.toList())));
        return resultMap;
    }

    public List<String> listSecondOptionType() {
        return this.baseUnderlyingMapper.getSecondOptionType("商品");
    }

}

