package com.smppw.analysis.domain.manager.position.stock;

import cn.hutool.core.map.MapUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.smppw.analysis.domain.dto.position.stock.BarraSensitivityParams;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.analysis.infrastructure.consts.BarraStyleRespEnum;
import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.common.pojo.enums.RaiseType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/12 16:46
 * @description barra敏感度分析，这里仅支持公募
 */
@Component
public class BarraSensitivityComponent {
    private static final Logger LOGGER = LoggerFactory.getLogger(BarraSensitivityComponent.class);
    private final String pyBaseUrl;

    public BarraSensitivityComponent(AnalysisProperty property) {
        this.pyBaseUrl = property.getPyUrl();
    }

    public Map<String, Object> bizHandle(BarraSensitivityParams params) {
        // 沪深300、中证500和中证1000
        Map<String, String> benchmarkMap = MapUtil.builder("IN00000008", "000300.SH")
                .put("IN0000007M", "000905.SH").put("IN0000008O", "000852.SH").build();
        try {
            RaiseType raiseType = RaiseType.Public;
            Map<String, Object> req = MapUtil.newHashMap();
            req.put("fundIds", params.getFundId());
            req.put("benchmarkId", benchmarkMap.get(params.getBenchmarkId()));
            req.put("valuationDate", params.getDate());
            req.put("raiseType", raiseType);
            String body = HttpUtil.post(this.pyBaseUrl + "GetBarraPosAttribution", JSONUtil.toJsonStr(req));
            JSONObject entries = JSONUtil.parseObj(body);
            JSONObject fund = entries.getJSONObject("fund");
            JSONObject index = entries.getJSONObject("index");
            JSONObject expose = entries.getJSONObject("expose");
            Set<String> strings = expose.keySet();
            Map<String, BigDecimal> exposure = MapUtil.newHashMap(false);
            for (String string : strings) {
                exposure.put(string, expose.getBigDecimal(string));
            }
            exposure = MapUtil.sortByValue(exposure, true);

            List<ValueLabelVO> collect = Arrays.stream(BarraStyleRespEnum.values())
                    .map(e -> new ValueLabelVO(e.getName(), e.getYName())).collect(Collectors.toList());
            return MapUtil.<String, Object>builder().put("fund", fund).put("benchmark", index)
                    .put("exposure", exposure).put("productNameMapping", collect).build();
        } catch (Exception e) {
            LOGGER.error("持仓分析Barra敏感度错误！" + e.getMessage());
            return MapUtil.newHashMap();
        }
    }
}
