package com.smppw.analysis.domain.manager.position.stock;

import cn.hutool.core.collection.ListUtil;
import com.smppw.analysis.domain.dto.position.AssetCategoryEnum;
import com.smppw.analysis.domain.dto.position.FundPositionBaseInfo;
import com.smppw.analysis.domain.dto.position.FundPositionDetail;
import com.smppw.analysis.domain.dto.position.stock.ChangeNumberParams;
import com.smppw.analysis.domain.dto.position.stock.ChangeNumberVO;
import com.smppw.analysis.domain.manager.position.AbstractNonSynthesizeBizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerConstants;
import com.smppw.analysis.domain.manager.position.PositionLoadFactory;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.common.pojo.dto.NewDateValue;
import com.smppw.constants.SecType;
import org.springframework.stereotype.Component;

import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/8 15:46
 * @description 持股数量变动
 */
@Component(BizHandlerConstants.CHANGE_NUMBER)
public class ChangeNumberBizHandler extends AbstractNonSynthesizeBizHandler<ChangeNumberParams, List<ChangeNumberVO>> {
    public ChangeNumberBizHandler(AnalysisProperty property, CacheFactory cacheFactory, PositionLoadFactory factory) {
        super(property, cacheFactory, factory);
    }

    @Override
    protected List<ChangeNumberVO> afterNonSynthesizeLoadData(ChangeNumberParams params, List<FundPositionBaseInfo> baseInfos, List<FundPositionDetail> positionInfos) {
        positionInfos = positionInfos.stream().filter(e -> SecType.PUBLICLY_OFFERED_FUNDS.equals(e.getFundType()) || (e.getLevel() != null && e.getLevel() >= 4))
                .filter(e -> AssetCategoryEnum.STOCK.name().equals(e.getAsset().getValue())).collect(Collectors.toList());
        List<ChangeNumberVO> resultList = ListUtil.list(false);
        Map<String, List<FundPositionDetail>> collect = positionInfos.stream().collect(Collectors.groupingBy(FundPositionDetail::getDate));
        collect.forEach((k, v) -> {
            long count = v.stream().map(FundPositionDetail::getRef).distinct().count();
            ChangeNumberVO vo = new ChangeNumberVO();
            vo.setDate(k);
            vo.setValue(String.valueOf(count));
            resultList.add(vo);
        });
        resultList.sort(Comparator.comparing(NewDateValue::getDate));
        return resultList;
    }
}
