package com.smppw.analysis.domain.manager.position.stock;

import cn.hutool.core.collection.ListUtil;
import com.smppw.analysis.domain.dto.position.CategoryConstraint;
import com.smppw.analysis.domain.dto.position.FundPositionDetail;
import com.smppw.analysis.domain.dto.position.stock.StockAllocationParams;
import com.smppw.analysis.domain.dto.position.stock.StockAllocationVO;
import com.smppw.analysis.domain.manager.position.AbstractAnalysisBizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerConstants;
import com.smppw.analysis.domain.manager.position.PositionLoadFactory;
import com.smppw.analysis.domain.dataobject.IndexSecWeightInfoDO;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.domain.dao.PositionAnalysisDao;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.common.pojo.ValueLabelVO;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/8 17:19
 * @description 行业配置
 */
@Component(BizHandlerConstants.INDUSTRY_ALLOCATION)
public class IndustryAllocationBizHandler extends AbstractAnalysisBizHandler<StockAllocationParams, List<StockAllocationVO>> {
    public IndustryAllocationBizHandler(AnalysisProperty property, CacheFactory cacheFactory,
                                        PositionLoadFactory factory, PositionAnalysisDao analysisService) {
        super(property, cacheFactory, factory, analysisService);
    }

    @Override
    protected List<StockAllocationVO> afterAnalysisLoadData(String benchmarkId, List<FundPositionDetail> positionDetails,
                                                            Map<String, List<IndexSecWeightInfoDO>> indexWeightMap) {
        if (logger.isDebugEnabled()) {
            logger.debug("行业配置 数据准备耗时：" + (System.currentTimeMillis() - this.current) + "ms");
        }
        List<StockAllocationVO> resultList = ListUtil.list(false);
        // 行业与A股对应关系
        Map<ValueLabelVO, List<String>> secIndustryMap = this.getIndustrySecMap();
        // 日期分组
        Map<String, List<FundPositionDetail>> positionMap = positionDetails.stream().collect(Collectors.groupingBy(FundPositionDetail::getDate));
        positionMap.forEach((k, v) -> {
            // 股票市值
            BigDecimal total = v.stream().map(FundPositionDetail::getMarketValue).filter(Objects::nonNull)
                    .reduce(BigDecimal::add).map(BigDecimal::abs).orElse(null);
            List<IndexSecWeightInfoDO> indexSecWeightList = indexWeightMap.getOrDefault(k, ListUtil.empty());
            List<CategoryConstraint> constraintList = ListUtil.list(false);
            List<String> categoryCodes = ListUtil.list(false);
            secIndustryMap.forEach((c, l) -> {
                CategoryConstraint constraint = this.buildCategoryConstraint(v, total, categoryCodes, indexSecWeightList, l);
                constraint.setCategory(c);
                constraintList.add(constraint);
            });
            StockAllocationVO vo = this.getStockAllocationVO(k, v, total, indexSecWeightList, constraintList, categoryCodes);
            resultList.add(vo);
        });
        // 按日期升序
        resultList.sort(Comparator.comparing(StockAllocationVO::getDate));
        return resultList;
    }
}
