package com.smppw.analysis.domain.manager.position.stock;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import com.smppw.analysis.domain.dto.position.CategoryConstraint;
import com.smppw.analysis.domain.dto.position.stock.IndustryAllocationPreferenceVO;
import com.smppw.analysis.domain.dto.position.stock.StockAllocationParams;
import com.smppw.analysis.domain.dto.position.stock.StockAllocationVO;
import com.smppw.analysis.domain.manager.position.BizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerFactory;
import com.smppw.common.pojo.ValueLabelVO;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import static com.smppw.analysis.domain.manager.position.BizHandlerConstants.INDUSTRY_ALLOCATION;

/**
 * @author wangzaijun
 * @date 2023/7/14 17:36
 * @description 行业配置业绩表现
 */
@Component
public class IndustryAllocationPreferenceComponent {
    private final BizHandlerFactory factor;

    public IndustryAllocationPreferenceComponent(BizHandlerFactory factor) {
        this.factor = factor;
    }

    public List<IndustryAllocationPreferenceVO> bizHandle(StockAllocationParams params) {
        List<IndustryAllocationPreferenceVO> resultList = ListUtil.list(false);
        BizHandler<StockAllocationParams, List<StockAllocationVO>> bizHandler = this.factor.getBizHandlerInstance(INDUSTRY_ALLOCATION);
        List<StockAllocationVO> dataList = bizHandler.bizHandle(params);
        // 取所有行业
        List<ValueLabelVO> industryList = ListUtil.list(false);
        for (StockAllocationVO vo : dataList) {
            List<ValueLabelVO> collect = vo.getIndustries().stream().map(CategoryConstraint::getCategory).distinct().collect(Collectors.toList());
            CollUtil.addAllIfNotContains(industryList, collect);
        }
        boolean flag = true;
        // 每个日期数据处理
        for (StockAllocationVO temp : dataList) {
            List<BigDecimal> offsetList = ListUtil.list(false);
            Map<String, String> industryMap = MapUtil.newHashMap();
            // 处理所有行业的行业偏离
            for (ValueLabelVO industry : industryList) {
                CategoryConstraint categoryConstraint = temp.getIndustries().stream().filter(e -> industry.equals(e.getCategory())).findFirst().orElse(null);
                BigDecimal pupil = Optional.ofNullable(categoryConstraint).map(CategoryConstraint::getPupil).orElse(BigDecimal.ZERO);
                BigDecimal benchmark = Optional.ofNullable(categoryConstraint).map(CategoryConstraint::getBenchmark).orElse(BigDecimal.ZERO);
                BigDecimal offset = pupil.subtract(benchmark);
                industryMap.put(industry.getValue(), offset.toPlainString());
                offsetList.add(offset);
            }
            // 对应日期内按偏离度降序
            MapUtil.sortByValue(industryMap, true);
            // 求平均行业偏离
            double avg = offsetList.stream().mapToDouble(e -> Double.parseDouble(e.toPlainString())).average().orElse(0.0);
            IndustryAllocationPreferenceVO vo = new IndustryAllocationPreferenceVO();
            vo.setDate(temp.getDate());
            vo.setIndustryPreference(industryMap);
            vo.setAvg(String.valueOf(avg));
            // 第一个日期保存所有行业关系，方便取值
            if (flag) {
                vo.setIndustryList(industryList);
                flag = false;
            }
            resultList.add(vo);
        }
        // 按日期升序
        resultList.sort(Comparator.comparing(IndustryAllocationPreferenceVO::getDate));
        return resultList;
    }
}
