package com.smppw.analysis.domain.manager.position.synthesize;

import com.smppw.analysis.domain.dto.position.FundPositionBaseInfo;
import com.smppw.analysis.domain.dto.position.FundPositionDetail;
import com.smppw.analysis.domain.dto.position.synthesize.LeverageChangeParams;
import com.smppw.analysis.domain.dto.position.synthesize.LeverageChangeVO;
import com.smppw.analysis.domain.manager.position.AbstractBizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerConstants;
import com.smppw.analysis.domain.manager.position.PositionLoadFactory;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/8 8:39
 * @description 杠杆变化 接口逻辑处理
 */
@Component(BizHandlerConstants.LEVERAGE_CHANGE)
public class LeverageChangeBizHandler extends AbstractBizHandler<LeverageChangeParams, List<LeverageChangeVO>> {
    public LeverageChangeBizHandler(AnalysisProperty property, CacheFactory cacheFactory, PositionLoadFactory factory) {
        super(property, cacheFactory, factory);
    }

    @Override
    protected List<LeverageChangeVO> afterLoadData(LeverageChangeParams params, List<FundPositionBaseInfo> baseInfos, List<FundPositionDetail> positionInfos) {
        return baseInfos.stream().map(e -> {
            LeverageChangeVO vo = new LeverageChangeVO();
            vo.setDate(e.getDate());
            vo.setTotal(Optional.ofNullable(e.getTotalAsset()).map(BigDecimal::toPlainString).orElse(null));
            vo.setNv(Optional.ofNullable(e.getAssetNv()).map(BigDecimal::toPlainString).orElse(null));
            if (e.getAssetNv() != null && e.getTotalAsset() != null && e.getAssetNv().abs().compareTo(BigDecimal.ZERO) > 0) {
                vo.setLeverage(e.getTotalAsset().divide(e.getAssetNv(), 10, RoundingMode.HALF_UP).toPlainString());
            }
            return vo;
        }).collect(Collectors.toList());
    }
}
