package com.smppw.analysis.domain.manager.position.synthesize;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import com.smppw.analysis.domain.dto.position.AssetCategoryEnum;
import com.smppw.analysis.domain.dto.position.FundPositionBaseInfo;
import com.smppw.analysis.domain.dto.position.FundPositionDetail;
import com.smppw.analysis.domain.dto.position.synthesize.PositionInfoParams;
import com.smppw.analysis.domain.dto.position.synthesize.PositionInfoVO;
import com.smppw.analysis.domain.manager.position.AbstractBizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerConstants;
import com.smppw.analysis.domain.manager.position.PositionLoadFactory;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.constants.SecType;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/8 9:47
 * @description 其他特殊的综合组件
 */
@Service(BizHandlerConstants.POSITION_PARAMS)
public class PositionParamsBizHandler extends AbstractBizHandler<PositionInfoParams, PositionInfoVO> {
    public PositionParamsBizHandler(AnalysisProperty property, CacheFactory cacheFactory, PositionLoadFactory factory) {
        super(property, cacheFactory, factory);
    }

    @Override
    protected PositionInfoVO afterLoadData(PositionInfoParams params, List<FundPositionBaseInfo> baseInfos, List<FundPositionDetail> positionInfos) {
        if (CollUtil.isEmpty(positionInfos)) {
            return new PositionInfoVO();
        }
        String fundType = positionInfos.get(0).getFundType();
        List<ValueLabelVO> types = ListUtil.list(true);
        // 公募基金没有 全部 选项
        if (!SecType.PUBLICLY_OFFERED_FUNDS.equals(fundType)) {
            types.add(new ValueLabelVO("ALL", "全部"));
        }
        for (AssetCategoryEnum value : AssetCategoryEnum.values()) {
            types.add(new ValueLabelVO(value.name(), value.getName()));
        }
        List<String> dateList = positionInfos.stream().filter(e -> e.getAsset() != null).map(FundPositionDetail::getDate)
                .distinct().sorted(Comparator.reverseOrder()).collect(Collectors.toList());
        PositionInfoVO vo = new PositionInfoVO();
        vo.setDates(dateList);
        vo.setTypes(types);
        return vo;
    }
}
