package com.smppw.analysis.domain.mapper;

import com.smppw.analysis.domain.dataobject.*;
import com.smppw.common.pojo.ValueLabelVO;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @author wangzaijun
 * @date 2023/8/4 17:58
 * @description 股票、期货、债券关联的数据获取
 */
@Repository
public interface BaseUnderlyingMapper {
    /**
     * 获取所有股票对应行业
     *
     * @return /
     */
    List<SwSecIndustryInfoDO> querySecIndustryInfo();

    /**
     * 获取所有股票对应风格
     *
     * @param dateList 风格截止日集合
     * @return /
     */
    List<SecStyleInfoDO> querySecStyleInfo(List<String> dateList);

    /**
     * 获取当前日期之后最近的权重数据
     *
     * @param indexId 基准id
     * @param endDate 当前日期
     * @return /
     */
    List<IndexSecWeightInfoDO> queryIndexSecWeight(@Param("indexId") String indexId, @Param("endDate") String endDate);

    /**
     * 获取股票行情
     *
     * @param endDate 当前时间
     * @return /
     */
    List<SecClosePriceDO> listSecClosePrice(@Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 获取科创版股票行情
     *
     * @param endDate 当前时间
     * @return /
     */
    List<SecClosePriceDO> listSciTechSecClosePrice(@Param("startDate") String startDate, @Param("endDate") String endDate);

    List<SecClosePriceDO> listDateSciTechSecClosePrice(List<String> dateList);

    List<SecClosePriceDO> listDateSecClosePrice(List<String> dateList);

    /**
     * 流动性标准数据
     *
     * @param dateList /
     * @return /
     */
    List<SecLiquidityInfoDO> querySecLiquidityInfo(@Param("dateList") List<String> dateList);

    /**
     * 流动性 5日平均换手率
     *
     * @param dateList /
     * @return /
     */
    List<SecTurnoverInfoDO> querySecTurnoverInfo(@Param("dateList") List<String> dateList);

    /**
     * 获取商品期货区间内的收盘价序列
     *
     * @param secIds    期货标识
     * @param startDate 开始时间
     * @param endDate   结束时间
     * @return /
     */
    List<FutureDailyPriceDO> getProductSecClosePrice(@Param("secIds") List<String> secIds, @Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 新的获取金融期货（股指和国债）的区间内的收盘价涨跌幅
     *
     * @param secCodes      期货标识
     * @param valuationDate 开始日期，开区间（区别于商品期货的闭区间）
     * @param endDate       结束日期
     * @return /
     */
    List<FutureDailyPriceDO> getFinanceSecClosePrice(@Param("secCodes") List<String> secCodes, @Param("startDate") String valuationDate, @Param("endDate") String endDate);

    List<ValueLabelVO> getOptionTypeList();

    Map<String, String> getOptionTypeMap(@Param("tradingCode") String tradingCode);

    List<Map<String, String>> getSecondOptionTypeByCode(@Param("tradingCode") String tradingCode);

    List<Map<String, String>> fundFuturesOptionView();

    List<ValueLabelVO> getOptionTypeAndTradeCode(String firstOptionType);

    List<String> getSecondOptionType(String firstOptionType);
}