package com.smppw.analysis.domain.mapper;

import com.smppw.analysis.domain.dataobject.FundInformationDo;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository
public interface FundInformationDoMapper {

    List<String> listFundIdByFundIdListAndVisibilityId(@Param("fundIdList") List<String> fundIdList, @Param("visibilityId") int visibilityId);

    /**
     * 私募管理人:获取基金id
     *
     * @param fundIdList   基金ids
     * @param visibilityId 是否可见
     * @return /
     */
    List<String> listFundIdByFundIdsAndVisibilityIdForInvestAdvisor(@Param("fundIdList") List<String> fundIdList, @Param("visibilityId") int visibilityId);

    List<FundInformationDo> listFundInfo(@Param("fundIdList") List<String> fundIdList);


    List<Map<String, String>> getFundFrequency(@Param("fundIds") List<String> marketIndexList);

    /**
     * 获取基金经理的主要投资策略种类和为主的投资策略
     *
     * @param managerId 基金经理id
     * @return 基金经理的主要投资策略种类和为主的投资策略
     */
    List<Map<String, Object>> getManagerFundFeature(@Param("managerId") String managerId);

    /**
     * 获取市场基金的名字
     *
     * @param fundId 基金id
     * @return 市场基金的名字
     */
    FundInformationDo getFundByFundId(@Param("fundId") String fundId);

    List<Map<String, String>> getFundInfos(@Param("fundIdList") List<String> fundIdList);

    List<Map<String, String>> getFundRet(@Param("fundIdList") List<String> fundIdList);

    String getMaxFundEndDate(@Param("fundIdList") List<String> fundIdList);

    /**
     * 获取机构下的基金id
     *
     * @param companyId 机构id
     * @return 基金id
     */
    List<String> listFundIdByCompanyId(@Param("companyId") String companyId);

    /**
     * 获取基金经理或高管历史任职机构id
     *
     * @param personnelIdList 机构id
     * @return 基金经理或高管历史任职机构id
     */
    List<String> listCompanyIdByPersonnelId(@Param("personnelIdList") List<String> personnelIdList);

    List<Map<String, Object>> listFundInfoByCompanyId(String companyId);

    List<Map<String, Object>> listFundRegisterNumber(List<String> fundIdList);

    /**
     * 基金业绩排名
     *
     * @param params /
     * @return /
     */
    List<Map<String, Object>> getFundRankByCall(Map<String, Object> params);
}