package com.smppw.analysis.infrastructure.task;

import com.smppw.analysis.domain.mapper.IndexesTradeDateDoMapper;
import com.smppw.common.cache.CaffeineLocalCache;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;

@Component
public class NonTradingDayTask {

    private static final Logger logger = LoggerFactory.getLogger(NonTradingDayTask.class);

    @Autowired
    private IndexesTradeDateDoMapper indexesTradeDateDoMapper;

    /**
     * 将非交易日日期放入本地缓存，启动后十秒，及之后每24小时执行一次
     * 所有机器启动时均需加载，且执行间隔较长，不适宜用XxlJob进行管理
     */
    @Scheduled(initialDelay = 10000, fixedRate = 24 * 3600 * 1000)
    public void cacheNonTradingDays() {
        logger.info("start cacheNonTradingDays");
        List<String> allHolidayDays = indexesTradeDateDoMapper.listHolidays();

        CaffeineLocalCache.addNonTradingDay(allHolidayDays);
        logger.info("finish cacheNonTradingDays");
    }
}
